      subroutine backup(task,name)
 
c     Subroutine to make a backup copy of a file in its own directory.
c     If name does not include a dot ("."), the backup will be named
c     name.bak.  Otherwise, its extension will be replaced by ".bak".
c     If the back-up name already exists, it is overwritten.
c
c     Arguments:
c     task [in]  1 = preserve the original (copy the file)
c                2 = rename the file.  Error and no action if
c                    file is read-only.
c
c     In case of error, this routine does _nothing_.
c
c     BUG: not implemented for MS Fortran
c     BUG: Sun always copies, i.e., task is irrelevant
c
c     Constantijn Panis, 11 April 2002
c     Last change: check for read-only
 
      implicit none
      integer task,length,dot,trm
      character name*128
      integer readonly,rename
      integer unit1,unit2,unusedunit,code,myfputc
      integer fgetc
c     integer fgetc
      character c*1
      logical exist
 
      length=trm(name)
 
c     Make sure input file exists:
      if(.not.exist(name))goto 800
 
c     Determine output file name:
      call char1match('.',name,length,dot)
 
      if(task.eq.2)then
c        Rename.  First check whether it is read-only.
         unit1=unusedunit()
         open(unit=unit1,file=name,err=800)
         if(readonly(unit1).eq.1)goto 800
         close(unit1)
         if(dot.eq.0)then
c           No dot in file name
            if(rename(name(1:length),name(1:length)//'.bak').ne.0)
     *         goto 800
         else
c           File name has extension:
            if(rename(name(1:length),name(1:dot)//'bak').ne.0)goto 800
         endif
         goto 999
      endif
 
c     Open files to units that are not used yet:
      unit1=unusedunit()
      open(unit=unit1,file=name,err=999)
      unit2=unusedunit()
      if(dot.eq.0)then
         open(unit=unit2,file=name(1:length)//'.bak',err=800)
      else
         open(unit=unit2,file=name(1:dot)//'bak',err=800)
      endif
  100 code=fgetc(unit1,c)
      if(code.eq.-1)then
         close(unit1)
         close(unit2)
         goto 999
      elseif(code.eq.0)then
c        Put character out; if error, return
         if(myfputc(unit2,c).ne.0)goto 800
         goto 100
      else
c        Input error:
         goto 800
      endif
 
  800 goto 999
  999 return
      end
 
