      subroutine chkrange(nfull,iiffull,fulltype,nround,fullbeta,
     *   fullname,ipt,jpt,status)
 
c     Subroutine to check domain restrictions of parameters, and to
c     check that all parameters which contribute jointly to some
c     parameter's derivative are either jointly free or jointly fixed.
c     The latter is relevant for strictly increasing vectors
c     and parameters related to distributions which are searched upon
c     using the Cholesky decomposition.
c
c     Constantijn Panis, 29 August 2002
c     Last change: introduce status
 
      implicit none
      integer nfull,iiffull(nfull),fulltype(nfull),nround,ipt,jpt,
     *   status,dim,round,type,i,j,intcontig,trm
      double precision fullbeta(nfull),beta
      character fullname(nfull)*8,name*8
 
      status=0
 
      i=0
  100 i=i+1
      if(i.gt.nfull)goto 999
      type=fulltype(i)
      beta=fullbeta(i)
      name=fullname(i)
c     Check domain restrictions, if any:
      if(type.eq.0)goto 100
      if(type.eq.1.and.beta.gt.0.d0)goto 100
      if(type.eq.2.and.abs(beta).lt.1.d0)goto 100
      if(type.eq.3.and.abs(beta-.5d0).lt..5d0)goto 100
      if(type.eq.41.or.type.eq.4)then
c        Member of a strictly increasing series.  Its derivative depends
c        on the derivatives wrt all subsequent parameters of this series.
c        E.g., FFT corresponds to binary 100; FTT to 110; et cetera
         do 110 round=1,nround
            if(mod(iiffull(i)/2**(round-1),2).ne.1)goto 110
            do 120 j=i+1,nfull
               if(fulltype(j).ne.4)goto 110
c              This parameter must also be estimated:
               if(mod(iiffull(j)/2**(round-1),2).ne.1)goto 800
  120       continue
  110    continue
c        Must be strictly larger than predecessor:
         if(type.eq.4.and.beta.gt.fullbeta(i-1))goto 100
         if(type.eq.41)goto 100
      endif
      if(type.eq.5)then
c        Cholesky search; first dim elements have fulltype=5.
         dim=intcontig(5,fulltype(i),nfull-i+1)
c        Make sure that all dim*(dim+1)/2 parameters are either
c        all estimated or all fixed, in every round.
         do 140 j=i+1,i+(dim*(dim+1))/2-1
            if(iiffull(j).ne.iiffull(i))goto 810
  140    continue
c        The first dim parameters are standard deviations; the next
c        dim*(dim-1)/2 are correlations.
         do 150 j=1,dim
            type=1
            if(fullbeta(i).le.0.d0)goto 820
            i=i+1
  150    continue
         do 160 j=1,(dim*(dim-1))/2
            type=2
            if(abs(fullbeta(i)).ge.1.d0)goto 820
            i=i+1
  160    continue
         goto 100
      endif
      goto 820
 
 
  800 if(ipt.gt.0)write(8,801)name(1:trm(name))
      if(jpt.gt.0)write(6,801)name(1:trm(name))
  801 format(/,'ERROR:  parameter ''',a,''' is part of a strictly ',
     *   'increasing vector.',/,'If it is estimated, all subsequent ',
     *   'parameters of that same set must',/,'also be estimated.')
      goto 899
  810 if(ipt.gt.0)write(8,811)name(1:trm(name)),
     *   fullname(j)(1:trm(fullname(j)))
      if(jpt.gt.0)write(6,811)name(1:trm(name)),
     *   fullname(j)(1:trm(fullname(j)))
  811 format(/,'ERROR:  all parameters related to a distribution ',
     *   'which is estimated',/,'using a Cholesky decomposition must ',
     *   'either all be estimated or all',/,
     *   'be fixed.  This is not the case for ',a,' and ',a,'.')
      goto 899
  820 name=fullname(i)
      beta=fullbeta(i)
      if(ipt.gt.0)then
c        write(8,821)name(1:trm(name))
         write(8,821)name(1:trm(name))
cD        write(8,821,advance='no')name(1:trm(name))
         call writedble(8,beta,1,-10,0,0)
         if(type.eq.1)write(8,822)'strictly positive'
         if(type.eq.2)write(8,822)'between -1 and 1'
         if(type.eq.3)write(8,822)'between 0 and 1'
         if(type.eq.4)write(8,822)'larger than '//
     *      fullname(i-1)(1:trm(fullname(i-1)))
      endif
      if(jpt.gt.0)then
c        write(6,821)name(1:trm(name))
         write(6,821)name(1:trm(name))
cD        write(6,821,advance='no')name(1:trm(name))
         call writedble(6,beta,1,-10,0,0)
         if(type.eq.1)write(6,822)'strictly positive'
         if(type.eq.2)write(6,822)'between -1 and 1'
         if(type.eq.3)write(6,822)'between 0 and 1'
         if(type.eq.4)write(6,822)'larger than '//
     *      fullname(i-1)(1:trm(fullname(i-1)))
      endif
c 821 format(/,'ERROR:  parameter ''',a,''' is initialized at',$)
  821 format(/,'ERROR:  parameter ''',a,''' is initialized at',$)
cD 821 format(/,'ERROR:  parameter ''',a,''' is initialized at')
  822 format(';',/,8x,'it must be ',a,'.')
      goto 899
  899 status=1
      goto 999
  999 return
      end
