      subroutine compsig44(task,numdist,sumbigintdim,
     *   sumdim2,nested,distyp,distdim,ndraw,nel,nep,nec,lamtyp,lampos,
     *   bdc,ifint4,ifint6,allint6,dimii,dimlam,lam,dlam,sigdd,dsigdd,
     *   sigii,dsigii,sigl4l6,dsigl4l6,stda4,siga4a4,dsiga4a4,
     *   sigil6,dsigil6,cmui,dcmui,dxblp,siga6a6i,dsiga6a6,siga6a6ia6,
     *   da6,cmua4,sigl4l6a6a6i,iflam,iid4,ibnnid4,ipt,jpt,status)
 
c     Residuals in probit models may be correlated with residuals in
c     continuous models.  This routine computes the distribution of
c     probit residuals, conditional on realized continuous outcomes.
c     If the continuous likelihood has not been computed yet, a6=Y-BX
c     is not known yet, and the conditional mean cmua4 cannot be
c     determined yet.  In that case, onepoint computes cmua4 and
c     sumlnl2l unravels it.
c
c     This routine also computes the integrated distribution's
c     conditional mean and covariance matrix, if applicable.
c
c     Note: straight application of unraveling formulas will make
c     matrices of derivatives wrt covariances asymmetric, under
c     certain circumstances.  Since the number of degrees of freedom
c     of such matrices is really only n*(n+1)/2, this routine
c     always restores symmetry after potentially destroying it.
c
c     Naming conventions:
c        d  = residual, as Defined
c        e  = residual, Expanded
c        l4 = probit residual, Loaded (by-passed in this routine)
c        a4 = probit residual, Accumulated (over distributions)
c        l6 = continuous residual, Loaded (by-passed in this routine)
c        a6 = continuous residual, Accumulated (over distributions)
c        i  = Integrated residual
c
c     Throughout aML, there are six covariance matrices and seven load matrices,
c     hence lamtyp(numdist,7), lampos(numdist,7).  The seven are:
c        1 = hazard models
c        2 = logit models
c        3 = probit models
c        4 = continuous models
c        5 = binomial models
c        6 = negative binomial models
c        7 = multinomial logit models
c     In this routine, only probit and continuous models are relevant.
c
c     Constantijn Panis, 17 October 2002
 
      implicit none
      integer task,numdist,sumbigintdim,sumdim2,nested(numdist),
     *   distyp(numdist),distdim(numdist),ndraw(numdist),nel,nep,nec,
     *   lamtyp(numdist,7),lampos(numdist,7),bdc(nep),ifint4(numdist),
     *   ifint6(numdist),allint6,
     *   dimii,dimlam,ipt,jpt,status,posdd,posii,posl4l6,
     *   dim,bigdim,pos,i,getbigposi1,countint
      double precision lam(dimlam),dlam(dimlam),
     *   sigdd(sumdim2),dsigdd(sumdim2),sigii(dimii),dsigii(dimii),
     *   sigl4l6(numdist*nep*nec),dsigl4l6(numdist*nep*nec),
     *   stda4(nep),siga4a4(nep,nep),dsiga4a4(nep,nep),
     *   sigil6(sumbigintdim*nec),dsigil6(sumbigintdim*nec),
     *   cmui(sumbigintdim),dcmui(sumbigintdim),dxblp(nel+nep),
     *   siga6a6i(nec,nec),dsiga6a6(nec,nec),
     *   siga6a6ia6(nec),da6(nec),cmua4(nep),sigl4l6a6a6i(*)
c     Scratch array sigl4l6a6a6i is (nep,nec) for non-integrated
c     distributions and (sumbigintdim,nec) for the integrated distribution.
      logical iflam(numdist),iid4,ibnnid4,diagonal
 
      status=0
      posdd=1
      posii=1
      posl4l6=1
      if(task.eq.2)goto 200
 
c     Compute conditional distribution:
c     mu = Sl4l6*Sa6a6i*a6
c        = L4*See*L6'*Sa6a6i*a6
c     Sigma44 = Sl4l4 - Sl4l6*Sa6a6i*Sl4l6'
c             = L4*See*L4' - L4*See*L6'*Sa6a6i*L6*See*L4'
 
  100 ibnnid4=.true.
      do 110 i=1,numdist
         dim=distdim(i)
         bigdim=dim*ndraw(i)
         if(bigdim.eq.0)goto 120
c        Finite mixture distribution:  there is no covariance matrix
         if(mod(distyp(i),10).eq.4)goto 120
         if(nested(i).gt.0)then
c           This distribution is integrated out in some model.
c           Error if it enters as non-integrated residual in a probit model:
            call blowup(1,dim,ndraw(i),sigdd(posdd),sigii(posii))
            if(allint6.eq.-1)then
c              It may also enter continuous models, but is not integrated
c              out there.  The integrated distribution therefore needs
c              to be conditioned on continuous outcomes.
c              Conditional mean = Sil6*Sa6a6i*a6   where Sil6 = Sie*L6'
               pos=getbigposi1(i,nested,ndraw,distdim)-1
c              Careful: continuous modules often denoted by "6", but
c              in lamtyp/lampos, continuous is at "4".
               if(lamtyp(i,4).eq.0)then
                  call m2mult300(ndraw(i),dim,nec,sigdd(posdd),
     *               lam(lampos(i,4)+1),sigil6(pos*nec+1),0)
               elseif(lamtyp(i,4).eq.1)then
                  call zerodp(bigdim*nec,sigil6(pos*nec+1))
                  call blowup(1,dim,ndraw(i),sigdd(posdd),
     *               sigil6(pos*nec+1))
               elseif(lamtyp(i,4).eq.4)then
                  call m2mult340(ndraw(i),dim,nec,sigdd(posdd),
     *               lam(lampos(i,4)+1),sigil6(pos*nec+1),0)
               endif
               call m2mult(000,bigdim,nec,1,sigil6(pos*nec+1),
     *            siga6a6ia6,cmui(pos+1),0)
c              Conditional covariance = Sii- Sil6*Sa6a6i*Sl6i
               call quadratic(0,bigdim,nec,sigil6(pos*nec+1),siga6a6i,
     *            sigii(posii),-1)
c              sigii should be block-diagonal (but not with identical blocks).
            endif
         endif
         if(ifint4(i).eq.-1.and.nep.gt.0)then
c           This distribution enters probit models and is not
c           integrated out.  Compute and accumulate the unconditional
c           covariance matrix:
c           Careful: probit modules often denoted by "4", but
c           in lamtyp/lampos, probit is at "3".
            if(lamtyp(i,3).eq.0)then
               call quadratic032(0,nep,ndraw(i),dim,lam(lampos(i,3)+1),
     *            sigdd(posdd),siga4a4,1)
            elseif(lamtyp(i,3).eq.1)then
c              Accumulate sigdd onto ndraw blocks along siga4a4's diagonal.
               call blowup(1,dim,ndraw(i),sigdd(posdd),siga4a4)
            elseif(lamtyp(i,3).eq.4)then
               call m3mult4340(nep,ndraw(i),dim,nep,lam(lampos(i,3)+1),
     *            sigdd(posdd),lam(lampos(i,3)+1),siga4a4,1)
            endif
cc           Is the covariance structure diagonal?
c            if(ibnnid4)call diagcovar(nep,dim,lamtyp(i,3),lampos(i,3),
c     *         ndraw(i),dimlam,lam,ibnnid4)
c           Determine block-diagonality
            call blockdiagcovar(nep,bdc,dim,lamtyp(i,3),lampos(i,3),
     *         ndraw(i),dimlam,lam,ibnnid4,status)
            if(status.ne.0)goto 800
 
c           If the distribution also enters continuous models,
c           the distribution is conditional on continuous outcomes.
c           The conditional covariance matrix may always be computed
c           here.  The conditional mean depends on continuous outcomes
c           a6, which are only available here if no distribution is
c           integrated out of continuous models.  (If any residual is
c           integrated out of continuous models, this subroutine will
c           be called again for each draw in subroutine integrate,
c           with task=3.)
            if(nec.gt.0.and.ifint6(i).ne.0)then
c              Premultiply draw-expanded Sigee by Lambda4 and postmultiply by La
c              to get covariance between loaded dimensions: Sl4l6 = L4*See*L6'
c              Lambdas unlikely to be identity, as continuous and probit
c              models probably have correlated, not same residuals.
c              Careful: probit and continuous modules often denoted by "4" and "
c              respectively, but in lamtyp/lampos, probit and continuous are at
c              and "4", respectively.
               if(lamtyp(i,3).eq.0)then
                  if(lamtyp(i,4).eq.0)then
                     call m3mult0300(0010,nep,ndraw(i),dim,nec,
     *                  lam(lampos(i,3)+1),sigdd(posdd),
     *                  lam(lampos(i,4)+1),sigl4l6(posl4l6),0)
                  elseif(lamtyp(i,4).eq.1)then
                     call m2mult030(000,nep,ndraw(i),dim,
     *                  lam(lampos(i,3)+1),sigdd(posdd),
     *                  sigl4l6(posl4l6),0)
                  elseif(lamtyp(i,4).eq.4)then
                     call m3mult0340(nep,ndraw(i),dim,nec,
     *                  lam(lampos(i,3)+1),sigdd(posdd),
     *                  lam(lampos(i,4)+1),sigl4l6(posl4l6),0)
                  endif
               elseif(lamtyp(i,3).eq.1)then
                  if(lamtyp(i,4).eq.0)then
                     call m2mult300(ndraw(i),dim,nec,sigdd(posdd),
     *                  lam(lampos(i,4)+1),sigl4l6(posl4l6),0)
                  elseif(lamtyp(i,4).eq.1)then
                     call blowup(1,dim,ndraw(i),sigdd(posdd),
     *                 sigl4l6(posl4l6))
                  elseif(lamtyp(i,4).eq.4)then
                     call m2mult340(ndraw(i),dim,nec,sigdd(posdd),
     *                  lam(lampos(i,4)+1),sigl4l6(posl4l6),0)
                  endif
               elseif(lamtyp(i,3).eq.4)then
                  if(lamtyp(i,4).eq.0)then
                     call m3mult4300(0010,nep,ndraw(i),dim,nec,
     *                  lam(lampos(i,3)+1),sigdd(posdd),
     *                  lam(lampos(i,4)+1),sigl4l6(posl4l6),0)
                  elseif(lamtyp(i,4).eq.1)then
                     call m2mult430(000,nep,ndraw(i),dim,
     *                  lam(lampos(i,3)+1),sigdd(posdd),
     *                  sigl4l6(posl4l6),0)
                  elseif(lamtyp(i,4).eq.4)then
                     call m3mult4340(nep,ndraw(i),dim,nec,
     *                  lam(lampos(i,3)+1),sigdd(posdd),
     *                  lam(lampos(i,4)+1),sigl4l6(posl4l6),0)
                  endif
               endif
c              Conditional covariance correction:
               call quadratic(0,nep,nec,sigl4l6(posl4l6),siga6a6i,
     *            siga4a4,-1)
c              The conditional mean depends on continuous outcomes
c              a6, which are only available here if no distribution is
c              integrated out of continuous models.
               if(countint(1,ifint6,numdist).eq.0)call m2mult(000,nep,
     *            nec,1,sigl4l6(posl4l6),siga6a6ia6,cmua4,1)
c              If there were other, higher-level residuals in the continuous
c              models, siga6a6i was not diagonal and the conditional siga4a4
c              may no longer be diagonal.  See Test\Select\test3a
               if(.not.diagonal(siga4a4,nep))ibnnid4=.false.
            endif
         endif
  120    posl4l6=posl4l6+nep*nec
         posdd=posdd+dim*dim
         if(nested(i).gt.0)posii=posii+bigdim*bigdim
  110 continue
c     Convert the overall covariance matrix into a correlation matrix
c     with standard errors on the diagonal.  Only bi- and trivariate
c     dependent probits are currently allowed.
      if(nep.eq.0)then
c        Nothing needs to be done
         continue
      elseif(iid4)then
c        Probit residuals are all (implicitly) IID N(0,1)
         if(.not.ibnnid4)call bug(ipt,jpt)
         call putrow(1.d0,siga4a4,nep,nep+1)
      elseif(ibnnid4)then
c        Residuals are Independently But Not Necessarily Identically Distributed
         if(.not.diagonal(siga4a4,nep))call bug(ipt,jpt)
         do 160 i=1,nep
            if(stda4(i).eq.1.d0)then
               siga4a4(i,i)=1.d0
            else
               siga4a4(i,i)=dsqrt(siga4a4(i,i))
            endif
  160    continue
      else
c        Non-integrated probit residuals are correlated
         do 170 i=1,nep
            if(stda4(i).eq.1.d0)siga4a4(i,i)=1.d0
  170    continue
         call cov2corr(1,nep,siga4a4,siga4a4,0.d0,0.d0,status)
         if(status.ne.0)then
            if(status.lt.0)goto 810
            if(status.gt.0)call bug(ipt,jpt)
         endif
      endif
      goto 999
 
c     Unravel derivatives.
c     First unravel wrt standard deviations and correlations into wrt
c     covariance matrix.
  200 if(iid4.or.nep.eq.0)then
c        No need to compute dsiga4a4
         continue
      elseif(ibnnid4)then
c        Independent probits:
         do 210 i=1,nep
            dsiga4a4(i,i)=dsiga4a4(i,i)/(2.d0*siga4a4(i,i))
  210    continue
      else
c        Correlated probits:
         call cov2corr(2,nep,siga4a4,siga4a4,dsiga4a4,dsiga4a4,status)
         if(status.ne.0)then
            if(status.lt.0)goto 810
            if(status.gt.0)call bug(ipt,jpt)
         endif
      endif
c     Siga4a4 is a correlation matrix with standard deviations
c     on the diagonal.  It is not used below.
 
      do 220 i=1,numdist
         dim=distdim(i)
         bigdim=dim*ndraw(i)
         if(bigdim.eq.0)goto 230
c        Finite mixture distribution:  there is no covariance matrix
         if(mod(distyp(i),10).eq.4)goto 230
         if(nested(i).gt.0)then
            if(allint6.eq.-1)then
c              First compute scratch array Sil6*Sa6a6i
               pos=getbigposi1(i,nested,ndraw,distdim)-1
               call m2mult(000,bigdim,nec,nec,sigil6(pos*nec+1),
     *            siga6a6i,sigl4l6a6a6i,0)
c              Unravel wrt conditional mean into wrt Sil6:
               call m2mult(010,bigdim,1,nec,dcmui(pos+1),siga6a6ia6,
     *            dsigil6(pos*nec+1),0)
c              Unravel wrt conditioning variance correction into wrt Sil6:
               call m2mult(000,bigdim,bigdim,nec,dsigii(posii),
     *            sigl4l6a6a6i,dsigil6(pos*nec+1),-2)
c              Unravel wrt Sil6 into wrt Sie,  dSie = dSil6*L6:
               if(lamtyp(i,4).eq.0)then
                  call m2mult003(000,bigdim,nec,ndraw(i),dim,
     *               dsigil6(pos*nec+1),lam(lampos(i,4)+1),
     *               dsigdd(posdd),1)
               elseif(lamtyp(i,4).eq.1)then
                  call blowup(2,dim,ndraw(i),dsigdd(posdd),
     *               dsigil6(pos*nec+1))
               elseif(lamtyp(i,4).eq.4)then
                  call m2mult043(000,bigdim,nec,ndraw(i),dim,
     *               dsigil6(pos*nec+1),lam(lampos(i,4)+1),
     *               dsigdd(posdd),1)
               endif
               if(dim.gt.1)call makesym(dim,dsigdd(posdd))
 
               if(iflam(i))then
c                 Unravel wrt Sil6 into wrt L6,   dL6 = dSil6'*Sii
                  if(lamtyp(i,4).eq.0)then
                     call m2mult030(100,nec,ndraw(i),dim,
     *                  dsigil6(pos*nec+1),sigdd(posdd),
     *                  dlam(lampos(i,4)+1),1)
                  elseif(lamtyp(i,4).eq.1)then
c                    If a lambda is estimated, its type will not be identity.
c                    However, iflam is based on all model types, lamtyp is
c                    specific to model types.  See, for example,
c                    Trouble\trouble0a, where a residual is loaded in a hazard
c                    model but not continuous model.
                     continue
                  elseif(lamtyp(i,4).eq.4)then
                     call copy8(lam(lampos(i,4)+1),dlam(lampos(i,4)+1),
     *                  nec)
                     call m2mult034(100,nec,ndraw(i),dim,
     *                  dsigil6(pos*nec+1),sigdd(posdd),
     *                  dlam(lampos(i,4)+1),1)
                  endif
               endif
c              Unravel wrt conditioning correction of covariance matrix
c              into wrt Sa6a6.
               call quadratic(1,nec,bigdim,sigl4l6a6a6i,dsigii(posii),
     *            dsiga6a6,1)
c              Unravel wrt conditional mean into wrt Sa6a6.  This will
c              ruin symmetry of dSa6a6, but special care is subsequently
c              taken to guarantee that dSa6a6 remains symmetric.
               call m3mult(1010,nec,bigdim,1,nec,sigl4l6a6a6i,
     *            dcmui(pos+1),siga6a6ia6,dsiga6a6,-1)
               if(nec.gt.1)call makesym(nec,dsiga6a6)
c              Unravel wrt conditional mean into wrt a6:
               call m2mult(100,nec,bigdim,1,sigl4l6a6a6i,dcmui(pos+1),
     *            da6,1)
            endif
c           Accumulate derivatives wrt the covariance matrix of the integrated
c           distribution back in:
            call blowup(2,dim,ndraw(i),dsigdd(posdd),dsigii(posii))
         endif
         if(ifint4(i).eq.-1.and.nep.gt.0)then
c           Unravel wrt S44 into wrt L4 and See.
c           Note that if S44 = L4*See*L4' (- correction),
c              dL4 = 2*dS44*L4*See  (provided See and dS44 are symmetric)
c              dSee = L4'*dS44*L4
c           Note: special care was taken to guarantee symmetry of dS44.
            if(iflam(i))then
               if(lamtyp(i,3).eq.0)then
c                 Generic Lambda.  Symmetry of dsiga4a4 could be exploited,
c                 not done here.
                  call m3mult0030(0,nep,nep,ndraw(i),dim,dsiga4a4,
     *               lam(lampos(i,3)+1),sigdd(posdd),
     *               dlam(lampos(i,3)+1),2)
               elseif(lamtyp(i,3).eq.1)then
c                 Lambda is identity.  Cannot occur when Lambda is estimated.
c                 However, iflam(i) may be true due to another model type.
                  continue
               elseif(lamtyp(i,3).eq.4)then
c                 Lambda is sparse.  Symmetry of dsiga4a4 not exploited.
                  call m3mult0434(0,nep,nep,ndraw(i),dim,dsiga4a4,
     *               lam(lampos(i,3)+1),sigdd(posdd),
     *               dlam(lampos(i,3)+1),2)
               endif
            endif
c           Unravel dS44 wrt dSee:   dSee = L4'*dS44*L4
            if(lamtyp(i,3).eq.0)then
c              Lambda is generic
               call quadratic023(1,bigdim,nep,dim,lam(lampos(i,3)+1),
     *            dsiga4a4,dsigdd(posdd),1)
            elseif(lamtyp(i,3).eq.1)then
c              Lambda is identity
               call blowup(2,dim,ndraw(i),dsigdd(posdd),dsiga4a4)
            elseif(lamtyp(i,3).eq.4)then
c              Lambda is sparse
               call quadratic423(nep,ndraw(i),dim,lam(lampos(i,3)+1),
     *            dsiga4a4,dsigdd(posdd),1)
            endif
            if(nec.gt.0.and.ifint6(i).ne.0)then
c              Unravel wrt S44 into wrt Sl4l6 and Sa6a6.
c              Note that if C = -Sl4l6*Sa6a6i*Sl4l6',
c                 dSl4l6 = -2*dC*Sl4l6*Sa6a6i (provided dC and Sa6a6i are symmet
c                 dSa6a6 = Sa6a6i*Sl4l6'*dC*Sl4l6*Sa6a6i
c                      = (Sl4l6*Sa6a6i)'*dC*(Sl4l6*Sa6a6i)
c              First compute scratch array Sl4l6*Sa6a6i
               call m2mult(000,nep,nec,nec,sigl4l6(posl4l6),
     *            siga6a6i,sigl4l6a6a6i,0)
               call m2mult(000,nep,nep,nec,dsiga4a4,sigl4l6a6a6i,
     *            dsigl4l6(posl4l6),-2)
c              Note that special care was taken to guarantee that dSa6a6
c              is symmetric, so that subroutine quadratic may be used.
               call quadratic(1,nec,nep,sigl4l6a6a6i,dsiga4a4,
     *            dsiga6a6,1)
c              Unravel wrt the conditional mean of probit residuals (cmua4)
c              into wrt Sl4l6, Sa6a6 and a6.  This involves Sa6a6i*a6 and
c              is thus dependent on a6.  If the integrated distribution
c              is integrated out of continuous models, a6=Y-XB-i6,
c              i.e., Sa6a6i*a6 varies by integration group.  In that case,
c              there is no unraveling wrt Sl4l6 and Sa6a6.  We always unravel
c              wrt a6, since this does not involve a6.
c              Note that mu = Sl4l6*Sa6a6i*a6, so
c                 dSl4l6 = dmu*(Sa6a6i*a6)'
c                 dSa6a6 = -Sa6a6i*Sl4l6'*dmu*a6'*Sa6a6i
c                      = -(Sl4l6*Sa6a6i)'*dmu*(Sa6a6i*a6)'
c                 da6 = (Sl4l6*Sa6a6i)'*dmu
c              The derivative wrt conditional mean cmua4 is equal to wrt
c              the aggregate regressor sets dxbp.
               if(countint(1,ifint6,numdist).eq.0)then
                  call m2mult(010,nep,1,nec,dxblp(nel+1),siga6a6ia6,
     *               dsigl4l6(posl4l6),1)
c                 The following ruins symmetry of dSa6a6, so restore it:
                  call m3mult(1010,nec,nep,1,nec,sigl4l6a6a6i,
     *               dxblp(nel+1),siga6a6ia6,dsiga6a6,-1)
                  if(nec.gt.1)call makesym(nec,dsiga6a6)
               endif
               call m2mult(100,nec,nep,1,sigl4l6a6a6i,dxblp(nel+1),
     *            da6,1)
c              Unravel wrt sigl4l6 into wrt Lambda4, sigee and Lambda6.
c              Note that if Sl4l6 = L4*See*L6',
c                 dL4 = dSl4l6*L6*See'
c                 dSee = L4'*dSl4l6*L6
c                 dL6 = dSl4l6'*L4*See
               if(iflam(i))then
c                 dL4 = dSl4l6*L6*See'
                  if(lamtyp(i,3).eq.0)then
                     if(lamtyp(i,4).eq.0)then
c                       Both Lambdas generic:
                        call m3mult0030(0010,nep,nec,ndraw(i),dim,
     *                     dsigl4l6(posl4l6),lam(lampos(i,4)+1),
     *                     sigdd(posdd),dlam(lampos(i,3)+1),1)
                     elseif(lamtyp(i,4).eq.1)then
c                       Lambda4 generic, Lambda6 identity
c                       dL4 = dSl4l6*L6*See' = dSl4l6*See'
                        call m2mult030(010,nep,ndraw(i),dim,
     *                     dsigl4l6(posl4l6),sigdd(posdd),
     *                     dlam(lampos(i,3)+1),1)
                     elseif(lamtyp(i,4).eq.4)then
c                       Lambda4 generic, Lambda6 sparse
                        call m3mult0430(0000,nep,nec,ndraw(i),dim,
     *                     dsigl4l6(posl4l6),lam(lampos(i,4)+1),
     *                     sigdd(posdd),dlam(lampos(i,3)+1),1)
                     endif
                  elseif(lamtyp(i,3).eq.1)then
c                    Lambda4 is identity.  Cannot occur when Lambda4 is estimate
c                    However, iflam(i) may be true due to another model type.
                     continue
                  elseif(lamtyp(i,3).eq.4)then
                     if(lamtyp(i,4).eq.0)then
c                       Lambda4 is sparse, Lambda6 generic
                        call m3mult0034(0010,nep,nec,ndraw(i),dim,
     *                     dsigl4l6(posl4l6),lam(lampos(i,4)+1),
     *                     sigdd(posdd),dlam(lampos(i,3)+1),1)
                     elseif(lamtyp(i,4).eq.1)then
c                       Lambda4 is sparse, Lambda6 identity
c                       dL4 = dSl4l6*L6*See' = dSl4l6*See'
                        call m2mult034(010,nep,ndraw(i),dim,
     *                     dsigl4l6(posl4l6),sigdd(posdd),
     *                     dlam(lampos(i,3)+1),1)
                     elseif(lamtyp(i,4).eq.4)then
c                       Lambda4 and Lambda6 are both sparse (but not
c                       necessarily with the same structure).
                        call m3mult0434(0010,nep,nec,ndraw(i),dim,
     *                     dsigl4l6(posl4l6),lam(lampos(i,4)+1),
     *                     sigdd(posdd),dlam(lampos(i,3)+1),1)
                     endif
                  endif
c                 dL6 = dSl4l6'*L4*See
                  if(lamtyp(i,4).eq.1)then
c                    Lambda6 identity.  Cannot occur when Lambda6 is estimated.
c                    However, iflam(i) may be true due to another model type.
                     continue
                  elseif(lamtyp(i,3).eq.0)then
                     if(lamtyp(i,4).eq.0)then
c                       Both Lambdas generic:
                        call m3mult0030(1000,nec,nep,ndraw(i),dim,
     *                     dsigl4l6(posl4l6),lam(lampos(i,3)+1),
     *                     sigdd(posdd),dlam(lampos(i,4)+1),1)
                     elseif(lamtyp(i,4).eq.4)then
c                       Lambda4 generic, Lambda6 sparse
                        call m3mult0034(1000,nec,nep,ndraw(i),dim,
     *                     dsigl4l6(posl4l6),lam(lampos(i,3)+1),
     *                     sigdd(posdd),dlam(lampos(i,4)+1),1)
                     endif
                  elseif(lamtyp(i,3).eq.1)then
c                    Lambda4 is identity.  dL6 = dSl4l6'*L4*See = dSl4l6'*See
                     if(lamtyp(i,4).eq.0)then
c                       Lambda6 generic:
                        call m2mult030(100,nec,ndraw(i),dim,
     *                     dsigl4l6(posl4l6),sigdd(posdd),
     *                     dlam(lampos(i,4)+1),1)
                     elseif(lamtyp(i,4).eq.4)then
c                       Lambda6 sparse
                        call m2mult034(100,nec,ndraw(i),dim,
     *                     dsigl4l6(posl4l6),sigdd(posdd),
     *                     dlam(lampos(i,4)+1),1)
                     endif
                  elseif(lamtyp(i,3).eq.4)then
                     if(lamtyp(i,4).eq.0)then
c                       Lambda4 is sparse, Lambda6 generic
                        call m3mult0430(1000,nec,nep,ndraw(i),dim,
     *                     dsigl4l6(posl4l6),lam(lampos(i,3)+1),
     *                     sigdd(posdd),dlam(lampos(i,4)+1),1)
                     elseif(lamtyp(i,4).eq.4)then
c                       Lambda4 and Lambda6 are both sparse (but not
c                       necessarily with the same structure).
                        call m3mult0434(1000,nec,nep,ndraw(i),dim,
     *                     dsigl4l6(posl4l6),lam(lampos(i,3)+1),
     *                     sigdd(posdd),dlam(lampos(i,4)+1),1)
                     endif
                  endif
               endif
c              dSee = L4'*dSl4l6*L6
               if(lamtyp(i,3).eq.0)then
                  if(lamtyp(i,4).eq.0)then
c                    Lambda4 is generic, Lambda6 generic:
                     call m3mult0003(1000,bigdim,nep,nec,ndraw(i),dim,
     *                  lam(lampos(i,3)+1),dsigl4l6(posl4l6),
     *                  lam(lampos(i,4)+1),dsigdd(posdd),1)
                  elseif(lamtyp(i,4).eq.1)then
c                    Lambda4 is generic, Lambda6 identity: dSee = L4'*dSl4l6*L6
                     call m2mult003(100,bigdim,nep,ndraw(i),dim,
     *                  lam(lampos(i,3)+1),dsigl4l6(posl4l6),
     *                  dsigdd(posdd),1)
                  elseif(lamtyp(i,4).eq.4)then
c                    Lambda4 is generic, Lambda6 sparse:
                     call m3mult0043(1000,bigdim,nep,nec,ndraw(i),dim,
     *                  lam(lampos(i,3)+1),dsigl4l6(posl4l6),
     *                  lam(lampos(i,4)+1),dsigdd(posdd),1)
                  endif
               elseif(lamtyp(i,3).eq.1)then
                  if(lamtyp(i,4).eq.0)then
c                    Lambda4 is identity, Lambda6 generic: dSee = L4'*dSl4l6*L6
                     call m2mult003(000,bigdim,nec,ndraw(i),dim,
     *                  dsigl4l6(posl4l6),lam(lampos(i,4)+1),
     *                  dsigdd(posdd),1)
                  elseif(lamtyp(i,4).eq.1)then
c                    Lambda4 is identity, Lambda6 identity: dSee = L4'*dSl4l6*L6
                     call blowup(2,dim,ndraw(i),dsigdd(posdd),
     *                 dsigl4l6(posl4l6))
                  elseif(lamtyp(i,4).eq.4)then
c                    Lambda4 is identity, Lambda6 sparse: dSee = L4'*dSl4l6*L6 =
                     call m2mult043(000,bigdim,nec,ndraw(i),dim,
     *                  dsigl4l6(posl4l6),lam(lampos(i,4)+1),
     *                  dsigdd(posdd),1)
                  endif
               elseif(lamtyp(i,3).eq.4)then
                  if(lamtyp(i,4).eq.0)then
c                    Lambda4 is sparse, Lambda6 generic:
                     call m3mult4003(1000,bigdim,nep,nec,ndraw(i),dim,
     *                  lam(lampos(i,3)+1),dsigl4l6(posl4l6),
     *                  lam(lampos(i,4)+1),dsigdd(posdd),1)
                  elseif(lamtyp(i,4).eq.1)then
c                    Lambda4 is sparse, Lambda6 identity: dSee = L4'*dSl4l6*L6 =
                     call m2mult403(100,bigdim,nep,ndraw(i),dim,
     *                  lam(lampos(i,3)+1),dsigl4l6(posl4l6),
     *                  dsigdd(posdd),1)
                  elseif(lamtyp(i,4).eq.4)then
c                    Lambda4 is sparse, Lambda6 sparse:
                     call m3mult4043(1000,bigdim,nep,nec,ndraw(i),dim,
     *                  lam(lampos(i,3)+1),dsigl4l6(posl4l6),
     *                  lam(lampos(i,4)+1),dsigdd(posdd),1)
                  endif
               endif
               if(dim.gt.1)call makesym(dim,dsigdd(posdd))
            endif
         endif
  230    posl4l6=posl4l6+nep*nec
         posdd=posdd+dim*dim
         if(nested(i).gt.0)posii=posii+bigdim*bigdim
  220 continue
      goto 999
 
c     Diagnosis: probit residual covariance matrix is not diagonal and
c     dependent probit exceeds dimension 3.
  800 if(ipt.gt.0)then
         write(8,801)
         if(allint6.eq.-1)write(8,802)
      endif
      if(jpt.gt.0)then
         write(6,801)
         if(allint6.eq.-1)write(6,802)
      endif
  801 format(/,'The function evaluation failed because the joint ',
     *   'covariance matrix of',/,'(simple, ordered, multinomial) ',
     *   'probit residuals is not diagonal.',/,'Block-diagonality ',
     *   'is allowed up to blocksizes of 3-by-3, but they',/,
     *   'are larger in your model.  This implies that there exists ',
     *   'correlation',/,'across probit equations and requires ',
     *   'multivariate cumulative normal',/,'normal routines, which ',
     *   'are only implemented up to trivariate.')
  802 format('This correlation may be due to correlation between ',
     *   'residuals in',/,'continuous outcomes.  Remove this ',
     *   'correlation by integrating out',/,'the higher-level ',
     *   'residuals out of continuous models.',/)
      status=1
      goto 999
  810 if(ipt.gt.0)write(8,811)
      if(jpt.gt.0)write(6,811)
  811 format(/,'ERROR:  a covariance matrix of probit residuals is ',
     *   'singular.',/,'Is it perhaps the case that multiple probit ',
     *   'residuals are the',/,'same, with the same draw?')
      status=2
      goto 999
  999 return
      end
