      subroutine compsig66(task,nfull,nec,numdist,distdim,distyp,
     *   distpos,ndraw,lamtyp6,lampos6,ifint6,sumdim2,dimarar,
     *   dimlam,sigdd,dsigdd,sigarar,dsigarar,lam,dlam,siga6a6,
     *   dsiga6a6,lndeta6a6,fullbeta,dfullbeta,v6,iflam,ipt,jpt,status)
 
c     Subroutine to compute the accumulated loaded residual covariance
c     of one observation on continuous outcomes, and to take its inverse.
c     The covariance matrix is the sum over as many matrices as
c     distributions have been defined.  Each matrix is a quadratic
c     form, Lambda6*Sigmaee*Lambda6', where Lambda6 is a matrix of load
c     factors and Sigmaee is a covariance matrix as defined by the user,
c     but expanded to reflect independent draws within an observation.
c     If the integrated distribution is also integrated out of
c     continuous models, then this distribution does not contribute to
c     the overall covariance matrix.
c
c     Even for task=1, dsiga6a6 is used (as scratch array for the
c     inverse of siga6a6.)
c     If task=2, derivatives are unraveled.
c
c     Note: straight application of unraveling formulas will make
c     matrices of derivatives wrt covariances asymmetric, under
c     certain circumstances.  Since the number of degrees of freedom
c     of such matrices is really only n*(n+1)/2, this routine
c     always restores symmetry after potentially destroying it.
c
c     Notation:
c        d  = residual, as Defined
c        e  = continuous residual, Expanded (multiple draws).
c             Starting in 3.00, e only exists in concept.  In reality,
c             d is used and expanded as needed by multiplying routines.
c             (Namely using matrix type 3, symmetric and block-diagonal.)
c        ar = C/ARMA.  Space allocated for C/ARMA matrices.
c        a6 = continuous residual, Accumulated (over distributions)
c
c     Constantijn Panis, 3 December 2002
c     Last change: log-determinant
 
      implicit none
      integer task,nfull,nec,numdist,distdim(numdist),distyp(numdist),
     *   distpos(numdist),ndraw(numdist),lamtyp6(numdist),
     *   lampos6(numdist),ifint6(numdist),sumdim2,dimarar,dimlam,
     *   ipt,jpt,status,bigdim,dim,i,posdd,posarar,p,q,vp
      double precision sigdd(sumdim2),dsigdd(sumdim2),sigarar(dimarar),
     *   dsigarar(dimarar),lam(dimlam),
     *   dlam(dimlam),siga6a6(nec,nec),dsiga6a6(nec,nec),sign,lndeta6a6
c     Dimension really is fullbeta(nfull), but array may go out of bounds
c     when calling subroutine arma when q=0.
      double precision fullbeta(nfull+1),dfullbeta(nfull+1)
      real v6(*)
      logical iflam(numdist),cumulative
 
      posdd=1
      posarar=1
      vp=nec+1
      if(task.eq.2)goto 200
 
      call zerodp(nec*nec,siga6a6)
      do 110 i=1,numdist
         dim=distdim(i)
         bigdim=dim*ndraw(i)
c        Exit if none of distribution i's residuals enters a continuous module.
         if(bigdim.eq.0)goto 120
c        Not all distributions contribute to the overall covariance
c        matrix.  Some do not enter into continuous models (ifint6=0);
c        others are integrated out (ifint6=1).
         if(ifint6(i).ne.-1)goto 120
         if(distyp(i).eq.1)then
c           Normal distribution.
c           Pre- and pos-multiply by Lambda6 and accumulate.
c           Lambda6 may be 0 (generic), 1 (identity), or 4 (sparse)
            if(lamtyp6(i).eq.0)then
               call quadratic032(0,nec,ndraw(i),dim,lam(lampos6(i)+1),
     *            sigdd(posdd),siga6a6,1)
            elseif(lamtyp6(i).eq.1)then
c              Lambda6 is identity matrix (nec=bigdim)
c              Accumulate sigdd onto ndraw blocks along siga6a6's diagonal
               call blowup(1,dim,ndraw(i),sigdd(posdd),siga6a6)
            elseif(lamtyp6(i).eq.4)then
               call m3mult4340(nec,ndraw(i),dim,nec,lam(lampos6(i)+1),
     *            sigdd(posdd),lam(lampos6(i)+1),siga6a6,1)
            endif
         elseif(mod(distyp(i),10).eq.2.or.mod(distyp(i),10).eq.3)then
c           C/ARMA distribution.  Branch level and time variables are stored
c           after the outcome variable.
c           distyp = 1000*l + 100*p + 10*q + 2 or
c           distyp = ... + 3 for cumulative ARMA processes.
            p = mod(distyp(i),1000)/100
            q = mod(distyp(i),100)/10
            cumulative=(mod(distyp(i),10).eq.3)
            call arma(1,p,q,ndraw(i),nec,fullbeta(distpos(i)),
     *         sigdd(posdd),fullbeta(distpos(i)+p+1),sigarar(posarar),
     *         0.d0,0.d0,0.d0,0.d0,v6(vp),v6(vp+nec),
     *         cumulative,ipt,jpt)
c           Pre- and pos-multiply by Lambda6 and accumulate.
c           Lambda6 may be 0 (generic), 1 (identity), or 4 (sparse)
            if(lamtyp6(i).eq.0)then
               call quadratic(0,nec,bigdim,lam(lampos6(i)+1),
     *            sigarar(posarar),siga6a6,1)
            elseif(lamtyp6(i).eq.1)then
c              Lambda is identity matrix (nec=bigdim)
               call vadd(sigarar(posarar),siga6a6,siga6a6,nec*nec)
            elseif(lamtyp6(i).eq.4)then
               call m3mult4040(0010,nec,bigdim,bigdim,nec,
     *            lam(lampos6(i)+1),sigarar(posarar),
     *            lam(lampos6(i)+1),siga6a6,1)
            endif
         elseif(mod(distyp(i),10).eq.4)then
c           Finite mixtures do not contribute to the covariance matrix
            goto 120
         else
            call bug(ipt,jpt)
         endif
  120    continue
c        Prepcov allocates dim*dim in sigdd, even for finite mixture
c        distributions
         posdd=posdd+dim*dim
         if(mod(distyp(i),10).eq.2.or.mod(distyp(i),10).eq.3)then
c           Space in sigarar for C/ARMA distributions:
            posarar=posarar+bigdim*bigdim
c           Prep6 reserved 2*nec spaces for every C/ARMA distribution,
c           even if not used (in this observation):
            vp=vp+2*nec
         endif
  110 continue
c     Compute the inverse of siga6a6.  Subroutine nrminv will
c     destroy siga6a6, so dsiga6a6 is used as scratch array.
c     Siga6a6 is positive definite, so we could use the more efficient
c     subroutine definv.  However, we also need the determinant.
      call nrminv(2,siga6a6,dsiga6a6,nec,sign,lndeta6a6,status)
      if(status.gt.0.or.sign.le.0.d0)goto 800
      call copy8(dsiga6a6,siga6a6,nec*nec)
      call zerodp(nec*nec,dsiga6a6)
      goto 999
 
  200 continue
      do 210 i=1,numdist
         dim=distdim(i)
         bigdim=dim*ndraw(i)
         if(bigdim.eq.0)goto 220
         if(ifint6(i).ne.-1)goto 220
         if(distyp(i).eq.1)then
c           Normal distribution.
c           Derivatives wrt Lambda and Sigma.  Since Sa6a6 = L6*See*L6',
c              dL6 = dSa6a6*L6*See' + dSa6a6'*L6*See = 2*dSa6a6*L6*See
c                            provided dSa6a6 and See are symmetric
c              dSee = L6'*dSa6a6*L6
            if(iflam(i))then
c              Check both iflam and lamtyp6; iflam could be true from other
c              model type, and Lambda could then be identity
               if(lamtyp6(i).eq.0)then
                  call m3mult0030(0,nec,nec,ndraw(i),dim,dsiga6a6,
     *              lam(lampos6(i)+1),sigdd(posdd),dlam(lampos6(i)+1),2)
               elseif(lamtyp6(i).eq.1)then
c                 Lambda6 is identity.  If Lambda6 were really estimated,
c                 figure_res would have ensured that lamtyp6 is not identity.
c                 Iflam must be true because of another model type.  (Very rare.
                  continue
               elseif(lamtyp6(i).eq.4)then
                  call m3mult0434(0,nec,nec,ndraw(i),dim,dsiga6a6,
     *              lam(lampos6(i)+1),sigdd(posdd),dlam(lampos6(i)+1),2)
               endif
            endif
            if(lamtyp6(i).eq.0)then
c              Lambda is generic
               call quadratic023(1,bigdim,nec,dim,lam(lampos6(i)+1),
     *            dsiga6a6,dsigdd(posdd),1)
            elseif(lamtyp6(i).eq.1)then
c              Lambda is the identity matrix.
c              Collapse block-diagonal dsiga6a6 onto dsigdd
               call blowup(2,dim,ndraw(i),dsigdd(posdd),dsiga6a6)
            elseif(lamtyp6(i).eq.4)then
c              Lambda is sparse
               call quadratic423(nec,ndraw(i),dim,lam(lampos6(i)+1),
     *            dsiga6a6,dsigdd(posdd),1)
            endif
         elseif(mod(distyp(i),10).eq.2.or.mod(distyp(i),10).eq.3)then
            p = mod(distyp(i),1000)/100
            q = mod(distyp(i),100)/10
            cumulative=(mod(distyp(i),10).eq.3)
c           Derivatives wrt Lambda and Sigma.  Since Sa6a6 = L6*Saa*L6',
c              dL6 = dSa6a6*L6*Saa' + dSa6a6'*L6*Saa = 2*dSa6a6*L6*Saa
c                            provided dSa6a6 and Saa are symmetric
c              dSaa = L6'*dSa6a6*L6
c           (Saa is the CAR/ARMA covariance matrix.)
            if(iflam(i))then
c              Check both iflam and lamtyp6; iflam could be true from other
c              model type, and Lambda could then be identity
               if(lamtyp6(i).eq.0)then
                  call m3mult(0,nec,nec,bigdim,bigdim,dsiga6a6,
     *               lam(lampos6(i)+1),sigarar(posarar),
     *               dlam(lampos6(i)+1),2)
               elseif(lamtyp6(i).eq.1)then
c                 C/ARMA distributions can only be used in continuous models.
                  call bug(ipt,jpt)
               elseif(lamtyp6(i).eq.4)then
                  call m3mult0404(0,nec,nec,bigdim,bigdim,dsiga6a6,
     *               lam(lampos6(i)+1),sigarar(posarar),
     *               dlam(lampos6(i)+1),2)
               endif
            endif
            if(lamtyp6(i).eq.0)then
c              Lambda is generic
               call quadratic(1,bigdim,nec,lam(lampos6(i)+1),
     *            dsiga6a6,dsigarar(posarar),1)
            elseif(lamtyp6(i).eq.1)then
c              Lambda is the identity matrix.
c              Normal distributions would now require collapsing block-diagonal
c              dsiga6a6 onto non-expanded covariance derivative matrix.
c              CAR/ARMA distributions have full covariance derivative matrix, ds
               call vadd(dsigarar(posarar),dsiga6a6,dsigarar(posarar),
     *            nec*nec)
            elseif(lamtyp6(i).eq.4)then
c              Lambda is sparse
               call m3mult4040(1000,nec,bigdim,bigdim,nec,
     *            lam(lampos6(i)+1),dsiga6a6,lam(lampos6(i)+1),
     *            dsigarar(posarar),1)
            endif
c           Then map wrt C/ARMA covariance into wrt defined covariance:
            call arma(2,p,q,ndraw(i),nec,fullbeta(distpos(i)),
     *         sigdd(posdd),fullbeta(distpos(i)+p+1),sigarar(posarar),
     *         dsigarar(posarar),dfullbeta(distpos(i)),dsigdd(posdd),
     *         dfullbeta(distpos(i)+p+1),v6(vp),v6(vp+nec),
     *         cumulative,ipt,jpt)
         elseif(mod(distyp(i),10).eq.4)then
c           Finite mixtures do not contribute to the covariance matrix
            goto 220
         else
            call bug(ipt,jpt)
         endif
  220    continue
         posdd=posdd+dim*dim
         if(mod(distyp(i),10).eq.2.or.mod(distyp(i),10).eq.3)then
            posarar=posarar+bigdim*bigdim
            vp=vp+2*nec
         endif
  210 continue
      goto 999
 
c     Singular covariance matrix of residual structure.
  800 if(ipt.gt.0)write(8,801)
      if(jpt.gt.0)write(6,801)
  801 format(/,'The function evaluation failed because the covariance ',
     *   'matrix of the',/,'continuous residual structure is ',
     *   'singular.  Include an additional',/,'residual that has an ',
     *   'independent draw for each outcome in continuous',/,'models. ',
     *   ' It may also be that singularity is the result of under-',/,
     *   'or overflows.',/)
      status=1
  999 return
      end
