      subroutine corr2chol(dim,fullpar,chol,status)
 
c     Subroutine to compute the Cholesky decomposition of a "covariance"
c     matrix that is stored in compressed format: first dim standard
c     deviations, then dim*(dim-1)/2 correlation coefficients.
c     The Cholesky decomposition is also stored in compressed format:
c     first diagonal elements, then off-diagonal, similar to the way
c     the correlations are stored.  (Note that this storage type may
c     be equally viewed as upper- and lower-triangular.
c     This routine is derived from subroutine cholesky.
c
c     Derivatives wrt the Cholesky decompisition may be converted into
c     wrt standard deviations and correlations, but this is quite tedious.
c     See subroutine cholesky for this purpose.
c
c     Constantijn Panis, 22 January 1994
 
      implicit none
      integer dim,status,i,j,k,ij,sq2cmp
      double precision fullpar(dim*(dim+1)/2),chol(dim*(dim+1)/2),s,t,
     *   cov
 
      status=0
      do 100 j=1,dim
         s = fullpar(j)*fullpar(j)
         do 200 i=1,j-1
            ij=sq2cmp(i,j,dim)
            cov = fullpar(i)*fullpar(j)*fullpar(ij)
c           Deduct the innerproduct of columns (or rows) i and j of
c           the Cholesky decomposition, elements 1 through i-1:
            do 210 k=1,i-1
               cov=cov-chol(sq2cmp(k,i,dim))*chol(sq2cmp(k,j,dim))
  210       continue
            t = cov/chol(i)
            chol(ij) = t
            s = s - t*t
  200    continue
         if(s.le.0.d0)goto 800
         chol(j) = dsqrt(s)
  100 continue
      goto 999
 
c     Not positive definite:
  800 status=1
  999 return
      end
