      subroutine define_spl(maxref,maxxnode,maxnumspl,splref,numspl,
     *   nfull,beginspl,numnodes,nodepos,spltype,xnodepos,xnode,
     *   splname,line,il,ipt,jpt)
 
c     Subroutine to define a spline, i.e., to define a set of nodes
c     and gammas, to provide it with a name and/or a reference number.
c     The syntax is as follows:
c     DEFINE SPLINE [name];  NODES = [x...x];
c                            [REF = [n...n];]
c                            [INTERCEPT = yes|no;]
c                            [EFFECT = full|right;]
c
c     maxref    [in] maximum number of reference numbers for all spline
c               definitions combined.
c     maxxnode  [in] maximum number of nodes in spline definitions and
c               values in dummy variable definitions and some other
c               data (dimension of xnode)
c     maxnumspl [in] maximum number of spline definitions
c     splref    [in] spline definition reference numbers
c     numspl    [in/out] number of splines defined
c     nfull     [in/out] number of model parameters
c     beginspl  [out] begin position of spline (intercept and) slopes
c               in model parameter vector.
c     spltype   [out] spline type: 1 = no intercept, full effect (default)
c                                  2 = intercept, full effect
c                                  3 = no intercept, right effect (rare)
c                                  4 = intercept, right effect
c     numnodes  [out] number of nodes in this spline
c     nodepos   [out] begin position in xnode of nodes of this spline
c     xnodepos  [in/out] current position in xnode
c     xnode     [out] vector of spline nodes and other things
c     splname   [out] name of spline
c     line      [in/out] current control file line
c     il        [in/out] current control file line position
c     ipt       [in] level of information to output file
c     jpt       [in] level of information to screen
c     *         error return
c
c     Constantijn Panis, 8 July 1996
 
      implicit none
      integer maxref,maxxnode,maxnumspl,splref(maxref),numspl,nfull,
     *   beginspl(maxnumspl),numnodes(maxnumspl),nodepos(maxnumspl),
     *   spltype(maxnumspl),xnodepos,nref,beginref,
     *   ipt,jpt,il,i1,i2,status,flag
      real xnode(maxxnode)
      character line*128,splname(maxnumspl)*12
      logical nodesdefined,intercept,kick,smtxt,realordered,
     *   dupchar12
 
      numspl=numspl+1
      if(numspl.gt.maxnumspl)call bug(ipt,jpt)
 
      nodesdefined=.false.
      nref=0
c     Default: no intercept, full effect
      intercept=.false.
      kick=.false.
 
      call getword(4,line,il,i1,i2,status)
      if(status.gt.0)goto 800
      if(i2-i1.ge.12)then
         goto 870
      elseif(line(i1:i2).eq.';')then
         splname(numspl)='(unnamed)'
      else
         splname(numspl)=line(i1:i2)
         if(.not.smtxt(4,line,il,';'))goto 800
c        Check that this name is unique
         if(dupchar12(splname(numspl),splname,numspl-1))goto 820
      endif
 
  100 if(smtxt(4,line,il,'REFerence='))goto 110
      if(smtxt(4,line,il,'NODEs='))goto 130
      if(smtxt(4,line,il,'INTERCEPT'))goto 140
      if(smtxt(4,line,il,'EFFECT='))goto 150
c     Nothing matched; check that the spline is named or referenced,
c     and that nodes have been defined.
      if(splname(numspl).eq.'(unnamed)'.and.nref.eq.0)goto 800
      if(.not.nodesdefined)goto 800
      beginspl(numspl)=nfull+1
c     spltype = 1: no intercept, full effect (default)
c             = 2: intercept, full effect
c             = 3: no intercept, right effect (rare)
c             = 4: intercept, right effect
      if(intercept)then
         nfull=nfull+numnodes(numspl)+2
         spltype(numspl)=2
      else
         nfull=nfull+numnodes(numspl)+1
         spltype(numspl)=1
      endif
      if(kick)then
         spltype(numspl)=spltype(numspl)+2
c        If a spline kicks in at some point, nodes must be non-negative:
         if(xnode(nodepos(numspl)).lt.0.)goto 880
      endif
      goto 999
 
c     REF = [n...n];
  110 if(nref.gt.0)goto 800
c     Locate begin point of reference numbers:
      call find_br(0,numspl,maxref,splref,beginref,ipt,jpt)
      call readrefs(maxref,beginref,splref,nref,line,il,ipt,jpt)
      goto 100
 
c     NODES = [n...n];
  130 if(nodesdefined)goto 800
      call getreals(4,xnode(xnodepos+1),maxxnode-xnodepos,
     *   numnodes(numspl),line,il,status)
      if(status.gt.0)goto 850
      if(.not.realordered(xnode(xnodepos+1),numnodes(numspl)))goto 860
      nodepos(numspl)=xnodepos+1
      xnodepos=xnodepos+numnodes(numspl)
      nodesdefined=.true.
      goto 100
 
c     INTERCEPT [= {yes|no}];
  140 call semiyesno(flag,4,line,il,ipt,jpt)
      intercept=(flag.eq.1)
      goto 100
 
c     EFFECT = {full|right)};
  150 if(smtxt(4,line,il,'FULL;'))then
         kick=.false.
      elseif(smtxt(4,line,il,'RIGHT;'))then
         kick=.true.
      else
         goto 800
      endif
      goto 100
 
  800 if(ipt.gt.0)then
         call caretline(8,line,il)
         write(8,801)
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         write(6,801)
      endif
  801 format('ERROR encountered while trying to read in a spline ',
     *   'definition.',/,'The correct syntax is:',/,
     *   6x,'DEFINE SPLINE [name];  NODES = [x...x];',/,
     *   29x,'[REF = [n...n];]',/,29x,'[INTERCEPT = yes|no;]',/,
     *   29x,'[EFFECT = full|right;]',/,
     *   'Note that you must name or assign reference numbers ',
     *   'to a',/,'spline definition.  You must also specify ',
     *   '"NODES=", though there',/,'may be zero nodes.  ',
     *   'The default is no intercept, full effect.')
      stop
  820 if(ipt.gt.0)then
         call caretline(8,line,il)
         write(8,821)
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         write(6,821)
      endif
  821 format('ERROR: you assigned duplicate names to spline ',
     *   'definitions.')
      stop
  850 if(xnodepos+numnodes(numspl)-1.le.maxxnode)goto 800
      call maxexceed('maxxnode',xnodepos+numnodes(numspl)-1,maxxnode,
     *   ipt,jpt)
  860 if(ipt.gt.0)then
         call caretline(8,line,il)
         write(8,861)
      endif
      if(jpt.gt.0)then
         call caretline(8,line,il)
         write(6,861)
      endif
  861 format('ERROR: the nodes in a spline definition are not in ',
     *   'increasing order.')
      stop
  870 if(ipt.gt.0)then
         call caretline(8,line,il)
         write(8,871)line(i1:i2)
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         write(6,871)line(i1:i2)
      endif
  871 format('ERROR: names of spline definitions may only be up ',
     *   'to 12 characters.',/,'The following name is too long: ''',
     *   a,'''.')
      stop
  880 if(ipt.gt.0)then
         call caretline(8,line,il)
         write(8,881)
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         write(6,881)
      endif
  881 format('ERROR: negative nodes do not make sense for a spline ',
     *   'that operates only',/,'after its starting moment, i.e., ',
     *   'negative nodes may not be specified',/,'in combination with ',
     *   '"EFFECT=right;" in the definition of a spline.')
      stop
  999 return
      end
