      subroutine ensposdef(dim,distrestr,ipt,jpt,stackpar,corr,
     *   fullname,posdef,status)
 
c     Subroutine to "fix" a non-positive definite correlation matrix
c     by reducing the correlations proportionally until the matrix
c     is strictly positive definite.
c
c     Constantijn Panis, 29 August 2002
c     Last change: if not positive definite and posdef=0, return status=1
 
      implicit none
      integer dim,distrestr(dim,dim),ipt,jpt,loop,num1,num2,i,j,
     *   posdef,status,numstd,numrho
      double precision stackpar((dim*(dim+1))/2),corr(dim,dim),lowest,
     *   eps,dpvecmin
      character fullname((dim*(dim+1))/2)*8
      parameter(eps=1.d-12)
 
      status=0
      loop=1
 
c     First check whether any standard deviation is (near) zero:
      num1=numstd(distrestr,dim)
      num2=numrho(distrestr,dim)
      lowest=dpvecmin(stackpar,num1)
      if(lowest.lt.eps)then
c        Write out warning but don't do anything about it.  Function
c        evaluation may blow up and return NaN, or perhaps cope with it.
         if(ipt.gt.1)write(8,101)(fullname(j),stackpar(j),j=1,num1)
         if(jpt.gt.1)write(6,101)(fullname(j),stackpar(j),j=1,num1)
  101    format(/,'WARNING:  one or more standard deviations are ',
     *      '(near) zero:',/,(6x,a8,1pe12.6))
         if(ipt.gt.0)write(8,'(1x)')
         if(jpt.gt.0)write(6,'(1x)')
      endif
 
c     Is the correlation matrix positive definite?
c     Map std/corr into correlation matrix with std on diagonal;
c     set std=1 to only look at correlations:
  100 call map_corr(1,dim,stackpar,0.d0,corr,0.d0,distrestr,dim+1)
      call putrow(1.d0,corr,dim,dim+1)
 
c     Compute eigenvalues and place in first dim elements of corr,
c     in descending order:
      call eigen(1,corr,0.d0,dim)
      lowest = corr(dim,1)
      call map_corr(1,dim,stackpar,0.d0,corr,0.d0,distrestr,dim+1)
 
c     Exit if all eigenvalues are positive
      if(lowest.gt.eps)goto 200
 
c     If not positive definite, write out warning:
      if(loop.eq.1)then
         if(ipt.gt.1)write(8,102)
         if(jpt.gt.1)write(6,102)
  102    format(/,'WARNING: the following correlation matrix is not ',
     *      'positive definite:',/)
         do 110 i=1,dim
            if(ipt.gt.0)write(8,'(100f10.4)')(corr(i,j),j=1,i-1),1.d0
            if(jpt.gt.0)write(6,'(100f10.4)')(corr(i,j),j=1,i-1),1.d0
  110    continue
         if(ipt.gt.0)write(8,'(1x)')
         if(jpt.gt.0)write(6,'(1x)')
      endif
 
c     By default, correlations are reduced proportionally until
c     the matrix is positive definite.  However, the user may specify
c     "option ensure positive definite=no;"  In that case, return NaN.
      if(posdef.eq.0)then
         status=1
         goto 200
      endif
 
c     Repeatedly reduce correlation coefficients by 0.1 percent until
c     the matrix is positive definite.  Figure out the number of
c     correlations and their positions in fullbeta, which is not
c     obvious if there are restrictions.
      call mult(stackpar(1+num1),.999d0,stackpar(1+num1),num2)
      loop=loop+1
      if(loop.le.10000)goto 100
      call bug(ipt,jpt)
 
c     If correlations have been changed, write out the new matrix
  200 if(loop.gt.1)then
         if(ipt.gt.1)write(8,201)
         if(jpt.gt.1)write(6,201)
  201    format('All correlations are reduced proportionally to ',
     *      'ensure that the',/,'correlation matrix is positive ',
     *      'definite.  (See the Reference',/,'Manual under ',
     *      'OPTION ENSURE POSITIVE DEFINITE for alternatives.)',/)
      endif
 
      return
      end
