      subroutine episused(datadim,nummodel,levelmax,datastructure,
     *   strucfreqs,minlnl,datanobs,nobs,wghtpos,sumweight,scaleweight,
     *   detail,oldname,ipt,jpt)
 
c     Subroutine to write out a frequency distribution of data structures
c     and to summarize how many outcomes the data generated.  Also write
c     out the number of conditioning outcomes and the sum of weights.
c     If normalized weights are desired, compute scaleweight=
c     nobs/sumweight to be used to adjust weights, in accumfunc.
c
c     strucfreqs(datadim,3) contain the following:
c     column 1 is frequencies of data structures in the data insofar
c        as the observations were used.
c     column 2 is frequencies dropped from the numerator as a result of
c        a keep/drop statement.
c     column 3 is frequencies dropped from the denominator as a result of
c        a keep/drop statement.
c
c     The first step of this routine is to convert columns 2 and 3 into
c     number of outcomes used in the estimation.  Then a table is written
c     with frequencies present in the data, dropped or used.  Then some
c     summarizing info is written regarding the number of outcomes and
c     observations entering in the numerator and denominator.
c
c     Constantijn Panis, 15 March 1997
c     Last change: new terminology
 
      implicit none
      integer datadim,nummodel,levelmax,strucfreqs(datadim,3),
     *   minlnl(nummodel),datanobs,nobs,wghtpos,ipt,jpt,unit,
     *   trm,countint,dlen
      real datastructure(datadim),sumweight,scaleweight
      logical detail,anymin
      character oldname(*)*8,name*8
 
c     For weighting purposes use the number of observations that
c     actually end up in the estimation (i.e., after keep/drop
c     statements.)
      scaleweight=1.
      if(wghtpos.lt.0)scaleweight=real(nobs)/sumweight
      anymin = (countint(0,minlnl,nummodel).ne.nummodel)
 
      if(ipt.lt.4)goto 200
      unit=8
 
  210 call write_freq3(datastructure,strucfreqs,datadim,
     *   anymin,levelmax,datanobs,nobs,unit,detail)
 
      write(unit,'(1x)')
      if(wghtpos.gt.0)then
         name=oldname(wghtpos)
         call writeada(unit,72,0,'Observations are weighted by '//
     *      'variable '''//name(1:trm(name))//'''; the sum of '//
     *      'weights is ',-(dlen(dble(sumweight))+5),dble(sumweight),
     *      ', the number of observations that are used in the '//
     *      'estimation')
         call writeint(unit,nobs,1,0)
         write(unit,'(a)')'.'
      elseif(wghtpos.eq.0)then
         write(unit,'(a)')'All observations are equally weighted.'
      elseif(wghtpos.lt.0)then
         name=oldname(-wghtpos)
         call writeada(unit,72,0,'Observations are weighted by '//
     *      'normalized variable '''//name(1:trm(name))//'''; '//
     *      'the sum of weights is ',-(dlen(dble(sumweight))+5),
     *      dble(sumweight),', the number of observations that are '//
     *      'used in the estimation')
         call writeint(unit,nobs,1,0)
c        write(unit,205)
         write(unit,205)
cD        write(unit,205,advance='no')
c 205    format(', so weights are scaled by',$)
  205    format(', so weights are scaled by',$)
cD 205    format(', so weights are scaled by')
         call writedble(unit,dble(scaleweight),1,
     *      -(dlen(dble(scaleweight))+6),0,0)
         write(unit,'(a)')'.'
      endif
 
  200 if(unit.eq.6.or.jpt.lt.4)goto 999
      unit=6
      goto 210
 
  999 return
      end
