      subroutine fignest(numdist,ne,ndraw,draw,nested,nestpair,lamtyp,
     *   numnest,neh,nel,nep,nec,neb,nen,nepo,neml,ipt,jpt,iobs,id)
 
c     Subroutine to figure out how integrated distributions are nested.
c     The procedure is:
c        First determine pairwise nesting and independence for all
c           integrated distributions.
c        Then determine joint nesting, including multiple distributions
c           that are independent but all nested under some other
c           distribution.
c        Then determine whether certain zero draws of outer distributions
c           need to be changed to ensure proper evaluation.
c
c     This routine is called by prepscratch.
c
c     Constantijn Panis, 11 November 2002
c     Last change: Poisson
 
      implicit none
      integer numdist,ne,ndraw(numdist),draw(numdist,ne),
     *   nested(numdist,2),nestpair(numdist,numdist),lamtyp(numdist,8),
     *   numnest,neh,nel,nep,nec,neb,nen,nepo,neml,ipt,jpt,iobs,id,
     *   integrated,ii,kk,ll,i,j,k,ie,inner,outer,donor,dist,
     *   nest,dependent,numint,trm,intlen,intvecmax,countint
      character fmt1*16,fmt2*16,i2c*12
 
      call zeroint(numdist*numdist,nestpair)
      call zeroint(2*numdist,nested)
 
c     Cases to be handled properly:
c     Misc\nested6a:
c        delta  ( 0 1 1 0 1 0 1 )
c        eps    ( 0 0 0 1 0 1 0 )
c        eta    ( 0 0 1 0 2 0 3 )
c
c     Trouble\trouble9:
c       ( 1  1  0  0  0  0 )
c       ( 0  0  1  1  0  0 )
c       ( 0  0  0  0  1  0 )
c       ( 0  0  0  0  0  1 )
c
c     Misc\nested3a:
c        ( 0 0 0 0 0 0 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 )
c        ( 0 0 0 1 2 3 0 0 0 1 1 1 1 2 2 2 2 3 3 3 3 )
c
c     Misc\nested6c:
c        ( 1 1 1 1 1 1 1 0 0 )
c        ( 0 1 1 1 2 2 2 0 0 )
c        ( 1 2 3 4 5 6 7 0 0 )
c        ( 0 0 0 0 0 0 0 1 1 )
c        ( 0 0 0 0 0 0 0 1 2 )
c
c     Misc\nested6f:
c        ( 1 1 1 1 1 1 1 0 0 0 )
c        ( 0 1 1 1 2 2 2 0 0 0 )
c        ( 1 2 3 4 5 6 7 0 0 0 )
c        ( 0 0 0 0 0 0 0 1 1 0 )
c        ( 0 0 0 0 0 0 0 1 2 0 )
c        ( 0 0 0 0 0 0 0 0 0 1 )
 
c     Array draw(numdist,ne) keeps track of residual draws for each
c     distribution, each outcome module.  Non-integrated residuals
c     were assigned negative numbers (in figure_res) to prevent them
c     from being mixed with integrated residuals.  Figure_res also
c     made sure that all occurrences of each distribution are either
c     all or none integrated out (except for occurrences in continuous
c     models).  Draw is only used for integrated residuals; zero out
c     non-integrated draws, which simplifies things in ithobs (onepoint).
      numint=0
      do 100 i=1,numdist
         integrated=0
         do 110 ie=1,ne
            if(draw(i,ie).lt.0)then
               draw(i,ie)=0
            elseif(draw(i,ie).gt.0)then
               integrated=1
            endif
  110    continue
         if(integrated.le.0)then
c           Set ndraw to negative number; reverse later in this routine.
            ndraw(i)=-ndraw(i)
         else
c           This distribution is integrated out; count how many are
c           integrated out.
            numint=numint+1
         endif
  100 continue
 
c     For each integrated distribution pair, figure out which is nested
c     inside the other, or whether they are independent.
c     Start by assuming that all distributions are independent.
      do 200 i=2,numdist
         if(ndraw(i).le.0)goto 200
         do 210 j=1,i-1
            if(ndraw(j).le.0)goto 210
c           Consider the following draws:
c              ( 1 2 3 4 5 6 7 8 8 8 8 4 )
c              ( 0 0 0 1 0 0 0 2 3 4 5 2 )
c           Looping horizontally over modules, the distributions appear
c           independent until module 4.  Even though the second distribution
c           has the lower draw, this is no indication of which is nested
c           in which.  Modules 5, 6, and 7 do not add information.
c           Module 8 requires checking previous modules in which both
c           draws were nonzero; nether (1)8 nor (2)2 has been seen before.
c           Module 9 again requires checking.  (1)8 has been used before;
c           (2)3 not.  (2) must thus be nested in (1).
c           Modules 10 and 11 again require checking; (2) remains nested.
c           Module 12 requires checking: both (1)4 and (2)2 have been
c           used before, but not always in the same module.  The
c           distributions are thus not nested.
            dependent=0
            do 220 ie=1,ne
               if(draw(i,ie).eq.0.or.draw(j,ie).eq.0)goto 220
c              The distributions are dependent.  If dependency was detected
c              earlier, check earlier modules for the same draws.
               if(dependent.eq.1)then
                  do 230 ii=1,ie-1
                     if(draw(i,ii).eq.0.or.draw(j,ii).eq.0)goto 230
                     if(draw(i,ii).eq.draw(i,ie))then
                        if(draw(j,ii).eq.draw(j,ie))goto 230
c                       (i) draws are equal, (j) draws nonzero and unequal,
c                       so (j) must be nested in (i).
                        if(nestpair(i,j).eq.0)nestpair(i,j)=i
                        if(nestpair(i,j).ne.i)goto 800
                     elseif(draw(j,ii).eq.draw(j,ie))then
c                       (j) draws are equal, (i) draws nonzero and unequal,
c                       so (i) must be nested in (j).
                        if(nestpair(i,j).eq.0)nestpair(i,j)=j
                        if(nestpair(i,j).ne.j)goto 800
                     endif
c                    Else: draws are both different; no info on nesting.
  230             continue
               else
                  dependent=1
               endif
  220       continue
c           Consider the following draw (Trouble\trouble8b, obs=209):
c              ( 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 )
c              ( 1 2 1 2 1 2 1 2 1 2 1 2 1 2 1 2 )
c              ( 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 )
c           Distributions (1) and (3) are dependent, but the above could not
c           determine which is nesting which.  It is indeed arbitrary.
            if(nestpair(i,j).eq.0.and.dependent.eq.1)nestpair(i,j)=j
c           Make matrix symmetric (facilitates 300 loop):
            nestpair(j,i)=nestpair(i,j)
  210    continue
  200 continue
 
c     Nestpair indicates which distributions are nested within others on a
c     pairwise basis.  Determine all nest hierarchy (first column of nested)
c     and set pointers to the distribution under which each distribution
c     is nested (second column of nested).
c     Consider the following draws:
c        ( 1 1 1 1 1 1 1 2 2 )
c        ( 0 1 1 1 2 2 2 0 0 )
c        ( 1 2 3 4 5 6 7 0 0 )
c        ( 0 0 0 0 0 0 0 1 2 )
c     Distribution (1) is outermost; (2) and (4) are one level in; (3) is
c     one more level in, under (2).  Nestpair is and nested needs to be:
c                   ( . 1 1 1 )             ( 1 0 )
c                   ( 1 . 2 0 )             ( 2 1 )
c        nestpair = ( 1 2 . 0 )    nested = ( 3 2 )
c                   ( 1 0 0 . )             ( 2 1 )
 
c     First determine the outermost distribution(s).  An outermost distribution
c     has all its nestpair elements equal to zero or its own value.
c     Then determine the next inner distribution(s).  A next inner distribution
c     has all its nestpair elements equal to zero, more outer distributions,
c     or its own value.
c     Then determine the next inner distribution(s), et cetera.
      nest=1
  300 do 310 i=1,numdist
c        Is distribution (i) at nest?
         if(ndraw(i).le.0)goto 310
         if(nested(i,1).ne.0)goto 310
         do 320 j=1,numdist
            if(ndraw(i).le.0)goto 320
            if(i.eq.j)goto 320
            if(nestpair(i,j).eq.0.or.nestpair(i,j).eq.i)then
c              Distribution i could be at any nest
               continue
            else
c              nestpair(i,j) is neither zero nor i, so it is j; if (j) has
c              not yet been identified as more outer, (i) is not at nest.
               if(nested(j,1).eq.0)goto 310
               if(nested(j,1).eq.nest)goto 310
            endif
  320    continue
c        Distribution i is indeed at nest:
         nested(i,1)=nest
c        Under what distribution does it nest?
         if(nest.eq.1)goto 310
         do 330 j=1,numdist
            if(ndraw(i).le.0)goto 330
            if(i.eq.j)goto 330
            if(nested(j,1).ne.nest-1)goto 330
            if(nestpair(i,j).eq.0)goto 330
c           Check that nestpair(i,j)=j
            if(nestpair(i,j).ne.j)call bug(ipt,jpt)
            nested(i,2)=j
            goto 310
  330    continue
  310 continue
      if(countint(0,nested,numdist).gt.numdist-numint)then
         nest=nest+1
         goto 300
      endif
      numnest=nest
 
c     Set ndraw back to all positive integers again; verify that
c     nested is OK
      do 700 i=1,numdist
         if(nested(i,1).eq.0)then
            if(ndraw(i).gt.0)call bug(ipt,jpt)
            ndraw(i)=-ndraw(i)
         else
            if(ndraw(i).le.0)call bug(ipt,jpt)
            if(nested(i,1).gt.1)then
               j=nested(i,2)
               if(nested(j,1).ne.nested(i,1)-1)call bug(ipt,jpt)
            endif
         endif
  700 continue
 
c     Consider a model with two integrated distributions, say delta
c     and eps with ndraw=(1,3).  Consider the following draw array:
c        ( 0 0 0 0 0 0 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 )
c        ( 0 0 0 1 2 3 0 0 0 1 1 1 1 2 2 2 2 3 3 3 3 )
c
c     (This corresponds to the first observation of Misc\nested3a in the
c     test bank.)  Note the 4th, 5th, and 6th outcomes.  They have
c     draw(1)=0, i.e., they are unaffected by (1).  They still
c     need to be evaluated inside the (1) integration, though, because
c     they are affected by nonzero draws of (2), which is nested within
c     (1).  For example, outcomes 4 and 10-13 all involve the first
c     draw of (2).  If they were to be evaluated with draw 0 for (1)
c     (outcome 4) and draw 1 of (1) (outcome 10-13), then (1) would
c     be nested in (2).  Higher draws of (2) would not be allowed anymore.
c     Instead, (2) may be viewed as nested within (1), but only if
c     a specific draw of (2) is evaluated within the same (1) draw.
c     The situation above may be saved by setting the draw of (1) to
c     1 for outcomes 4-6.
c
c     The loop below therefore changes the above draws to:
c        ( 0 0 0 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 )
c        ( 0 0 0 1 2 3 0 0 0 1 1 1 1 2 2 2 2 3 3 3 3 )
c
c     Note that this trick does not imply that (1) is suddenly integrated
c     out of modules 4-6.  The corresponding load matrix consists of all
c     zeroes.  It is true, though, that the conditional likelihood is
c     computed more times than an ultra-smart algorithm with lots of
c     well organized space to store derivatives would require.  The
c     proposed solution is magnitudes simpler, though.
 
      do 410 outer=1,numdist
         if(nested(outer,1).eq.0.or.nested(outer,1).eq.numnest)goto 410
         do 420 inner=1,numdist
c           Is (inner) is nested within (outer)?
            if(nested(inner,1).le.nested(outer,1))goto 420
c           They may still not be nested, namely if inner is nested within
c           distributions that are part of another integration branch.
            i=inner
  450       i=nested(i,2)
            if(i.eq.0)goto 420
            if(i.ne.outer)goto 450
c           (inner) is nested within (outer)
            do 430 ie=1,ne
               if(draw(outer,ie).eq.0.and.draw(inner,ie).ne.0)then
c                 Other modules must have shown both nonzero draws,
c                 otherwise they would have been marked independent.
c                 Find another module with the same inner draw.
                  do 440 donor=1,ne
                     if(donor.eq.ie.or.draw(outer,donor).eq.0)goto 440
                     if(draw(inner,donor).eq.draw(inner,ie))then
                        draw(outer,ie)=draw(outer,donor)
                        if(ie.le.neh)then
                           if(lamtyp(outer,1).eq.-1)lamtyp(outer,1)=4
                        elseif(ie.le.neh+nel)then
                           if(lamtyp(outer,2).eq.-1)lamtyp(outer,2)=4
                        elseif(ie.le.neh+nel+nep)then
                           if(lamtyp(outer,3).eq.-1)lamtyp(outer,3)=4
                        elseif(ie.le.neh+nel+nep+nec)then
                           if(lamtyp(outer,4).eq.-1)lamtyp(outer,4)=4
                        elseif(ie.le.neh+nel+nep+nec+neb)then
                           if(lamtyp(outer,5).eq.-1)lamtyp(outer,5)=4
                        elseif(ie.le.neh+nel+nep+nec+neb+nen)then
                           if(lamtyp(outer,6).eq.-1)lamtyp(outer,6)=4
                        elseif(ie.le.neh+nel+nep+nec+neb+nen+nepo)then
                           if(lamtyp(outer,7).eq.-1)lamtyp(outer,7)=4
                        elseif(ie.le.neh+nel+nep+nec+neb+nen+nepo+neml)
     *                     then
                           if(lamtyp(outer,8).eq.-1)lamtyp(outer,8)=4
                        endif
                        goto 430
                     endif
  440             continue
c                 No donor module found.  Apparently the inner draw does
c                 not appear in any other module with nonzero outer draw.
c                 The zero outer draw is then OK.  See, for example,
c                 Misc\nested6c.
               endif
  430       continue
  420    continue
  410 continue
 
c     Now consider two distributions that were defined independently
c     and that separate in outcomes.  For example, obs=3 in
c     Speed\independent has one marriage and one divorce outcome:
c            ( 1 0 )
c     draw = ( 0 1 )
c
c     Both distributions have 4 points, so that the current smart
c     algorithm only evaluates 2*4 conditional likelihoods.  The two
c     distributions are namely sisters (nested at the same level),
c     and the 120 loop of nextdraw moves to the next distribution,
c     if any, at any given nesting level.  This is fine as long as
c     the outcomes involved are truly independent.  Now consider
c     Select\select2a.  One outcome is continuous, the other probit,
c     and their non-integrated residuals are correlated.  The residual
c     distribution of the probit now depends on continuous outcomes,
c     so they must be evaluated jointly.  Make the two integrated
c     distributions artificially nested within each other.
      if(nep.eq.0.or.nec.eq.0)goto 600
      do 500 i=1,numdist
         do 510 j=i+1,numdist
            if(nested(i,1).eq.nested(j,1).and.
     *         nested(i,2).eq.nested(j,2))then
c              Distributions i and j are sister distributions.
c              Do they involve both continuous and probit outcomes?
               do 520 ie=neh+nel+1,neh+nel+nep
                  if(draw(i,ie).gt.0.or.draw(j,ie).gt.0)goto 530
  520          continue
c              Neither distribution enters probit modules
               goto 510
  530          do 540 ie=neh+nel+nep+1,neh+nel+nep+nec
                  if(draw(i,ie).gt.0.or.draw(j,ie).gt.0)goto 550
  540          continue
c              Neither distribution enters continuous modules
               goto 510
c              Distributions affect both probit and continuous outcomes:
  550          if(ndraw(i).gt.1.or.ndraw(j).gt.1)goto 810
               nested(j,1)=nested(i,1)+1
               nested(j,2)=i
c              Numnest probably increases by one, but not necessarily
               numnest=intvecmax(nested,numdist)
c              Adjust draws similar to in 400 loop
               do 525 ie=neh+nel+1,neh+nel+nep
                  draw(j,ie)=1
                  draw(i,ie)=1
                  if(lamtyp(i,3).eq.-1)lamtyp(i,3)=4
                  if(lamtyp(j,3).eq.-1)lamtyp(j,3)=4
  525          continue
               do 545 ie=neh+nel+nep+1,neh+nel+nep+nec
                  draw(j,ie)=1
                  draw(i,ie)=1
                  if(lamtyp(i,4).eq.-1)lamtyp(i,4)=4
                  if(lamtyp(j,4).eq.-1)lamtyp(j,4)=4
  545          continue
               if(numnest.eq.nested(j,1))goto 510
c              Move all higher nesting levels under j one level up
c              Note: the 560 loop has not been tested.
               do 560 k=1,numdist
                  if(k.eq.j.or.nested(k,1).lt.nested(j,1))goto 560
c                 Distribution k is nested at at least j's level.  If it
c                 is eventually nested within j, increase nesting level.
                  kk=k
  570             dist=nested(kk,2)
                  if(dist.eq.j)then
                     nested(kk,1)=nested(kk,1)+1
                     goto 560
                  elseif(nested(dist,1).le.nested(j,1))then
                     goto 560
                  else
                     kk=dist
                     goto 570
                  endif
  560          continue
            endif
  510    continue
  500 continue
 
  600 goto 999
 
  800 if(ipt.gt.0)then
         call writea4ia(8,72,1,'ERROR:  the integrated distributions '//
     *      'in your model for observation ',iobs,' (ID=',id,') are '//
     *      'not nested.  The following matrix shows the draw '//
     *      'numbers for all ',ne,' outcomes (horizontally) and all ',
     *      numdist,' distributions (vertically).')
         write(8,801)
      endif
      if(jpt.gt.0)then
         call writea4ia(6,72,1,'ERROR:  the integrated distributions '//
     *      'in your model for observation ',iobs,' (ID=',id,') are '//
     *      'not nested.  The following matrix shows the draw '//
     *      'numbers for all ',ne,' outcomes (horizontally) and all ',
     *      numdist,' distributions (vertically).')
         write(6,801)
      endif
  801 format('Note that the draws may deviate from your "intres(',
     *   'draw=..."',/,'specifications; new contiguous numbers have ',
     *   'been assigned based on',/,'first occurrence in the data.  ',
     *   'Draws for distributions that are not',/,'integrated out ',
     *   'have been suppressed and shown with a period.',/)
      fmt1='(3x,a,'//i2c(ne,0)
      ll=intvecmax(draw,ne*numdist)
      fmt2=fmt1(1:trm(fmt1))//'a'//i2c(max(3,intlen(ll)+1),0)
      fmt1=fmt1(1:trm(fmt1))//'i'//i2c(max(3,intlen(ll)+1),0)
      fmt1=fmt1(1:trm(fmt1))//',a)'
      fmt2=fmt2(1:trm(fmt2))//',a)'
      do 805 ll=1,numdist
         if(ndraw(ll).gt.0)then
            if(ipt.gt.0)write(8,fmt1)'(',(draw(ll,kk),kk=1,ne),' )'
            if(jpt.gt.0)write(6,fmt1)'(',(draw(ll,kk),kk=1,ne),' )'
         else
            if(ipt.gt.0)write(8,fmt2)'(',('.',kk=1,ne),' )'
            if(jpt.gt.0)write(6,fmt2)'(',('.',kk=1,ne),' )'
         endif
  805 continue
      if(ipt.gt.0)then
         write(8,'(1x)')
         call writea4ia(8,72,1,'Hint: the ',ii,'-th and ',ie,'-th '//
     *      'outcome modules appear to have draws for the ',i,'-th '//
     *      'and ',j,'-th distribution which are logically impossible.')
      endif
      if(jpt.gt.0)then
         write(6,'(1x)')
         call writea4ia(6,72,1,'Hint: the ',ii,'-th and ',ie,'-th '//
     *      'outcome modules appear to have draws for the ',i,'-th '//
     *      'and ',j,'-th distribution which are logically impossible.')
      endif
      stop
  810 if(ipt.gt.0)write(8,811)
      if(jpt.gt.0)write(6,811)
  811 format(/,'ERROR.  You defined at least two distributions that ',
     *   'are integrated out',/,'in probit and continuous outcomes, ',
     *   'such that the distributions are not',/,'nested.  Since ',
     *   'probit residuals are conditional on continuous outcomes,',/,
     *   'probit and continuous outcomes must be evaluated at the ',
     *   'same time.',/,'This requires that the distributions be ',
     *   'nested or may be treated as-if',/,'they are nested.  aML ',
     *   'does not know how to do the latter, because at',/,'least ',
     *   'one distribution has multiple draws.  Remedy 1: re-define ',
     *   'the two',/,'distributions as a single multivariate ',
     *   'distribution.  Remedy 2:',/,'re-specify your models such ',
     *   'that the distributions are nested.  This',/,'message should ',
     *   'only arise in extraordinarily complex models.  You may wish',
     *   /,'to request assistance from Technical Support ',
     *   '<support@applied-ml.com>.')
      stop
  999 return
      end
 
