      subroutine fignewbeta(task,nfull,nfree,fulltype,iffull,
     *   oldfullbeta,searchdir,factor,newfullbeta,rss,nchol,chol,status)
 
c     Subroutine to compute new parameter vector as old parameter
c     vector plus a stretch factor times the search direction:
c          newfreebeta = oldfreebeta + factor*searchdir
c     Integer array domaintype indicates whether there are any
c     restrictions on the domain of the parameters; note that
c     the search direction is applied to the transformed parameter,
c     so two steps is different from twice the search direction.
c
c     Depending on the value of task, the action undertaken varies:
c     task=1 return newfreebeta
c          2 compute maximum relative step, max(newfreebeta/oldfreebeta-1)
c
c     Task 2 requires factor=1.d0; this is _not_ assumed!
c
c     Note: oldfreebeta and newfreebeta may not share the same memory space;
c     domaintype=4,5 screw things up.
c
c     Upon output, if status=1, one or more parameters lead to overflow.
c
c     Constantijn Panis, 28 January 1997
c     Last change:  bug fix.  In case not all elements of an increasing
c        vector are estimated, temp was set to 0 but should be set to
c        the last fixed parameter value.
 
      implicit none
      double precision pi,halfpi
      parameter(pi=3.141592653589793238462643d0,halfpi=.5d0*pi)
      integer task,nfull,nfree,fulltype(nfull),iffull(nfull),
     *   nchol,i,j,dim,cmpdim,pos,status,intcontig
      double precision oldfullbeta(nfull),searchdir(nfree),
     *   newfullbeta(nfull),factor,rss,chol(nchol),temp,dpinf,maxexp
 
      status=0
c     Initialize newfullbeta (so that values that are not estimated
c     are properly assigned)
      if(task.eq.1)call copy8(oldfullbeta,newfullbeta,nfull)
      if(task.eq.2)then
         rss=0.d0
c        Quick check for whether any old, free parameter is zero
         j=0
         do 100 i=1,nfull
            if(iffull(i).eq.0)goto 100
            j=j+1
            if(oldfullbeta(i).ne.0.d0.or.searchdir(j).eq.0.d0)goto 100
            rss=dpinf()
            goto 999
  100    continue
      endif
 
      pos=1
      i=0
      j=0
  200 i=i+1
      if(i.gt.nfull)goto 999
      if(iffull(i).eq.0)goto 200
      j=j+1
      if(fulltype(i).eq.0)then
c        No transformation
         temp = oldfullbeta(i) + factor*searchdir(j)
      elseif(fulltype(i).eq.1)then
c        Strictly positive via natural logarithm
         temp = dlog(oldfullbeta(i)) + factor*searchdir(j)
         if(temp.lt.maxexp())then
            temp=dexp(temp)
         else
            temp=dpinf()
            status=1
         endif
      elseif(fulltype(i).eq.2)then
c        Between -1 and 1 via arc-tangent
         temp = datan(dtan(oldfullbeta(i)*halfpi) +
     *      factor*searchdir(j))/halfpi
      elseif(fulltype(i).eq.3)then
c        Between 0 and 1 via arc-tangent
         temp = .5d0+datan(dtan(oldfullbeta(i)*pi-halfpi) +
     *      factor*searchdir(j))/pi
      elseif(fulltype(i).eq.41)then
c        First in series of increasing; not transformed
         temp = oldfullbeta(i) + factor*searchdir(j)
      elseif(fulltype(i).eq.4)then
c        Subsequent in increasing series; change accumulates
         if(iffull(i-1).eq.0)temp=oldfullbeta(i-1)
         if(factor*searchdir(j).lt.maxexp())then
            temp=temp + (oldfullbeta(i)-oldfullbeta(i-1))*
     *         dexp(factor*searchdir(j))
         else
            temp=dpinf()
            status=1
         endif
      endif
c     Break here because of choice between newfullbeta(i)/rss:
      if(fulltype(i).eq.5)then
c        next parameters (oldfullbeta/newfullbeta) are standard deviations and
c        correlations, to be searched using the Cholesky decomposition.
c        Search direction searchdir is in terms of Cholesky decomposition,
c        except that the first dim elements are transformed again to be
c        strictly positive.
         dim=intcontig(5,fulltype(i),nfull-i)
         cmpdim=dim*(dim+1)/2
         call newcholpar(task,dim,cmpdim,oldfullbeta(i),factor,
     *      newfullbeta(i),chol(pos),searchdir(j),rss,status)
         i=i+cmpdim-1
c        Assume that all matrix parameters are free:
         j=j+cmpdim-1
         pos=pos+cmpdim
      else
         if(task.eq.1)then
            newfullbeta(i)=temp
         elseif(task.eq.2)then
            if(oldfullbeta(i).ne.0.d0)rss=
     *         max(rss,abs(temp/oldfullbeta(i)-1.d0))
         endif
      endif
      goto 200
 
  999 return
      end
c=================================================================
      subroutine newcholpar(task,dim,cmpdim,oldpar,factor,newpar,
     *   chol,searchdir,rss,status)
 
c     Subroutine to figure out dim*(dim+1)/2 new parameters that are
c     searched on by a Cholesky decomposition.
c
c     Constantijn Panis, 22 January 1994
 
      implicit none
      integer task,dim,cmpdim,status,i,j,k,sq2cmp
      double precision oldpar(cmpdim),factor,newpar(cmpdim),
     *   chol(cmpdim),searchdir(cmpdim),rss,
     *   newsigmai,newsigmaj,temp,temp1,temp2,maxexp,dpinf
 
      do 100 j=1,dim
         temp=dlog(chol(j))+factor*searchdir(j)
         if(temp.ge..5d0*maxexp())goto 800
         temp=dexp(temp)
         newsigmaj=temp*temp
         do 110 k=1,j-1
            temp=chol(sq2cmp(k,j,dim))+factor*searchdir(sq2cmp(k,j,dim))
            newsigmaj=newsigmaj+temp*temp
  110    continue
         newsigmaj=dsqrt(newsigmaj)
         if(task.eq.1)then
            newpar(j)=newsigmaj
         elseif(task.eq.2)then
            if(oldpar(j).ne.0.d0)rss=
     *         max(rss,abs(newsigmaj/oldpar(j)-1.d0))
         endif
         do 140 i=1,j-1
            temp=dlog(chol(i))+factor*searchdir(i)
            if(temp.ge..5d0*maxexp())goto 800
            temp=dexp(temp)
            newsigmai=temp*temp
            do 120 k=1,i-1
               temp=chol(sq2cmp(k,i,dim))+
     *            factor*searchdir(sq2cmp(k,i,dim))
               newsigmai=newsigmai+temp*temp
  120       continue
            newsigmai=dsqrt(newsigmai)
c           Compute new parameter:
            temp=0.d0
            do 130 k=1,i
               if(k.eq.i)then
                  temp1=dlog(chol(k))+factor*searchdir(k)
                  if(temp1.gt.maxexp())goto 800
                  temp1=dexp(temp1)
               else
                  temp1=chol(sq2cmp(k,i,dim))+
     *               factor*searchdir(sq2cmp(k,i,dim))
               endif
               temp2=chol(sq2cmp(k,j,dim))+
     *               factor*searchdir(sq2cmp(k,j,dim))
               temp=temp+temp1*temp2
  130       continue
            temp=temp/(newsigmai*newsigmaj)
            if(task.eq.1)then
               newpar(sq2cmp(i,j,dim))=temp
            elseif(task.eq.2)then
               if(oldpar(sq2cmp(i,j,dim)).ne.0.d0)rss=
     *            max(rss,abs(temp/oldpar(sq2cmp(i,j,dim))-1.d0))
            endif
  140    continue
  100 continue
      goto 999
 
c     One or more parameters flow over
  800 status=1
      if(task.eq.1)then
         call put(dpinf(),newpar,cmpdim)
      elseif(task.eq.2)then
         rss=dpinf()
      endif
      goto 999
 
  999 return
      end
