      subroutine figure_res(levelmax,nbdim,nb,branch,mdllvl,nmapmoel,
     *   mapmoel,meno,maxmeno,moel,xpntr,namepntr,nxold,maxxnode,nold,
     *   numdist,sumdim,distdim,distyp,lamtyp,lamrow,distpos,neint,
     *   ifint,intpnts,dimused,nfull,fulltype,modeltype,mtype,maxdraws,
     *   draws,draw,ndraw,numterm,resdone,sparsity,maxref,resref,vdim,
     *   oldvdim,struc,iobs,id,ipt,jpt,xdim,x,nec,ie,v6,xnode,distvar,
     *   oldname,resname,iflam)
 
c     Subroutine to figure out the draw and residual of any module.
c     Upon input, moel contains (see read_res):
c        moel(1) = 4  This is a integrated residual;
c                  7  this is a non-integrated residual
c        moel(2) = 1  draw is given by variable or expression
c                  2  draw is _leveln
c                  3  draw is _iid
c                  4  residual(draw=_iid, ref=N(0,1))
c        moel(3) =    position in xnode where expression starts if moel(2)=1
c                     level if moel(2)=2
c        moel(4) = 1  residual is directly referenced
c                  2  residual is indirectly referenced
c        moel(5) = distribution rank number if moel(4)=1
c                  expression position in xnode if moel(4)=2
c        moel(6) = dimension rank number if moel(4)=1
c
c     Upon output, mapmoel contains:
c        mapmoel(1) = 4  This is a integrated residual;
c                     7  this is a non-integrated residual
c               (2) = distribution number
c               (3) = dimension in this distribution
c               (4) = draw number (sequential, not from draw=varname)
c
c     Note: mapmoel(2)=9999 indicates an implicit iid N(0,1) residual.
c     In those cases, there is no mapmoel(3) or mapmoel(4).
c
c     If the residual belongs to an C/ARMA distribution (only allowed in
c     continuous modules, as non-integrated residual), additional
c     information is placed in scratch vector v.  For each C/ARMA distribution,
c     2*nec spaces are allocated.  The first nec indicate
c     the absolute branch number to which a continuous outcome belongs,
c     at the level across which C/ARMA residuals are uncorrelated.  For
c     example, suppose the user specified
c        DEFINE ARMA(p,q) DISTRIBUTION;  DIM = 1;
c           TIMEVAR (WITHIN LEVEL 2) = varname;
c           NAME = eps;
c
c     This implies that C/ARMA residuals are autocorrelated within level 2,
c     (i.e., across branches of level 3+), but uncorrelated across level 2.
c     The first nec values in v then indicate the level 2 branch number.
c     Subroutine arma will create a block-diagonal autocovariance matrix
c     where the blocks are determined from unique branch numbers.
c
c     The second nec positions in v are for the time variable.
c     These must be strictly increasing within (here) level 2.
c
c     Constantijn Panis,  3 December 2002
c     Last change: set intpnts to default if needed
 
      implicit none
      integer levelmax,nbdim,nb(nbdim,2),branch(levelmax),mdllvl,
     *   nmapmoel,mapmoel(*),meno,maxmeno,moel(maxmeno),
     *   xpntr(2:levelmax),namepntr(levelmax),nxold(levelmax),maxxnode,
     *   nold,numdist,sumdim,distdim(numdist),dimused(sumdim),
     *   distyp(numdist),lamtyp(numdist),lamrow(numdist),
     *   distpos(numdist),neint,
     *   ifint(numdist),intpnts(sumdim),nfull,fulltype(nfull),
     *   modeltype,mtype,maxdraws,draws(maxdraws,numdist),draw(numdist),
     *   ndraw(numdist),drawvar,thisdraw,numterm,resdone,sparsity,
     *   maxref,resref(maxref),vdim,oldvdim,nec,ie,
     *   struc,iobs,id,ipt,jpt,level,varpos,distype,
     *   xdim,dist,dim,crossdim,i,n,l,p,q,vp,status,
     *   trm,absbranch,xpos,intvecsum
      real x(xdim),v6(*),xnode(maxxnode),outcome,refvar
      character oldname(*)*8,resname(sumdim)*12,distvar(numdist)*8,
     *   name*8,string*60,idstring*60
      logical iflam(numdist),integrated,iid
 
      iid=.false.
 
c     This term is a (integrated or non-integrated) residual:
c        4 = integrated residual
c        7 = non-integrated residual
      if(modeltype.eq.46.and.mtype.eq.6)then
c        Continuous branches of tobit models must always be non-integrated.
         integrated=.false.
         mapmoel(nmapmoel+1)=7
      else
         integrated=(moel(meno).eq.4)
         mapmoel(nmapmoel+1)=moel(meno)
      endif
 
c     DRAW={varname | expression}
      if(moel(meno+1).eq.1)then
         call evalexpr(1,maxxnode,xdim,moel(meno+2),nold,levelmax,
     *      mdllvl,nbdim,nb,branch,xpntr,nxold,namepntr,xnode,x,outcome,
     *      oldname,struc,iobs,id,ipt,jpt,status)
         if(status.gt.0)stop
         if(outcome.le.0..or.int(outcome).ne.outcome)goto 800
         drawvar=int(outcome)
      elseif(moel(meno+1).eq.2)then
c        DRAW was specified as DRAW=_LEVELn
c        Assign a unique branch number.  This number must be unique even
c        across levels and level 2 branches.  We need to store
c           level (<=levelmax)
c           ib2 (<=nb2)
c           absolute branch number within the current level 2 branch
c        Use negative numbers to prevent confusion with draw numbers
c        through a user-specified variable name.
c        Note that ib2=branch(2) and nb2=nb(2,1)
         level=moel(meno+2)
         if(level.eq.1)then
            drawvar=-1
         else
            n=absbranch(level,branch,nbdim,nb)
            drawvar=-(n*(nb(2,1)+1)*(levelmax+1)+
     *         branch(2)*(levelmax+1)+level)
         endif
      elseif(moel(meno+1).eq.3)then
c        DRAW was specified as DRAW=_iid; circumvent draw numbers
         iid=.true.
      elseif(moel(meno+1).eq.4)then
c        IID N(0,1) residual; treated entirely differently
         mapmoel(nmapmoel+2)=9999
         nmapmoel=nmapmoel+2
         meno=meno+2
         if(resdone.eq.-1)call bug(ipt,jpt)
         resdone=1
         goto 999
      else
         call bug(ipt,jpt)
      endif
 
      if(moel(meno+3).eq.1)then
c        Directly referenced:
         dist=moel(meno+4)
         dim=moel(meno+5)
         meno=meno+6
      elseif(moel(meno+3).eq.2)then
c        Indirect reference
         call evalexpr(1,maxxnode,xdim,moel(meno+4),nold,levelmax,
     *      mdllvl,nbdim,nb,branch,xpntr,nxold,namepntr,xnode,x,
     *      refvar,oldname,struc,iobs,id,ipt,jpt,status)
         if(status.gt.0)stop
         if(refvar.eq.0.)then
c           This term and everything interacted with it drops out.
            call dropblock(vdim,oldvdim,nmapmoel,mapmoel,
     *         meno,maxmeno,moel,ipt,jpt)
            goto 999
         elseif(refvar.ne.int(refvar))then
            goto 815
         endif
         call resref2def(int(refvar),maxref,resref,dist,dim)
         if(dist.eq.0)goto 810
         meno=meno+5
      else
         call bug(ipt,jpt)
      endif
      mapmoel(nmapmoel+2)=dist
      mapmoel(nmapmoel+3)=dim
c     Crossdim is dimension of current residual among all residuals,
c     regardless of whether the corresponding distributions are integrated
c     out.  Prepscratch compresses dimused, intpntr, intpnts.
      crossdim=intvecsum(distdim,dist-1)+dim
 
c     Absolute value to defuse ARMA time variable increase check.
      distype=abs(distyp(dist))
 
      if(integrated)then
c        Count number of modules with any integrated residual.
c        To prevent double counting, neint is negated for each module.
         if(neint.le.0)neint=-neint+1
c        Make sure number of points was specified.  If not, set to default.
         if(intpnts(crossdim).eq.0)then
c           Only set number of points for first observation.  Otherwise
c           write_scr writes other dimused than read_scr reads in.
c           sumintdim is re-calculated in prepscratch.
            if(iobs.gt.1)goto 880
            n=intvecsum(distdim,dist-1)+1
            if(distdim(dist).eq.1)intpnts(n)=12
            if(distdim(dist).eq.2)call putint(8,intpnts(n),2)
            if(distdim(dist).ge.3)call putint(6,intpnts(n),
     *         distdim(dist))
         endif
      else
c        Finite mixtures must be integrated out.
         if(mod(distype,10).eq.4)goto 820
      endif
 
c     What draw?  Has this value (in DRAW=varname) been encountered before?
c     If iid, always new draw.
      if(.not.iid)call intmatch(drawvar,draws(1,dist),
     *   ndraw(dist),thisdraw)
      if(iid.or.thisdraw.eq.0)then
c        IID draw, or the draw variable value has not been encountered
c        before, so it generates a new draw.
         thisdraw=ndraw(dist)+1
         ndraw(dist)=thisdraw
         if(thisdraw.gt.maxdraws)call
     *      maxexceed('maxdraws',thisdraw,maxdraws,ipt,jpt)
         if(iid)then
            draws(thisdraw,dist)=0
         else
            draws(thisdraw,dist)=drawvar
         endif
      endif
 
c     Try to gain efficiency by recognizing special structure of Lambda.
c     Matrix types lamtyp(numdist,6) can be:
c       -1 = this matrix is not used, i.e., there is no load matrix of this
c            distribution on this model type.  Readaml initializes all load
c            matrices to -1.
c        0 = generic
c        1 = identity (Only its dimension is stored.)  Careful: type 1 may
c            incorrectly be assigned; see note below.
c        2 = symmetric (but everything is stored.)  Not relevant for
c            load matrices.
c        3 = symmetric and block-diagonal (Only its first block, its
c            dimension, and number of blocks are stored.)  Not relevant for
c            load matrices.
c        4 = sparse in the sense that there is at most one nonzero
c            element in any one row.  (Only its nonzero elements
c            and their column position are stored.)
c     If the matrix is used, it is first assumed that the matrix is identity.
c     If this is falsified (because ie!=draw), it is assumed sparse.  If
c     this is suspected incorrect (because ie<draw), it is assumed generic.
c
c     Note: identity is assumed if all ie=draw.  However, it may be the case tha
c     ne>ndraw.  For example, Test\Misc\nested1a leads to the following (15,3) l
c     matrix for the first distribution:
c        ( 1 0 0 )
c        ( 0 1 0 )
c        ( 0 0 1 )
c        ( 0 0 0 )
c        ( 0 0 0 )
c            :
c        ( 0 0 0 )
c     Every time the current subroutine is called, ie=draw, but there are 12
c     outcomes without draw.  This erroneous assignment is corrected in figure_d
c     Similarly, Trouble\trouble10 generates the following load matrix on probit
c     residuals of the second distribution:
c        ( 1 0 0 )
c        ( 0 0 0 )
c     This matrix is incorrectly deemed identity; correct this in figure_data.
c     Prior to Version 2.08, the following load matrix was incorrectly deemed
c     identity:
c        ( 1 0 )
c        ( 0 0 )
c     See David Ribar's e-mail of 10/11/2004 and Trouble\ribar.
c
c     Key to catching these misclassifications is that the last encountered row
c     (last processed ie) is not equal to the dimension of the matrix (ndraw).
c
c     Load matrix types are determined observation-by-observation and (encoded b
c     codelamtyp) become part of the scratch data set for every observation.
c     This may seem wasteful but may save very substantially on scratch space.
      if(integrated)then
c        Efficiency gains are minimal or negative for integrated distributions,
c        so only non-integrated distributions are considered for special structu
         lamtyp(dist)=0
      elseif(sparsity.eq.0)then
c        If "option sparsity=no" is set, no matrix is ever assumed sparse.
c        Option sparsity=no allows proper execution if sparsity was ever
c        incorrectly diagnosed.
         lamtyp(dist)=0
      elseif(lamtyp(dist).eq.0)then
c        Already generic; no need to look for efficiency
         continue
      else
c        At this point, lamtyp is -1, 1 or 4.
         if(lamtyp(dist).eq.-1)then
c           If load matrix has not been seen before, flag its existence and assu
c           identity (if dim=1) or sparse (if dim>1):
            if(distdim(dist).eq.1.and.ie.eq.1)then
               lamtyp(dist)=1
            else
               lamtyp(dist)=4
            endif
c           If residual is loaded, do not allow identity; sparse is OK
            if(numterm.gt.0)lamtyp(dist)=4
         endif
c        At this point, lamtyp is 1 or 4.
         if(lamtyp(dist).eq.1)then
            if(ie.ne.thisdraw.or.numterm.gt.0)lamtyp(dist)=4
         endif
c        At this point, lamtyp is 1 or 4.
c        Identity matrices must have exactly one element per row;
c        sparse matrices must have at most one element per row.
c        Check this by tracking the previous assignment.
         if(lamtyp(dist).eq.1)then
c           If the previous nonzero row was before ie-1: sparse.
            if(lamrow(dist).lt.ie-1)lamtyp(dist)=4
         endif
c        If the previous nonzero row was also ie: generic.  (Could be
c        sparse if the same residual enters multiple times.  That is rare
c        and does not make sense; do not attempt efficiency.)
         if(lamrow(dist).eq.ie)lamtyp(dist)=0
         lamrow(dist)=ie
      endif
 
      mapmoel(nmapmoel+4)=thisdraw
      nmapmoel=nmapmoel+4
c     Array draw(numdist,ne) is used for integration purposes only.
c     Make sure that residuals of a certain distribution are either
c     all or none integrated out.  Also make sure no multiple draws
c     of an integrated distribution enter in continuous models.
c     Non-integrated residuals get a negative draw number, which is
c     set to zero in figure_data.
      if(integrated)then
         if(draw(dist).eq.0)draw(dist)=thisdraw
         if(draw(dist).ne.thisdraw)goto 830
         if(modeltype.eq.6.and.draw(dist).ne.1)goto 830
      else
         if(draw(dist).gt.0)goto 830
c        No safeguard against multiple non-integrated draws:
         draw(dist)=-thisdraw
      endif
c     Note that draws contains values of draw=varname, whereas draw
c     indicates which draw number enters.
 
c     For probit and continuous models, it is important to make sure
c     there is at least one non-integrated residual.
      if(.not.integrated)then
         if(resdone.eq.-1)call bug(ipt,jpt)
         resdone=1
      endif
 
c     Is this residual (and thus distribution) loaded by parameter
c     or regressor set?  This could not be determined in read_model*
c     because residuals may be referenced indirectly.  If it is loaded,
c     flag that derivatives wrt the load matrix must be computed.
      if(numterm.ge.1)iflam(dist)=.true.
 
c     Can't mix residuals and integrated residuals from same distribution
c     within model types.  This is relevant for probit and continuous
c     models only.  Integer array ifint indicates whether a distribution
c     is (1) integrated out or (-1) not.
      if(modeltype.eq.43.or.modeltype.eq.44.or.
     *   modeltype.eq.6.or.modeltype.eq.46.or.modeltype.eq.72)then
         if(integrated)then
            if(ifint(dist).eq.-1)goto 870
            ifint(dist)=1
         else
            if(ifint(dist).eq.1)goto 870
            ifint(dist)=-1
         endif
      endif
 
c     Keep track of which dimensions are actually integrated out in this
c     observation (in order to gain efficiency when integrating out
c     he distribution later).
      if(integrated)dimused(crossdim)=1
 
      if(mod(distype,10).eq.2.or.mod(distype,10).eq.3)then
c        C/ARMA residuals are only allowed in continuous models, may not
c        be integrated out, and must be drawn independently by draw=_iid.
         if(modeltype.ne.6.or.integrated.or..not.iid)goto 840
c        Place the absolute branch level and timevar values in v6.
c        Check that the time variable is an integer, except for AR(1)
c        and CAR(1) with phi>0.
c        Check that the time variable is strictly increasing within
c        the level in which autocorrelation exists is done in figdata6.
         l = distype/1000
         p = mod(distype,1000)/100
         q = mod(distype,100)/10
         if(l.lt.1)call bug(ipt,jpt)
         if(l.ge.mdllvl)goto 845
         call findvar(levelmax,levelmax,level,distvar(dist),oldname,
     *      namepntr,nxold,varpos,' ',-1,struc,ipt,jpt,status)
         if(status.eq.0.and.level.ne.mdllvl)goto 850
         if(status.gt.0)stop
         n=xpos(level,nbdim,nb,branch,xpntr,nxold,varpos)
         if(x(n).ne.int(x(n)))then
c           Non-integer valued time variables are only allowed for
c           AR(1) and CAR(1) processes with ardomain=(0,1).
            if(.not.(p.eq.1.and.q.eq.0.and.
     *         fulltype(distpos(dist)).eq.3))goto 860
         endif
c        The first nec positions in v6 are for the outcome; for each
c        C/ARMA distribution another 2*nec are reserved.
         vp=2*nec+ie
         do 100 i=1,dist-1
            if(mod(abs(distyp(i)),10).eq.2.or.
     *         mod(abs(distyp(i)),10).eq.3)vp=vp+2*nec
  100    continue
         v6(vp)=x(n)
      endif
 
      goto 999
 
  800 string=idstring(struc,id,iobs,mdllvl,branch)
      if(ipt.gt.0)then
         call wrapstring(8,0,72,0,1,'ERROR:  a residual draw '//
     *      'is either not positive or not integer-valued ('//
     *      string(1:trm(string))//'):')
         call writeexpr(111,8,maxxnode,xdim,levelmax,nbdim,
     *      -moel(meno+2),nb,branch,xpntr,1,nold,nxold,namepntr,
     *      -1,outcome,xnode,x,oldname,struc)
      endif
      if(jpt.gt.0)then
         call wrapstring(6,0,72,0,1,'ERROR:  a residual draw '//
     *      'is either not positive or not integer-valued ('//
     *      string(1:trm(string))//'):')
         call writeexpr(111,6,maxxnode,xdim,levelmax,nbdim,
     *      -moel(meno+2),nb,branch,xpntr,1,nold,nxold,namepntr,
     *      -1,outcome,xnode,x,oldname,struc)
      endif
      stop
  810 call norefmatch(1,struc,id,iobs,mdllvl,branch,ipt,jpt,maxxnode,
     *   xdim,levelmax,nbdim,nb,moel(meno+4),xpntr,nold,nxold,
     *   namepntr,refvar,xnode,x,oldname,'residual',
     *   'distribution dimension')
  815 call norefmatch(2,struc,id,iobs,mdllvl,branch,ipt,jpt,maxxnode,
     *   xdim,levelmax,nbdim,nb,moel(meno+4),xpntr,nold,nxold,
     *   namepntr,refvar,xnode,x,oldname,'residual',
     *   'distribution dimension')
  820 if(ipt.gt.0)write(8,821)
      if(jpt.gt.0)write(6,821)
  821 format(/,'ERROR:  finite mixture residuals must be integrated ',
     *   'out.')
      stop
  830 string=idstring(struc,id,iobs,mdllvl,branch)
      if(draw(dist)*thisdraw.lt.0)then
c        At least one integrated and one non-integrated residual from
c        the same distribution:
         if(ipt.gt.0)call wrapstring(8,0,72,0,1,'ERROR:  For one '//
     *      'single outcome, either all or none of the residuals '//
     *      'from any one distribution must be integrated out.  '//
     *      'Your model mixed at least one integrated and one non-'//
     *      'integrated residual ('//string(1:trm(string))//').')
         if(jpt.gt.0)call wrapstring(6,0,72,0,1,'ERROR:  For one '//
     *      'single outcome, either all or none of the residuals '//
     *      'from any one distribution must be integrated out.  '//
     *      'Your model mixed at least one integrated and one non-'//
     *      'integrated residual ('//string(1:trm(string))//').')
      elseif(integrated.and.modeltype.eq.6.and.draw(dist).ne.1)then
c        All continuous modules must have the same draw from integrated
c        distributions (if any).  See subroutine onecontin for an
c        explanation of the reasons herefor.  Since continuous modules
c        are done first, that draw can only be 1.
         if(ipt.gt.0)write(8,831)
         if(jpt.gt.0)write(6,831)
  831    format(/,'ERROR:  multiple draws of integrated residuals ',
     *      'are not allowed for',/,'continuous models.  Specify ',
     *      'your continuous models with non-integrated',/,
     *      'residuals instead.')
      else
c        Multiple draws from an integrated residual in one module:
         if(ipt.gt.0)call wrapstring(8,0,72,0,1,'ERROR:  multiple '//
     *      'draws of an integrated distribution are not allowed in '//
     *      'one single outcome.  This is violated in your model for '//
     *      string(1:trm(string))//'.')
         if(jpt.gt.0)call wrapstring(6,0,72,0,1,'ERROR:  multiple '//
     *      'draws of an integrated distribution are not allowed in '//
     *      'one single outcome.  This is violated in your model for '//
     *      string(1:trm(string))//'.')
      endif
      stop
  840 string=idstring(struc,id,iobs,mdllvl,branch)
      if(ipt.gt.0)then
         if(modeltype.ne.6)call wrapstring(8,0,72,0,1,'ERROR '//
     *      'encountered in a residual specification.  (Cumulative) '//
     *      'ARMA residuals are only allowed in continuous models.')
         if(integrated)call wrapstring(8,0,72,0,1,'ERROR '//
     *      'encountered in a residual specification ('//
     *      string(1:trm(string))//').  (Cumulative) ARMA residuals '//
     *      'may not be integrated out.')
         if(.not.iid)call wrapstring(8,0,72,0,1,'ERROR encountered '//
     *      'in a residual specification ('//string(1:trm(string))//
     *      ').  The innovation term of a (cumulative) ARMA residual '//
     *      'must be drawn independently:  RES(DRAW=_iid, ...)')
      endif
      if(jpt.gt.0)then
         if(modeltype.ne.6)call wrapstring(6,0,72,0,1,'ERROR '//
     *      'encountered in a residual specification.  (Cumulative) '//
     *      'ARMA residuals are only allowed in continuous models.')
         if(integrated)call wrapstring(6,0,72,0,1,'ERROR '//
     *      'encountered in a residual specification ('//
     *      string(1:trm(string))//').  (Cumulative) ARMA residuals '//
     *      'may not be integrated out.')
         if(.not.iid)call wrapstring(6,0,72,0,1,'ERROR encountered '//
     *      'in a residual specification ('//string(1:trm(string))//
     *      ').  The innovation term of a (cumulative) ARMA residual '//
     *      'must be drawn independently:  RES(DRAW=_iid, ...)')
      endif
      stop
  845 if(ipt.gt.0)call writea2ia(8,72,1,'ERROR: you specified a '//
     *   'continuous model with an ARMA/CAR residual.  The outcome '//
     *   'is at level ',mdllvl,' and the residual is defined to be '//
     *   'autocorrelated within level ',l,'.  This does not make '//
     *   'sense; the residual would be iid rather than ARMA/CAR.')
      if(jpt.gt.0)call writea2ia(6,72,1,'ERROR: you specified a '//
     *   'continuous model with an ARMA/CAR residual.  The outcome '//
     *   'is at level ',mdllvl,' and the residual is defined to be '//
     *   'autocorrelated within level ',l,'.  This does not make '//
     *   'sense; the residual would be iid rather than ARMA/CAR.')
      stop
  850 name=distvar(dist)
      if(ipt.gt.0)call writea2ia(8,72,1,'ERROR: an outcome (level ',
     *   mdllvl,') and a (cumulative) ARMA time variable '''//
     *   name(1:trm(name))//''' (level ',level,
     *   ') are not at the same level.')
      if(jpt.gt.0)call writea2ia(6,72,1,'ERROR: an outcome (level ',
     *   mdllvl,') and a (cumulative) ARMA time variable '''//
     *   name(1:trm(name))//''' (level ',level,
     *   ') are not at the same level.')
      stop
  860 name=distvar(dist)
      string=idstring(struc,id,iobs,mdllvl,branch)
      if(ipt.gt.0)then
         call wrapstring(8,0,72,0,0,'ERROR:  (cumulative) ARMA time '//
     *      'variable '''//name(1:trm(name))//''' is not integer-'//
     *      'valued ('//string(1:trm(string))//'):')
         call writedble(8,dble(x(n)),2,-8,0,1)
         write(8,861)
      endif
      if(jpt.gt.0)then
         call wrapstring(6,0,72,0,0,'ERROR:  (cumulative) ARMA time '//
     *      'variable '''//name(1:trm(name))//''' is not integer-'//
     *      'valued ('//string(1:trm(string))//'):')
         call writedble(6,dble(x(n)),2,-8,0,1)
         write(6,861)
      endif
  861 format(/,'Note:  AR(1) and CAR(1) time variables need not be ',
     *   'integer-valued,',/,'provided that you restrict the ',
     *   'autoregressive coefficient to be',/,'positive by ',
     *   'adding "ARDOMAIN=(0,1)" to the distribution''s',/,
     *   'definition statements.')
      stop
  870 if(ipt.gt.0)write(8,871)
      if(jpt.gt.0)write(6,871)
  871 format(/,'ERROR: either all or none of the residuals from an ',
     *   'integrated',/,'distribution must be integrated within ',
     *   'model type.')
      stop
  880 string=idstring(struc,id,iobs,mdllvl,branch)
      name=resname(crossdim)
      if(ipt.gt.0)call wrapstring(8,0,72,0,1,'ERROR:  a model '//
     *   'specifies that residual '''//name(1:trm(name))//
     *   ''' must be integrated out ('//string(1:trm(string))//
     *   ').  You must specify "number of integration points" in '//
     *   'the definition statement of the corresponding distribution.')
      if(jpt.gt.0)call wrapstring(6,0,72,0,1,'ERROR:  a model '//
     *   'specifies that residual '''//name(1:trm(name))//
     *   ''' must be integrated out ('//string(1:trm(string))//
     *   ').  You must specify "number of integration points" in '//
     *   'the definition statement of the corresponding distribution.')
      stop
  999 return
      end
