      subroutine figure_tau(levelmax,mdllvl,modeltype,nbdim,nb,branch,
     *   nfull,numpar,maxref,moel,meno,maxmeno,mapmoel,nmapmoel,
     *   namepntr,nxold,nold,xpntr,xdim,parref,parpos,pardim,fulltype,
     *   maxxnode,x,v,xnode,outcome,outcomes,lowerlimit,upperlimit,vdim,
     *   iiffull,iftau,oldname,parname,struc,iobs,id,ipt,jpt)
 
c     Subroutine to figure out which parameters (or data) serve as
c     thresholds (for an ordered probit model).
c
c     (1) ordered probit/logit: outcomes are two categorical variables,
c         thresholds is parameter set:
c         moel(+1)    -61
c         moel(+2)    position in xnode for two outcome variables
c         moel(+3)    1 direct threshold vector reference
c                     3 indirect reference
c         moel(+4)    vector number if moel(+3)=1
c                     reference expression if moel(+3)=3
c     (2) ordered probit/logit: thresholds in data
c         moel(+1) = -62
c         moel(+2) = variable/expression for first threshold;
c                    -Infinity; second threshold; +Infinity
c     (3) simple probit/logit: outcome is binary variable,
c         threshold is parameter
c         moel(+1)    -63
c         moel(+2)    position in xnode for outcome variable
c         moel(+3)    1 direct threshold vector reference
c                     3 indirect reference
c         moel(+4)    vector number if moel(+3)=1
c                     reference expression if moel(+3)=3
c     (4) simple probit/logit: outcome is binary variable,
c         threshold is assumed fixed at zero
c         moel(+1)    -64
c         moel(+2)    position in xnode for outcome variable
c     (5) Tobit: figdata4 (calling filtertobit) already determined that
c         this outcome is a probit outcome.  Treat as ordered probit
c         with thresholds in data.
c
c     Upon output, mapmoel=-61 if parameters indicate the thresholds
c     and -62 if the thresholds are (placed in) data.
c
c     Constantijn Panis, 13 November 2002
c     Last change: Tobit changes
 
      implicit none
      integer levelmax,mdllvl,modeltype,nbdim,nb(nbdim,2),
     *   branch(levelmax),iiffull(*),nfull,numpar,maxref,meno,maxmeno,
     *   moel(maxmeno),mapmoel(*),nmapmoel,namepntr(levelmax),
     *   nxold(levelmax),nold,xpntr(2:levelmax),xdim,parref(maxref),
     *   parpos(numpar),status,pardim(numpar),fulltype(nfull),maxxnode,
     *   vdim,struc,iobs,id,savemeno,dim,ref,pos,out1,out2,termtype,
     *   ipt,jpt,xnodepntr,trm
      real x(xdim),v(*),xnode(maxxnode),outcome,outcomes(4),refvar,
     *   lowerlimit,upperlimit,realinf
      logical iftau
      character oldname(*)*8,parname(numpar)*12,
     *   string*60,idstring*60
 
      status=0
      if(modeltype.ne.46)then
         termtype=moel(meno)
         meno=meno+1
c        The outcome starts at savemeno:
         savemeno=meno
      else
c        Tobit (outcome and limit(s) already determined):
         termtype=-46
      endif
 
      if(termtype.eq.-61)then
c     (1) ordered probit/logit: outcomes are two categorical variables,
c         thresholds is parameter set:
c         moel(+1)    -61
c         moel(+2)    position in xnode for two outcome variables
c         moel(+3)    1 direct threshold vector reference
c                     3 indirect reference
c         moel(+4)    vector number if moel(+3)=1
c                     reference expression if moel(+3)=3
c        What are the outcome variables?
         call evalexpr(2,maxxnode,xdim,moel(meno),nold,levelmax,
     *      mdllvl,nbdim,nb,branch,xpntr,nxold,namepntr,xnode,x,
     *      outcomes,oldname,struc,iobs,id,ipt,jpt,status)
         if(status.gt.0)stop
         out1=int(outcomes(1))
         out2=int(outcomes(2))
c        What threshold vector applies?
         meno=meno+1
         if(moel(meno).eq.1)then
c           Direct reference:
            ref=moel(meno+1)
         elseif(moel(meno).eq.3)then
            call evalexpr(1,maxxnode,xdim,moel(meno+1),nold,levelmax,
     *         mdllvl,nbdim,nb,branch,xpntr,nxold,namepntr,xnode,x,
     *         refvar,oldname,struc,iobs,id,ipt,jpt,status)
            if(status.gt.0)stop
c           refvar may never be 0 here
            if(refvar.eq.0.)goto 800
            if(int(refvar).ne.refvar)goto 850
            call ref2def(int(refvar),maxref,parref,ref,status)
            if(status.gt.0)goto 800
            if(pardim(ref).gt.nfull)goto 840
         else
            call bug(ipt,jpt)
         endif
         if(out1.lt.0.or.out1.ge.out2.or.out2.gt.pardim(ref)+1.or.
     *      out1.ne.outcomes(1).or.out2.ne.outcomes(2))goto 810
         if(fulltype(parpos(ref)).ne.41.and.pardim(ref).gt.1)goto 870
         meno=meno+2
 
c        ref is definition rank number of the vector that serves
c        as thresholds.  Its parameters start at parpos(ref), and there
c        are ntau=pardim(ref) thresholds, i.e., pardim(ref)+1 categories.
c        Figure out which thresholds apply:
c        out1 and out2 contain the outcome; if out1=0, the lower bound
c        is -infinity; if out2=ntau+1, the upper bound is +infinity.
c        All information is stored in mapmoel(), which points at the
c        position in fullpar that contains the appropriate bounds.  If
c        mapmoel(2)=0, the lower bound is -infinity; if mapmoel(3)=fullmax+1,
c        the upper bound is +infinity.  Note that mapmoel(2) and mapmoel(3)
c        may differ by more than one, reflecting the possibility that
c        an outcome lies in one of several contiguous categories.
         mapmoel(nmapmoel+1)=-61
         if(out1.eq.0)then
            mapmoel(nmapmoel+2)=0
         else
            pos = parpos(ref)-1+out1
            mapmoel(nmapmoel+2)=pos
            iftau=iftau.or.(iiffull(pos).gt.0)
         endif
         if(out2.gt.pardim(ref))then
            if(out1.eq.0)goto 860
            mapmoel(nmapmoel+3)=nfull+1
         else
            pos = parpos(ref)-1+out2
            mapmoel(nmapmoel+3)=pos
            iftau=iftau.or.(iiffull(pos).gt.0)
         endif
         nmapmoel=nmapmoel+3
      elseif(termtype.eq.-62)then
c        (2) ordered probit/logit: thresholds in data
c            moel(+1) = -62
c            moel(+2) = variable/expression for first threshold;
c                       -Infinity; second threshold; +Infinity
c        Thresholds (and thus outcome) are in data as variables;
c        copy into v.  Check for +/- Infinity and make sure the thresholds
c        are strictly ordered.
         nmapmoel=nmapmoel+1
         mapmoel(nmapmoel)=-62
         call evalexpr(4,maxxnode,xdim,moel(meno),nold,levelmax,
     *      mdllvl,nbdim,nb,branch,xpntr,nxold,namepntr,xnode,x,
     *      outcomes,oldname,struc,iobs,id,ipt,jpt,status)
         if(status.gt.0)stop
         if(outcomes(1).eq.outcomes(2))then
c           First threshold is -Infinity
            if(outcomes(3).eq.outcomes(4))goto 830
            v(vdim+1)=-realinf()
            v(vdim+2)=outcomes(3)
         elseif(outcomes(3).eq.outcomes(4))then
c           Second threshold is +Infinity
            v(vdim+1)=outcomes(1)
            v(vdim+2)=realinf()
         else
            if(outcomes(1).ge.outcomes(3))goto 830
            v(vdim+1)=outcomes(1)
            v(vdim+2)=outcomes(3)
         endif
c        Accommodate tabulations:
         outcomes(2)=outcomes(3)
         meno=meno+1
         vdim=vdim+2
      elseif(termtype.eq.-63)then
c     (3) simple probit/logit: outcome is binary variable,
c         threshold is parameter
c         moel(+1)    -63
c         moel(+2)    position in xnode for outcome variable
c         moel(+3)    1 direct threshold vector reference
c                     3 indirect reference
c         moel(+4)    vector number if moel(+3)=1
c                     reference expression if moel(+3)=3
c        What is the outcome variabless?
         call evalexpr(1,maxxnode,xdim,moel(meno),nold,levelmax,
     *      mdllvl,nbdim,nb,branch,xpntr,nxold,namepntr,xnode,x,
     *      outcome,oldname,struc,iobs,id,ipt,jpt,status)
         if(status.gt.0)stop
c        What threshold vector applies?
         meno=meno+1
         if(moel(meno).eq.1)then
c           Direct reference:
            ref=moel(meno+1)
         elseif(moel(meno).eq.3)then
            call evalexpr(1,maxxnode,xdim,moel(meno+1),nold,levelmax,
     *         mdllvl,nbdim,nb,branch,xpntr,nxold,namepntr,xnode,x,
     *         refvar,oldname,struc,iobs,id,ipt,jpt,status)
            if(status.gt.0)stop
c           refvar may never be 0 here
            if(refvar.eq.0.)goto 800
            if(int(refvar).ne.refvar)goto 850
            call ref2def(int(refvar),maxref,parref,ref,status)
            if(status.gt.0)goto 800
            if(pardim(ref).gt.nfull)goto 840
         else
            call bug(ipt,jpt)
         endif
         meno=meno+2
         mapmoel(nmapmoel+1)=-61
         if(outcome.eq.0.)then
            mapmoel(nmapmoel+2)=0
            mapmoel(nmapmoel+3)=parpos(ref)
         elseif(outcome.eq.1.)then
            mapmoel(nmapmoel+2)=parpos(ref)
            mapmoel(nmapmoel+3)=nfull+1
         else
            goto 820
         endif
         iftau=iftau.or.(iiffull(parpos(ref)).gt.0)
         nmapmoel=nmapmoel+3
      elseif(termtype.eq.-64)then
c     (4) simple probit/logit: outcome is binary variable,
c         threshold is assumed fixed at zero
c         moel(+1)    -64
c         moel(+2)    position in xnode for outcome variable
         nmapmoel=nmapmoel+1
         mapmoel(nmapmoel)=-62
c        One outcome, possibly expression.
         call evalexpr(1,maxxnode,xdim,moel(meno),nold,levelmax,mdllvl,
     *      nbdim,nb,branch,xpntr,nxold,namepntr,xnode,x,outcome,
     *      oldname,struc,iobs,id,ipt,jpt,status)
         if(status.gt.0)stop
         if(outcome.eq.0.)then
            v(vdim+1)=-realinf()
            v(vdim+2)=0.
         elseif(outcome.eq.1.)then
            v(vdim+1)=0.
            v(vdim+2)=realinf()
         else
            goto 820
         endif
         meno=meno+1
         vdim=vdim+2
      elseif(modeltype.eq.46)then
c        Tobit model:  filtertobit already determined outcome and
c        lowerlimit, upperlimit.
c        Treat as ordered probit/logit with thresholds in data
         nmapmoel=nmapmoel+1
         mapmoel(nmapmoel)=-62
         if(outcome.le.lowerlimit)then
            v(vdim+1)=-realinf()
            v(vdim+2)=lowerlimit
c           For summary statistics:
            if(outcome.lt.lowerlimit)then
               outcome=1.
            else
               outcome=2.
            endif
         elseif(outcome.ge.upperlimit)then
            v(vdim+1)=upperlimit
            v(vdim+2)=realinf()
            if(outcome.gt.upperlimit)then
               outcome=5.
            else
               outcome=4.
            endif
         else
            call bug(ipt,jpt)
         endif
         vdim=vdim+2
      else
         call bug(ipt,jpt)
      endif
      goto 999
 
  800 call norefmatch(1,struc,id,iobs,mdllvl,branch,ipt,jpt,maxxnode,
     *   xdim,levelmax,nbdim,nb,moel(meno+1),xpntr,nold,nxold,
     *   namepntr,refvar,xnode,x,oldname,'parameter or vector',
     *   'parameter or vector')
  810 string=idstring(struc,id,iobs,mdllvl,branch)
      dim=pardim(ref)
      if(ipt.gt.0)then
         call writea2ia(8,72,1,'ERROR in the categorical outcome '//
     *      'variables of an ordered logit/probit model ('//
     *      string(1:trm(string))//').  The threshold vector '//
     *      'has dimension ',dim,', so outcome variables must '//
     *      'be between 0 and ',dim+1,' and integer-valued.  The '//
     *      'outcome variables are:')
         xnodepntr=moel(savemeno)
         call writeexpr(111,8,maxxnode,xdim,levelmax,nbdim,
     *      xnodepntr,nb,branch,xpntr,1,nold,nxold,namepntr,-1,
     *      outcomes(1),xnode,x,oldname,struc)
         call writeexpr(111,8,maxxnode,xdim,levelmax,nbdim,
     *      xnodepntr,nb,branch,xpntr,1,nold,nxold,namepntr,-1,
     *      outcomes(2),xnode,x,oldname,struc)
      endif
      if(jpt.gt.0)then
         call writea2ia(6,72,1,'ERROR in the categorical outcome '//
     *      'variables of an ordered logit/probit model ('//
     *      string(1:trm(string))//').  The threshold vector '//
     *      'has dimension ',dim,', so outcome variables must '//
     *      'be between 0 and ',dim+1,' and integer-valued.  The '//
     *      'outcome variables are:')
         xnodepntr=moel(savemeno)
         call writeexpr(111,6,maxxnode,xdim,levelmax,nbdim,
     *      xnodepntr,nb,branch,xpntr,1,nold,nxold,namepntr,-1,
     *      outcomes(1),xnode,x,oldname,struc)
         call writeexpr(111,6,maxxnode,xdim,levelmax,nbdim,
     *      xnodepntr,nb,branch,xpntr,1,nold,nxold,namepntr,-1,
     *      outcomes(2),xnode,x,oldname,struc)
      endif
      stop
  820 string=idstring(struc,id,iobs,mdllvl,branch)
      if(ipt.gt.0)then
         call wrapstring(8,0,72,0,1,'ERROR in the outcome variable '//
     *      'of a logit/probit model ('//string(1:trm(string))//
     *      ').  It must be either zero or one:')
         call writeexpr(111,8,maxxnode,xdim,levelmax,nbdim,
     *      -moel(savemeno),nb,branch,xpntr,1,nold,nxold,namepntr,-1,
     *      outcome,xnode,x,oldname,struc)
      endif
      if(ipt.gt.0)then
         call wrapstring(6,0,72,0,1,'ERROR in the outcome variable '//
     *      'of a logit/probit model ('//string(1:trm(string))//
     *      ').  It must be either zero or one:')
         call writeexpr(111,6,maxxnode,xdim,levelmax,nbdim,
     *      -moel(savemeno),nb,branch,xpntr,1,nold,nxold,namepntr,-1,
     *      outcome,xnode,x,oldname,struc)
      endif
      stop
  830 string=idstring(struc,id,iobs,mdllvl,branch)
      if(ipt.gt.0)then
         call wrapstring(8,0,72,0,1,'ERROR:  illegitimate threshold '//
     *      'values in an ordered logit/probit model ('//
     *      string(1:trm(string))//'):')
         xnodepntr=moel(savemeno)
         call writeexpr(111,8,maxxnode,xdim,levelmax,nbdim,
     *      xnodepntr,nb,branch,xpntr,1,nold,nxold,namepntr,-1,
     *      outcomes(1),xnode,x,oldname,struc)
         call writeexpr(100,-1,maxxnode,xdim,levelmax,nbdim,
     *      xnodepntr,nb,branch,xpntr,1,nold,nxold,namepntr,-1,
     *      outcomes(2),xnode,x,oldname,struc)
         call writeexpr(111,8,maxxnode,xdim,levelmax,nbdim,
     *      xnodepntr,nb,branch,xpntr,1,nold,nxold,namepntr,-1,
     *      outcomes(3),xnode,x,oldname,struc)
      endif
      if(jpt.gt.0)then
         call wrapstring(6,0,72,0,1,'ERROR:  illegitimate threshold '//
     *      'values in an ordered logit/probit model ('//
     *      string(1:trm(string))//'):')
         xnodepntr=moel(savemeno)
         call writeexpr(111,6,maxxnode,xdim,levelmax,nbdim,
     *      xnodepntr,nb,branch,xpntr,1,nold,nxold,namepntr,-1,
     *      outcomes(1),xnode,x,oldname,struc)
         call writeexpr(100,-1,maxxnode,xdim,levelmax,nbdim,
     *      xnodepntr,nb,branch,xpntr,1,nold,nxold,namepntr,-1,
     *      outcomes(2),xnode,x,oldname,struc)
         call writeexpr(111,6,maxxnode,xdim,levelmax,nbdim,
     *      xnodepntr,nb,branch,xpntr,1,nold,nxold,namepntr,-1,
     *      outcomes(3),xnode,x,oldname,struc)
      endif
      stop
  840 string=idstring(struc,id,iobs,mdllvl,branch)
      if(ipt.gt.0)then
         call wrapstring(8,0,72,0,1,'ERROR:  the indirect reference '//
     *      'of thresholds in an (ordered) logit/probit model '//
     *      'resolves to a matrix instead of to a vector or '//
     *      'parameter ('//string(1:trm(string))//
     *      ').  The reference variable is ')
         call writeexpr(111,8,maxxnode,xdim,levelmax,nbdim,
     *      -moel(meno+1),nb,branch,xpntr,1,nold,nxold,namepntr,
     *      -1,outcome,xnode,x,oldname,struc)
      endif
      if(jpt.gt.0)then
         call wrapstring(6,0,72,0,1,'ERROR:  the indirect reference '//
     *      'of thresholds in an (ordered) logit/probit model '//
     *      'resolves to a matrix instead of to a vector or '//
     *      'parameter ('//string(1:trm(string))//
     *      ').  The reference variable is ')
         call writeexpr(111,6,maxxnode,xdim,levelmax,nbdim,
     *      -moel(meno+1),nb,branch,xpntr,1,nold,nxold,namepntr,
     *      -1,outcome,xnode,x,oldname,struc)
      endif
      stop
  850 call norefmatch(2,struc,id,iobs,mdllvl,branch,ipt,jpt,maxxnode,
     *   xdim,levelmax,nbdim,nb,moel(meno+1),xpntr,nold,nxold,
     *   namepntr,refvar,xnode,x,oldname,'threshold',
     *   'parameter or vector')
  860 string=idstring(struc,id,iobs,mdllvl,branch)
      if(ipt.gt.0)call writeaia(8,72,1,'ERROR: ordered probit/logit '//
     *   'outcomes are 0 and ',out2,', i.e., this record contains no '//
     *   'information ('//string(1:trm(string))//').')
      if(jpt.gt.0)call writeaia(6,72,1,'ERROR: ordered probit/logit '//
     *   'outcomes are 0 and ',out2,', i.e., this record contains no '//
     *   'information ('//string(1:trm(string))//').')
      stop
  870 if(ipt.gt.0)write(8,871)parname(ref)(1:trm(parname(ref)))
      if(jpt.gt.0)write(6,871)parname(ref)(1:trm(parname(ref)))
  871 format(/,'ERROR:  thresholds vector ',a,' in an ordered ',
     *   'logit/probit model',/,'is not strictly increasing.  ',
     *   'Specify option "INCREASING;" in the',/,
     *   'vector definition.')
      stop
  999 return
      end
