      integer function flen2(x,n)
 
c     Integer function to determine the number of digits after the
c     decimal point that would look good for a double precision number.
c     For example, 6.603417E+01 looks best as 66.03417 if only 8
c     spaces are available.  The format is f8.5; this function
c     returns the 5.  Suppose one wants to write x=6.603417E+01 in
c     12 spaces, of 4 blanks and 8 for x.  Usage:
c
c     x=6.603417E+01
c     write(*,1)x
c   1 format(.....,f12.<flen2(x,8)>,........)
c
c     WARNING: This format statement requires an f77 extension.
c
c     The second argument thus tells the number of spaces that is actually
c     available for x.  Important: if x<0, the minus sign will take an
c     additional space!  Writing 66.03417 for positive numbers and
c     -66.0342 for negative wouldn't look good.  It is the user's
c     responsibility to set aside sufficient spaces.  I.e., in
c     format f<n1>,<flen2(x,n2)>, make sure than n1>=n2+1 if x may
c     ever be negative.
c
c     For x>1, the number of significant digits is n-1.
c
c     Constantijn Panis,  23 June 1993
 
      implicit none
      double precision x,absx
      integer n,dlen
 
      absx=abs(x)
      if(absx.ge.1.d0)then
         flen2 = n-1-dlen(absx)
c        If x was NaN or too large, problems may arise:
         flen2 = min(n-2,flen2)
      else
         flen2 = n-2
      endif
 
      return
      end
