      subroutine getoper(unit,line,il,i1,i2,oper,x,nest,ipt,jpt)
 
c     Subroutine to read in next operand, variable name, or number.
c     Integer oper indicates what was encountered:
c
c     <0 = error
c      0 = ;
c      1 = real number (stored in x)
c     10 = variable name (stored in line(i1:i2))
c     11 = uniform()
c     20 = (        (nest is incremented)
c     21 = exp(     (nest is incremented)
c     22 = log(     (nest is incremented)
c     23 = abs(     (nest is incremented)
c     24 = sqrt(    (nest is incremented)
c     25 = dummy(   (nest is incremented)
c     26 = spline(  (nest is incremented)
c     28 = min(     (nest is incremented)
c     29 = max(     (nest is incremented)
c     30 = int(     (nest is incremented)
c     39 = )        (nest is decremented)
c     40 = ^
c     41 = *
c     42 = /
c     43 = +
c     44 = -
c     50 = ==
c     51 = <
c     52 = <=
c     53 = >
c     54 = >=
c     55 = !=
c     60 = and
c     61 = or
c     62 = not
c     70 = ,
c     71 = node=  or  nodes=
c
c     Other relevant "operators," not used here:
c     80 = begin of an array (formed by spline)
c     81 = end of an array
c
c     IMPORTANT:  any changes to this routine must be matched by
c           changes in evalexpr and writeexpr, and vice versa.
c
c     Constantijn Panis, 8 October 2002
c     Last change: int() and uniform() added
 
      implicit none
      real x
      integer unit,il,i1,i2,oper,nest,ipt,jpt,status
      character line*128,name*8
      logical smtxt
 
      if(smtxt(unit,line,il,';'))then
         oper=0
         goto 999
      elseif(smtxt(unit,line,il,'('))then
         nest=nest+1
         oper=20
      elseif(smtxt(unit,line,il,'EXP('))then
         nest=nest+1
         oper=21
      elseif(smtxt(unit,line,il,'LOG(').or.
     *   smtxt(unit,line,il,'LN('))then
         nest=nest+1
         oper=22
      elseif(smtxt(unit,line,il,'ABS('))then
         nest=nest+1
         oper=23
      elseif(smtxt(unit,line,il,'SQRT('))then
         nest=nest+1
         oper=24
      elseif(smtxt(unit,line,il,'DUMMY('))then
         nest=nest+1
         oper=25
      elseif(smtxt(unit,line,il,'SPLINE('))then
         if(smtxt(unit,line,il,'VAR='))continue
         nest=nest+1
         oper=26
      elseif(smtxt(unit,line,il,'MIN('))then
         nest=nest+1
         oper=28
      elseif(smtxt(unit,line,il,'MAX('))then
         nest=nest+1
         oper=29
      elseif(smtxt(unit,line,il,'INT('))then
         nest=nest+1
         oper=30
      elseif(smtxt(unit,line,il,')'))then
         nest=nest-1
         oper=39
      elseif(smtxt(unit,line,il,'^'))then
         oper=40
      elseif(smtxt(unit,line,il,'*'))then
         oper=41
         if(smtxt(unit,line,il,'*'))goto 800
      elseif(smtxt(unit,line,il,'/'))then
         oper=42
      elseif(smtxt(unit,line,il,'+'))then
         oper=43
         if(smtxt(unit,line,il,'+'))goto 800
         if(smtxt(unit,line,il,'-'))oper=44
      elseif(smtxt(unit,line,il,'-'))then
         oper=44
         if(smtxt(unit,line,il,'-'))goto 800
      elseif(smtxt(unit,line,il,'='))then
         if(.not.smtxt(unit,line,il,'='))goto 810
         oper=50
      elseif(smtxt(unit,line,il,'<'))then
         oper=51
         if(smtxt(unit,line,il,'='))oper=52
      elseif(smtxt(unit,line,il,'>'))then
         oper=53
         if(smtxt(unit,line,il,'='))oper=54
      elseif(smtxt(unit,line,il,'!='))then
         oper=55
      elseif(smtxt(unit,line,il,'~='))then
         oper=55
      elseif(smtxt(unit,line,il,'&'))then
         if(smtxt(unit,line,il,'&'))continue
         oper=60
      elseif(smtxt(unit,line,il,'|'))then
         if(smtxt(unit,line,il,'|'))continue
         oper=61
      elseif(smtxt(unit,line,il,','))then
         oper=70
      else
         call getname(4,line,il,i1,i2)
         if(i2.lt.i1)goto 110
         if(i2-i1.gt.7)goto 820
         call uppercase(line(i1:i2),name)
         if(name.eq.'AND')then
            oper=60
         elseif(name.eq.'OR')then
            oper=61
         elseif(name.eq.'NOT')then
            oper=62
         elseif(name.eq.'NODE'.or.name.eq.'NODES')then
            if(.not.smtxt(unit,line,il,'='))goto 830
c           May be used in spline(var=xyz, nodes=...)
            oper=71
         elseif(name.eq.'DEFINE'.or.name.eq.'LOGIT'.or.name.eq.'PROBIT'
     *      .or.name.eq.'ORDERED'.or.name.eq.'HAZARD'
     *      .or.name.eq.'BINOMIAL'.or.name.eq.'NEGATIVE'
     *      .or.name.eq.'OPTION')then
c           Reserved names; user probably forgot semicolon
            goto 830
         elseif(name.eq.'UNIFORM')then
            oper=11
            if(.not.smtxt(unit,line,il,'()'))goto 840
         else
c           Variable name:
            oper=10
         endif
         goto 999
  110    call getreal(4,x,line,il,status)
         if(status.gt.0)goto 820
         oper=1
         goto 999
      endif
      goto 999
 
c     ** or ++ or -- encountered
  800 il=il-2
      if(ipt.gt.0)then
         call caretline(8,line,il)
         if(oper.eq.41)write(8,801)'**'
         if(oper.eq.41)write(8,802)
         if(oper.eq.43)write(8,801)'++'
         if(oper.eq.44)write(8,801)'--'
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         if(oper.eq.41)write(8,801)'**'
         if(oper.eq.41)write(8,802)
         if(oper.eq.43)write(8,801)'++'
         if(oper.eq.44)write(8,801)'--'
      endif
  801 format('ERROR:  ',a,' is not a valid operand.')
  802 format('(Note that powers are denoted by ^.)')
      stop
c     Single '='; should be ==
  810 il=il-1
      if(ipt.gt.0)then
         call caretline(8,line,il)
         write(8,811)
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         write(6,811)
      endif
  811 format('ERROR:  equality conditions require double equality ',
     *   'signs (==)')
      stop
c     Too large a name and/or cannot be variable name or number
  820 il=il-(i2-i1+1)
      if(ipt.gt.0)then
         call caretline(8,line,il)
         write(8,821)
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         write(6,821)
      endif
  821 format('ERROR:  unexpected symbol or string.')
      stop
c     Reserved names; user probably forgot semicolon
  830 il=il-(i2-i1+1)
      if(ipt.gt.0)then
         call caretline(8,line,il)
         write(8,831)
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         write(6,831)
      endif
  831 format('ERROR:  invalid variable name or expression.  You ',
     *   'probably forgot',/,'a semicolon.')
      stop
  840 il=il-(i2-i1+1)
      if(ipt.gt.0)then
         call caretline(8,line,il)
         write(8,841)
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         write(6,841)
      endif
  841 format(/,'ERROR: expected "uniform()".')
      stop
 
  999 return
      end
 
 
 
