      subroutine getstring(unit,line,il,i1,i2)
 
c     Subroutine to determine begin and end of a string.
c
c     The string may be delimited by a single or double quote.  In that
c     case, i1 immediately follows the opening quote, i2 immediately
c     precedes the closing quote, and il immediately follows the closing
c     quote.
c
c     If the string is not delimited by a quote, i1 is the first nonwhite
c     (non-space, non-tab) character; i2 is the last position before a
c     semicolon or closing white character; il is i2+1.
c
c     If the end of a line is encountered before the first character of string
c     is encountered, the next line will automatically be read from unit.
c     Strings are _not_ allowed to wrap around lines.
c
c     Upon entering, il is the first position to be checked; upon
c     return, il is the position immediately following the word.
c     i1 and i2 are beginning and end positions of the string.
c     If the first nonwhite character is a semicolon, the semicolon is
c     considered the string so that line(i1:i2)=';'
c
c     if i1=0, i2=-1, there was an error.  Either the string started
c     with a quote but did not end in a quote, or there was a read
c     error, or the file was empty after line(il:il).
c
c     Constantijn Panis,  3 January 1998
c     Last change: no more error return
 
      implicit none
      integer unit,il,i1,i2
      character line*128,c*1,ht*1,sp*1,sq*1,dq*1,sc*1,ascii*1
 
c     ht is horizontal tab; sp is space; sq is single quote; dq is double
c     quote; sc is semicolon.
      ht=ascii(9)
      sp=ascii(32)
      dq=ascii(34)
      sq=ascii(39)
      sc=ascii(59)
 
      if(il.gt.80)then
c        Read next line.
         read(unit,'(a80)',end=800,err=800)line
         il=1
      endif
 
c     skip blanks and tabs:
  110 c=line(il:il)
      if(c.eq.sc)goto 210
      if(c.eq.sq.or.c.eq.dq)goto 120
      if(c.ne.sp.and.c.ne.ht)goto 130
      il=il+1
      if(il.gt.80)then
c        until position 80 all white characters; read next line.
         read(unit,'(a80)',end=800,err=800)line
         il=1
      endif
      goto 110
 
c     Position il is a single or double quote.  Find the closing quote:
  120 i1=il+1
      call char1match(c,line(i1:i1),80-il,i2)
      if(i2.eq.0)goto 800
      i2=i1+i2-2
      il=i2+2
      goto 999
 
c     Position il is nonwhite.  String starts here.
  130 i1=il
c     Scan next positions for delimiter (space, tab, semicolon):
      il=il+1
  140 if(il.eq.81)goto 800
      c=line(il:il)
      if(c.eq.sp.or.c.eq.ht.or.c.eq.sc)goto 220
      il=il+1
      goto 140
 
c     The first nonwhite character is a semicolon
  210 i1=il
      i2=il
      il=il+1
      goto 999
 
c     Delimiter encountered:
  220 i2=il-1
      goto 999
 
c     il>80 or Error or end-of-file or no closing quote
  800 i1=0
      i2=-1
      goto 999
  999 return
      end
 
