      double precision function ginnerp(n,jsa,jsb,a,b)
 
c     Double precision function to compute a generalized innerproduct.
c     If the two vectors were simple vectors, double precision function
c     innerp could be used; ginnerp is useful when the innerproduct of
c     row vectors (stored in a matrix) needs to be taken.  Arguments:
c     n     [in]  number of elements over which innerproduct is taken
c     jsa   [in]  distance between storage locations of subsequent
c                 elements of first vector
c     jsb   [in]  idem for second vector
c     a     [in]  array in which first vector is stored
c     b     [in]  array in which second vector is stored
c
c     Examples of usage:
c     (1) the innerproduct of row i of matrix x(k1,k2) and row j of
c         matrix y(k3,k2):
c            x = ginnerp(k2,k1,k3,x(i,1),y(j,1))
c     (2) the innerproduct of row i of matrix x(k1,k2) and column j
c         of matrix y(k2,k3):
c            x = ginnerp(k2,k1,1,x(i,1),y(1,j))
c     (3) the innerproduct of column i of matrix x(k1,k2) and column j
c         of matrix y(k1,k3):
c            x = ginnerp(k1,1,1,x(1,i),y(1,j))
c         This is equivalent to:
c            x = innerp(x(i,1),y(1,j),k1)
c
c     Note that zero is returned if n is zero or less.
c
c     Constantijn Panis,  13 December 1995
 
      implicit none
      double precision a(*),b(*),temp
      integer n,jsa,jsb,i,j
 
      if(n.le.0)then
         ginnerp = 0.d0
         goto 999
      endif
      temp = 0.d0
      j=1
      do 100 i=1,jsa*n,jsa
         temp = temp + a(i)*b(j)
         j = j + jsb
  100 continue
      ginnerp = temp
 
  999 return
      end
 
