      subroutine gradcheck(maxcc,ccmax,nfull,nfree,fullbeta,fulltype,
     *   iffull,lnl,nchol,chol,dfreebeta,fp,fp2,fpt,fpt2,sp,spt,
     *   fullname,locdim,dd,rr,ii,ll,cc,ipt,jpt)
 
c     Gradient checker: computes and compares numerical and analytical
c     gradients for an arbitrary function.  Returns log-likelihood,
c     first and second derivatives.
c
c     fullname [in]   vector of parameter names
c     nfree    [in]   number of free parameters
c     fullbeta [in]   vector of parameter values
c     lnl      [out]  function value
c     dfreebeta [out] scratch array for derivatives of one observation
c     fp       [out]  first derivatives
c     fp2      [out]  first derivatives, copy for numerical Hessian
c     fpt      [out]  first derivatives, transformed
c     fpt      [out]  first derivatives, copy for numerical Hessian
c     sp       [out]  second derivatives
c     spt      [out]  second derivatives, transformed
c
c     Constantijn Panis,  29 August 2002
c     Last change: iffull integer
 
      implicit none
      integer maxcc,ccmax
      double precision dd(maxcc/8)
      real rr(maxcc/4)
      integer ii(maxcc/4),locdim(120)
      logical ll(maxcc/4)
      character cc(maxcc)*1
      integer nfull,nfree,fulltype(nfull),iffull(nfull),
     *   nchol,ipt,jpt,i,j
      double precision fullbeta(nfull),lnl,chol(nchol),dfreebeta(nfree),
     *   fp(nfree),fp2(nfree),fpt(nfree),fpt2(nfree),sp(nfree,nfree),
     *   spt(nfree,nfree),fh,numgr,save,eps,jacobeps
      character fullname(nfull)*8
      logical isnan
 
      if(ipt.gt.0)write(8,101)
      if(jpt.gt.0)write(6,101)
 
c     Compute analytical gradient and the function value (as well as
c     second derivatives, as optimization routine may want this.)
      call amlfunc(111,maxcc,ccmax,nfull,nfree,fullbeta,fulltype,iffull,
     *   lnl,nchol,chol,dfreebeta,fp,fp2,fpt,fpt2,sp,spt,
     *   locdim,dd,rr,ii,ll,cc,ipt,jpt,1)
 
      if(isnan(lnl))goto 800
 
c     Compute (one-sided) numerical gradient, compare,
c     and write out results:
      j=0
      do 100 i=1,nfull
         if(iffull(i).eq.0)goto 100
         j=j+1
         save = fullbeta(i)
         eps = jacobeps(fullbeta(i))
         fullbeta(i) = fullbeta(i) + eps
         call amlfunc(100,maxcc,ccmax,nfull,nfree,fullbeta,fulltype,
     *      iffull,fh,0,0.d0,0.d0,0.d0,0.d0,0.d0,0.d0,0.d0,0.d0,
     *      locdim,dd,rr,ii,ll,cc,ipt,jpt,0)
         if(isnan(fh))goto 810
         numgr = (fh-lnl)/eps
         fullbeta(i) = save
         if(ipt.gt.0)then
            if(fp(j).ne.0.d0)then
               write(8,102)fullname(i),numgr,fp(j),(numgr-fp(j))/fp(j)
            elseif(numgr.eq.0.d0)then
               write(8,103)fullname(i),0.d0,0.d0
            else
               write(8,104)fullname(i),numgr,0.d0
            endif
            call flush(8)
         endif
         if(jpt.gt.0)then
            if(fp(j).ne.0.d0)then
               write(6,102)fullname(i),numgr,fp(j),(numgr-fp(j))/fp(j)
            elseif(numgr.eq.0.d0)then
               write(6,103)fullname(i),0.d0,0.d0
            else
               write(6,104)fullname(i),numgr,0.d0
            endif
         endif
  100 continue
      goto 999
 
  101 format(/,70('='),/,'=',27x,'GRADIENT CHECK',27x,'=',/,70('='),//,
     *   'VARIABLE      NUMERICAL       ANALYTICAL      RELATIVE',/,
     *   '  NAME         GRADIENT        GRADIENT         ERROR',/)
  102 format(a8,2x,1p,3e16.7)
  103 format(a8,2x,1p,2e16.7)
  104 format(a8,2x,1p,2e16.7,5x,'Infinity')
 
  800 if(ipt.gt.0)write(8,801)
      if(jpt.gt.0)write(6,801)
  801 format(/,'ERROR: the function evaluation failed.')
      stop
  810 if(ipt.gt.0)write(8,811)
      if(jpt.gt.0)write(6,811)
  811 format(/,'WARNING:  the function evaluation with your starting ',
     *   'values succeeded, but',/,'a minute increment in one ',
     *   'parameter lead to inability to evaluate the',/,
     *   'function.  Gradient check skipped.')
      goto 999
  999 return
      end
