      double precision function jacobeps(beta)
 
c     Double precision function to compute a near-optimal epsilon for the
c     computation of numerical first derivatives (Jacobian).
c
c     A little trick is applied following the suggestion of
c     J.E. Dennis and Robert R. Schnabel (1983), "Numerical Methods for
c     Unconstrained Optimization and Nonlinear Methods," Prentice-Hall,
c     Inc, Englewood Cliff, New Jersey, around page 100.  They suggest
c     computing eps as an exact difference between the parameter value
c     and the increased value.  In practice, I have not found it to
c     make any substantial difference.
c
c     Constantijn Panis, 24 January 1996
 
      implicit none
      double precision beta,temp,sqrtsyseps,mineps
      parameter(sqrtsyseps=1.d-6,mineps=1.d-8)
 
      temp = max(mineps,sqrtsyseps*abs(beta))
      temp = beta+temp
      jacobeps = temp-beta
 
      return
      end
