      subroutine m2mult(task,n1,n2,n3,a,b,c,factor)
 
c     Subroutine to compute A*B = C:
c
c            A(n1,n2)   B(n2,n3)   C(n1,n3)
c
c     The four matrices may be in transposed format.  Integer task
c     may be 100,010,001, et cetera, depending on which matrices
c     are transposed.  The first digit refers to A; the second to B;
c     the third to C.  For example,
c        task = 110 means that A and B are transposed: A'*B' = C
c        task =   1 means that C is transposed: A*B = C'
c
c     Dimensions as given above assume no transpositions; if a matrix
c     is transposed, its dimensions really are reversed.
c
c     Integer factor determines:
c        factor = 0     C = A*B
c              else     C = C + factor*A*B
c
c     Constantijn Panis, 21 March 1993
 
      implicit none
      integer task,n1,n2,n3,factor,i,j,k,ia,ib,ic,ja,jb,jc,
     *   sa,sb,sc,jsa,jsb,jsc
      double precision a(n1*n2),b(n2*n3),c(n1*n3),abij
 
c     In its most basic form, C = C + A*B, the routine is very compact:
c
c     do 110 i=1,n1
c        do 120 j=1,n3
c           abij = 0.d0
c           do 130 k=1,n2
c              abij = abij + a(i,k)*b(k,j)
c 130       continue
c           c(i,j) = c(i,j) + abij
c 120    continue
c 110 continue
c
c     Complications arise because this routine allows for A, B and C to
c     be stored in transposed form.  Note that A and B are only used in the
c     130 loop, and C and D are only used in the 140 loop.  The positions
c     that are used are ia, ib, ic and id.  In the 130 and 140 loops, they
c     increase by ja, jb, jc and jd, where j stands for "jump".  If A, B,
c     C and D are not-transposed, these jumps are numrowa, 1, numrowc and numrow
c     respectively, where numrow stands for the number of rows.  If A is
c     transposed, this changes to 1; if B is transposed, it becomes numrowb,
c     et cetera.  The first time an element is accessed in the 130 and 140
c     loops, we need A(i,1), B(1,j), C(j,1) and D(i,1); these are coded
c     as sa, sb, sc and sd, where s stands for "start".  These start off at
c     1 in all four cases, the first time the 130/140 loops are used.
c     The next time they increase by 1, numrowb, 1 and 1; these are coded
c     as jsa, jsb, jsc and jsd, where js stands for "jump in start".
c     If a matrix is transposed, the jumps are different.
c     The code is identical for add<0 and add>=0, except for subtracting
c     abij*c(ic) in case add<0 instead of adding it, as in add>=0.
 
      if(mod(task/100,10).eq.0)then
         ja=n1
         jsa=1
      else
         ja=1
         jsa=n2
      endif
      if(mod(task/10,10).eq.0)then
         jb=1
         jsb=n2
      else
         jb=n3
         jsb=1
      endif
      if(mod(task,10).eq.0)then
         jc=n1
         jsc=1
      else
         jc=1
         jsc=n3
      endif
 
      if(factor.eq.0)call zerodp(n1*n3,c)
      sa=1
      sc=1
      do 210 i=1,n1
         sb=1
         ic=sc
         do 220 j=1,n3
            ia=sa
            ib=sb
            abij=0.d0
            do 230 k=1,n2
               abij = abij + a(ia)*b(ib)
               ia=ia+ja
               ib=ib+jb
  230       continue
            if(factor.ne.0.and.factor.ne.1)abij=dble(factor)*abij
            c(ic) = c(ic) + abij
            ic=ic+jc
            sb=sb+jsb
  220    continue
         sa=sa+jsa
         sc=sc+jsc
  210 continue
 
      return
      end
 
