      subroutine map_corr(task,dim,stochpar,dstochpar,corr,dcorr,
     *   corrrestrict,dimp1)
 
c     Subroutine to:
c     Task = 1: map standard deviations and correlation coefficients
c        into matrix corr(dim,dim), accounting for possible restrictions
c        as specified in the control file and recorded in corrrestrict.
c     Task = 2: map and accumulate derivatives.  If there are restrictions,
c        the original derivatives dcorr are lost.
c
c     Constantijn Panis, 4 January 1994
c     Last change: bug fixed in constrained correlation derivative accumulation
 
      implicit none
      integer task,dim,i1,i2,j1,j2,ii,corrrestrict(dim,dim),dimp1
      double precision stochpar(*),dstochpar(*),corr(dim,dim),
     *   dcorr(dim,dim)
 
c     Construct correlation matrix or unravel derivatives?
      goto(100,200)task
 
c     Expand (possibly) compressed correlations and standard deviations.
c     Note the possibility that earlier listed parameters are restricted
c     to equal later listed parameters -- hence 2 stages.
  100 if(corrrestrict(1,1).ne.0)then
c        There are restrictions: first deal with standard deviations
         ii=0
         do 110 i1=1,dim
            if(i1.ne.corrrestrict(i1,i1)/dimp1)goto 110
            ii=ii+1
            corr(i1,i1)=stochpar(ii)
  110    continue
c        And then correlation coefficients:
         do 120 i2=1,dim
            do 125 i1=i2+1,dim
               j1=mod(corrrestrict(i1,i2),dimp1)
               j2=corrrestrict(i1,i2)/dimp1
               if(i1.ne.j1.or.i2.ne.j2)goto 125
               ii=ii+1
               corr(i1,i2)=stochpar(ii)
  125       continue
  120    continue
         do 130 i2=1,dim
            j2=corrrestrict(i2,i2)/dimp1
            corr(i2,i2)=corr(j2,j2)
            do 135 i1=i2+1,dim
               j1=mod(corrrestrict(i1,i2),dimp1)
               j2=corrrestrict(i1,i2)/dimp1
               corr(i1,i2)=corr(j1,j2)
               corr(i2,i1)=corr(i1,i2)
  135       continue
  130    continue
      else
c        There are no restrictions:
         ii=dim
         do 140 i2=1,dim
            corr(i2,i2)=stochpar(i2)
            do 145 i1=i2+1,dim
               ii=ii+1
               corr(i1,i2)=stochpar(ii)
               corr(i2,i1)=corr(i1,i2)
  145       continue
  140    continue
      endif
      goto 999
 
c     Place derivatives in right positions.  Note that dcorr is lost.
c     Note that it is **not** assumed that dcorr is symmetric.
  200 if(corrrestrict(1,1).ne.0)then
c        There are restrictions.
c        First accumulate restricted parameters to the restrictors:
         do 210 i2=1,dim
            j2=corrrestrict(i2,i2)/dimp1
            if(i2.ne.j2)then
               dcorr(j2,j2)=dcorr(j2,j2)+dcorr(i2,i2)
               dcorr(i2,i2)=0.d0
            endif
            do 215 i1=i2+1,dim
               j1=mod(corrrestrict(i1,i2),dimp1)
               j2=corrrestrict(i1,i2)/dimp1
               if(i1.eq.j1.and.i2.eq.j2)then
c                 RHO(i1,i2) is not constrained; collapse in lower triangle:
                  dcorr(i1,i2)=dcorr(i1,i2)+dcorr(i2,i1)
               else
c                 RHO(i1,i2) is constrained to be equal to RHO(j1,j2);
c                 accumulate into (j1,j2) and collapse into lower triangle:
                  dcorr(j1,j2)=dcorr(j1,j2)+dcorr(i1,i2)+dcorr(i2,i1)
               endif
  215       continue
  210    continue
c        Then map standard deviations into stochpar:
         ii=0
         do 220 i1=1,dim
            if(i1.ne.corrrestrict(i1,i1)/dimp1)goto 220
            ii=ii+1
            dstochpar(ii)=dstochpar(ii)+dcorr(i1,i1)
  220    continue
c        And then map correlation coefficients into stochpar:
         do 230 i2=1,dim
            do 235 i1=i2+1,dim
               j1=mod(corrrestrict(i1,i2),dimp1)
               j2=corrrestrict(i1,i2)/dimp1
               if(i1.ne.j1.or.i2.ne.j2)goto 235
               ii=ii+1
               dstochpar(ii)=dstochpar(ii)+dcorr(i1,i2)
  235       continue
  230    continue
      else
c        There are no restrictions:
         ii=dim
         do 240 i2=1,dim
            dstochpar(i2)=dstochpar(i2)+dcorr(i2,i2)
            do 245 i1=i2+1,dim
               ii=ii+1
               dstochpar(ii)=dstochpar(ii)+dcorr(i1,i2)+dcorr(i2,i1)
  245       continue
  240    continue
      endif
      goto 999
  999 return
      end
