      subroutine onenegbin(nen,draw,cdraw,a5,ddda5,p5,dddp5,
     *   epsn,lnlg,v5,ifdrv,ipt,jpt,status)
 
c     Subroutine to compute the conditional (on heterogeneity) integrand
c     of all negative binomial modules.
c
c     For new-style (version 2+) negative binomials, the likelihood is
c     the product over modules of Gamma(1/A+Y)/(Gamma(1/A)*Gamma(Y+1))
c     times the integral over heterogeneity of theta**(1/A) * (1-theta)**Y.
c     Theta is 1/(1+A*E*exp(Beta'X+eps)).  Higher Beta'X implies lower
c     theta implies higher outcomes.  A=dispersion, E=exposure.
c
c     For old-style negative binomial, the likelihood is the product over
c     modules of Gamma(A+Y)/(Gamma(A)*Gamma(Y+1)) times the integral
c     over heterogeneity of theta**A * (1-theta)**Y.  Theta is the
c     probability, a logit/probit/identity link of Beta'X and eps.
c     E.g., for logit link:  theta = 1/(1+exp(-beta'X-eps)).  Note that
c     higher Beta'X implies higher probability implies lower outcomes(!)
c     Prep5n already computed Gamma(A+Y)/(Gamma(A)*Gamma(Y+1)).
c
c     Upon output, status may be
c        unchanged with normal execution
c        -1 with underflow
c        3 with illegitimate probability
c
c     Constantijn Panis, 20 December 2002
c     Last change: exposure
 
      implicit none
      integer nen,draw(nen),cdraw,ie,ipt,jpt,status
      double precision a5(nen),ddda5(nen),p5(nen),dddp5(nen),epsn(nen),
     *   lnlg,theta,dtheta,aa,bb,ee,yy,tmp,pnorm1,dnorm1
      real v5(3*nen)
      logical ifdrv
 
      status=0
c     Old-style and new-style parameterizations cannot be mixed.
c     Branch on the basis of the first module only:
      if(nen.eq.0)call bug(ipt,jpt)
      if(v5(nen+1).ne.4.)goto 200
 
c     New-style (version 2+) negative binomial modules:
c     Compute the (log of the) integrand theta**(1/A) * (1-theta)**Y
c     where theta = 1/(1+A*exp(Beta'X+eps)).
      do 100 ie=1,nen
c        Do we need to evaluate this (conditional) likelihood module?
         if(cdraw.ne.draw(ie))goto 100
         yy=dble(v5(ie))
         ee=dble(v5(ie+3*nen))
         aa=a5(ie)
         bb=p5(ie)+epsn(ie)
         theta = 1.d0/(1.d0 + aa*ee*dexp(bb))
         if(theta.lt.0.d0.or.theta.gt.1.d0)goto 800
         if(theta.eq.0.d0.or.theta.eq.1.d0)goto 900
 
         lnlg = lnlg + dlog(theta)/aa + yy*dlog(1.d0-theta)
         if(.not.ifdrv)goto 100
 
c        Derivatives of log-integrand wrt p5 (= wrt epsn) and a5:
         tmp = (-theta/aa + yy*theta*theta/(1.d0-theta))*ee*dexp(bb)
         ddda5(ie) = tmp - dlog(theta)/(aa*aa)
         dddp5(ie) = aa*tmp
  100 continue
      goto 999
 
c     Old-style negative binomial modules.
c     Compute the (log of the) integrand theta**A * (1-theta)**Y
  200 do 210 ie=1,nen
c        Do we need to evaluate this (conditional) likelihood module?
         if(cdraw.ne.draw(ie))goto 210
         if(v5(nen+ie).eq.1.)then
c           Logistic transformation:
            theta = 1.d0/(1.d0+dexp(-p5(ie)-epsn(ie)))
         elseif(v5(nen+ie).eq.2.)then
c           Probit transformation:
            theta = pnorm1(p5(ie)+epsn(ie))
         elseif(v5(nen+ie).eq.3.)then
c           No transformation (no integrated residual allowed):
            theta = p5(ie)
         else
            call bug(ipt,jpt)
         endif
         if(theta.lt.0.d0.or.theta.gt.1.d0)goto 800
         if(theta.eq.0.d0.or.theta.eq.1.d0)goto 900
         yy=dble(v5(ie))
         lnlg = lnlg + a5(ie)*dlog(theta) + yy*dlog(1.d0-theta)
         if(.not.ifdrv)goto 210
 
c        Derivatives of log-integrand wrt p5 (= wrt epsn) and a5:
         dtheta = a5(ie)/theta - yy/(1.d0-theta)
         ddda5(ie) = dlog(theta)
         if(v5(nen+ie).eq.1.)then
            dddp5(ie) = dtheta*theta*(1.d0-theta)
         elseif(v5(nen+ie).eq.2.)then
            dddp5(ie) = dtheta*dnorm1(p5(ie)+epsn(ie))
         elseif(v5(nen+ie).eq.3.)then
            dddp5(ie) = dtheta
         endif
  210 continue
      goto 999
 
c     Illegitimate probability.  Only possible with probability
c     without transformation.
  800 if(ipt.gt.0)write(8,801)
      if(jpt.gt.0)write(6,801)
  801 format(/,'ERROR: The function evaluation failed because of an ',
     *   'illegitimate',/,'negative binomial probability.  Use the ',
     *   'LOGISTIC or NORMPROB',/,'probability transformation, or ',
     *   'restrict the range of the probability',/,'parameter to be ',
     *   'in (0,1).')
      status=3
      goto 999
 
c     Likelihood underflows to zero:
  900 status=-1
  999 return
      end
