      subroutine ordlogit(prob,eps,tauminxb,dtau,dxb,ifdrv,ipt,jpt,
     *   status)
 
c     Subroutine to compute an ordered logit probability:
c        prob = F(tau_u-XB-eps) - F(tau_l-XB-eps)
c     where tau_u and tau_l are upper and lower thresholds; they may
c     be -inf and +inf, but not simultaneously.  F() is the cumulative
c     probability function, F(x)=1/(1+exp(-x))
c
c     Upon return, status is
c       -1 if the probability underflows to zero
c        2 if thresholds were not ordered (message is written)
c
c     Constantijn Panis, 20 March 1997
 
      implicit none
      double precision prob,eps,tauminxb(2),dxb,dtau(2),
     *   zu,zl,probu,probl
      integer ipt,jpt,status
      logical ifdrv,isinf
 
      status=0
      if(isinf(-tauminxb(1)))then
c        Lower threshold is -Inf
         zu=tauminxb(2)-eps
         probu = 1.d0/(1+dexp(-zu))
         prob = probu
         if(.not.ifdrv)goto 200
         dtau(1)=0.d0
         dtau(2)=probu*probu*dexp(-zu)
         dxb=-dtau(2)
      elseif(isinf(tauminxb(2)))then
c        Upper threshold is +Inf
         zl=tauminxb(1)-eps
         probl = 1.d0/(1+dexp(-zl))
         prob = 1.d0 - probl
         if(.not.ifdrv)goto 200
         dtau(1)=-probl*probl*dexp(-zl)
         dtau(2)=0.d0
         dxb=-dtau(1)
      else
         if(tauminxb(2).le.tauminxb(1))goto 810
         zu=tauminxb(2)-eps
         zl=tauminxb(1)-eps
         probu = 1.d0/(1+dexp(-zu))
         probl = 1.d0/(1+dexp(-zl))
         prob = probu-probl
         if(.not.ifdrv)goto 200
         dtau(1)=-probl*probl*dexp(-zl)
         dtau(2)=probu*probu*dexp(-zu)
         dxb=-dtau(2)-dtau(1)
      endif
 
  200 if(prob.le.0.d0)goto 900
      goto 999
 
  810 if(ipt.gt.0)write(8,811)
      if(jpt.gt.0)write(6,811)
  811 format(/,'The function evaluation failed because ordered logit ',
     *   'thresholds are not ordered.')
      status=2
      goto 999
c     Likelihood underflows to zero:
  900 status=-1
  999 return
      end
