      subroutine ordprobit(prob,eps,sigma,tauminax,dtau,dax,dsigma,
     *   ifdrv,ipt,jpt,status)
 
c     Subroutine to compute an ordered probit probability:
c     prob = Phi((tau_u-XB-eps)/sigma) - Phi((tau_l-XB-eps)/sigma)
c     where tau_u and tau_l are upper and lower thresholds; they may
c     be -inf and +inf, but not simultaneously.
c
c     Upon return, status is
c        0 if normal execution
c       -1 if the likelihood underflows to zero
c        1 if a standard deviation was zero (message is written)
c        2 if thresholds were not ordered (message is written)
c
c     Constantijn Panis, 8 July 1996
 
      implicit none
      double precision prob,eps,sigma,tauminax(2),dax,dtau(2),dsigma,
     *   zu,zl,phiu,phil,pnorm1,dnorm1,dpnan
      integer ipt,jpt,status
      logical ifdrv,isinf
 
      status=0
      if(sigma.eq.0.d0)goto 800
 
      if(isinf(-tauminax(1)))then
c        Lower threshold is -Inf
         zu=(tauminax(2)-eps)/sigma
         prob = pnorm1(zu)
      elseif(isinf(tauminax(2)))then
c        Upper threshold is +Inf
         zl=(tauminax(1)-eps)/sigma
         prob = 1.d0-pnorm1(zl)
      else
         if(tauminax(2).le.tauminax(1))goto 810
         zu=(tauminax(2)-eps)/sigma
         zl=(tauminax(1)-eps)/sigma
         if(zl.lt.0.d0)then
            prob = pnorm1(zu)-pnorm1(zl)
         else
c           Gain numerical precision:
            prob = pnorm1(-zl)-pnorm1(-zu)
         endif
      endif
      if(prob.eq.0.d0)goto 900
 
      if(.not.ifdrv)goto 999
      if(isinf(-tauminax(1)))then
         phil = 0.d0
         phiu = dnorm1(zu)
         dsigma = -phiu*zu/sigma
      elseif(isinf(tauminax(2)))then
         phil = dnorm1(zl)
         phiu = 0.d0
         dsigma = phil*zl/sigma
      else
         phil = dnorm1(zl)
         phiu = dnorm1(zu)
         dsigma = (phil*zl-phiu*zu)/sigma
      endif
 
      dax = (phil-phiu)/sigma
      dtau(1) = -phil/sigma
      dtau(2) = phiu/sigma
      goto 999
 
  800 if(ipt.gt.0)write(8,801)
      if(jpt.gt.0)write(6,801)
  801 format(/,'The function evaluation failed because the standard ',
     *   'deviation of a probit',/,'residual is zero.')
      status=1
      goto 999
  810 if(ipt.gt.0)write(8,811)
      if(jpt.gt.0)write(6,811)
  811 format(/,'The function evaluation failed because ordered probit ',
     *   'thresholds',/,'are not ordered.')
      prob=dpnan()
      status=2
      goto 999
c     Likelihood underflows to zero:
  900 status=-1
  999 return
      end
 
