      subroutine partition(sigma,k1,k2,sigma11,sigma12,sigma22)
 
c     Subroutine to take partitions of a symmetric matrix.  Consider a
c     matrix of which one needs 4 partitions:
c             (sigma11  sigma12)
c     sigma = (sigma21  sigma22)
c     The dimensions are sigma11(k1,k1), sigma12(k1,k2), sigma22(k2,k2)
c     and of course sigma(k1+k2,k1+k2).
c     The matrix must be symmetric and only sigma11, sigma12 and sigma22
c     are of interest.  This subroutine outputs the three partition
c     matrices, or alternatively lets you re-arrange terms of sigma such
c     that the first k1*k1 elements are sigma11, the next k1*k2 elements
c     sigma12 and the next k2*k2 elements are sigma22.  Note that this
c     still leaves k1*k2 elements free for whichever purpose the user
c     desires.  Should this rearrangement of terms be desired, this
c     routine should be called as follows:
c     call partition(sigma,k1,k2,
c    *    sigma(1),sigma(k1*k1+1),sigma(k1*k1+k1*k2+1))
c
c     Constantijn Panis,  9 December 1990
 
      implicit none
      integer k1,k2,i
      double precision sigma(k1+k2,k1+k2),sigma11(k1,k1),sigma12(k1,k2),
     *   sigma22(k2,k2)
 
c     Note that the order in which the following partitions are taken
c     is important, since it allows input and output matrices to share
c     the same memory space.
      do 100 i=1,k1
         call copy8(sigma(1,i),sigma11(1,i),k1)
  100 continue
      do 110 i=1,k2
         call copy8(sigma(1,k1+i),sigma12(1,i),k1)
  110 continue
      do 120 i=1,k2
         call copy8(sigma(k1+1,k1+i),sigma22(1,i),k2)
  120 continue
 
      return
      end
