      double precision function pnorm2(x1,x2,r)
 
c     Function to calculate the bivariate normal lower left area, i.e.,
c     the probability for two normal variates x and y whose correlation is r:
c        Prob(a<x1, b<x2 | r).
c
c     Communications of the ACM, October 1973, vol. 16, no. 10.,
c     algorithm 462, Thomas G. Donnelly.
c
c     Constantijn Panis, 1 December 2003
c     Last change: Lahey declares derf
 
      USE numerical_libraries
 
      implicit none
      double precision x1,x2,r,twopi,twopmh,ah,ak,b,gh,gk,rr,sqr,wh,
     *   wk,gw,sgn,t,g2,h4,h2,ex,ap,sp,s1,sn,conex,s2,w2,a2,cn,con
      integer idig,is
c     Digital declaration is implicit in USE statement, above.
c     double precision derf
c     double precision derf
c     double precision derf
c     external derf
 
c     twopmh stands for 2 to the power minus a half: 1.d0/dsqrt(2.d0)
c     idig gives the number of significant digits to the right of the
c     decimal point desired in the answer (if within computer capacity).
      data twopi/6.2831853071795864769d0/,
     *   twopmh/.7071067811865475244d0/,
     *   idig/15/
 
      if(abs(r).gt.1.d0)goto 800
 
      ah= -x1
      ak= -x2
      b = 0.d0
      gh=.25d0*(1.d0+derf(-twopmh*ah))
      gk=.25d0*(1.d0+derf(-twopmh*ak))
      if(r.eq.0.d0)then
         b=4.d0*gh*gk
         goto 350
      endif
      rr=1.d0-r*r
      if(rr.eq.0.d0)then
         if(r.lt.0.d0)then
            if(ah+ak.lt.0.d0)b=2.d0*(gh+gk)-1.d0
         else
            if(ah-ak.lt.0.d0)then
               b=2.d0*gk
            else
               b=2.d0*gh
            endif
         endif
         goto 350
      endif
      sqr=dsqrt(rr)
      con=.5d0*twopi*(10.d0**dble(-idig))
      if(ah*ak.gt.0.d0)then
         b=gh+gk
      elseif(ah*ak.lt.0.d0)then
         b=gh+gk-0.5d0
      elseif(ah.ne.0.d0.and.ak.eq.0.d0)then
         b=gh
      elseif(ah.eq.0.d0.and.ak.eq.0.d0)then
         b=datan(r/sqr)/twopi+0.25d0
         goto 350
      elseif(ah.eq.0.d0.and.ak.ne.0.d0)then
         b=b+gk
         goto 340
      endif
      wh=-ah
      wk=(ak/ah-r)/sqr
      gw=2.d0*gh
      is=-1
  210 sgn=-1.d0
      t=0.d0
      if(wk.eq.0.d0)goto 320
      if(dabs(wk).eq.1.d0)then
         t=wk*gw*.5d0*(1.d0-gw)
         goto 310
      elseif(dabs(wk).gt.1.d0)then
         sgn=-sgn
         wh=wh*wk
         g2=.5d0*(1.d0+derf(twopmh*wh))
         wk=1.d0/wk
         if(wk.lt.0.d0)b=b+0.5d0
         b=b-.5d0*(gw+g2)+gw*g2
      endif
      h2=wh*wh
      a2=wk*wk
      h4=.5d0*h2
      ex=dexp(-h4)
      w2=h4*ex
      ap=1.d0
      s2=ap - ex
      sp=ap
      s1=0.d0
      sn=s1
      conex=dabs(con/wk)
      goto 290
  280 sn=sp
      sp=sp+1.d0
      s2=s2-w2
      w2=w2*h4/sp
      ap=-ap*a2
  290 cn=ap*s2/(sn+sp)
      s1=s1+cn
      if(dabs(cn).gt.conex)goto 280
      t=(datan(wk)-wk*s1)/twopi
  310 b=b+sgn*t
  320 if(is.ge.0.d0)goto 350
      if(ak.eq.0.d0)goto 350
  340 wh = -ak
      wk=(ah/ak-r)/sqr
      gw=2.d0*gk
      is=1
      goto 210
 
  350 pnorm2=max(0.d0,min(b,1.d0))
      goto 999
 
c     abs(r)>1
  800 write(6,801)r
  801 format('ERROR:  Correlation coefficient is',g26.16)
      call bug(0,1)
  999 return
      end
