      subroutine quadratic023(task,nrowa,dimb,dimblockc,a,b,cblock,
     *   factor)
 
c     Subroutine to compute a quadratic form:
c
c     task = 0: compute A*B*A' = C    A(nrowa,dimb)
c                                     B(dimb,dimb)
c                                     C(dimblockc,dimblockc)
c     task = 1: compute A'*B*A = C    A(dimb,nrowa)
c                                     B(dimb,dimb)
c                                     C(dimblockc,dimblockc)
c
c     Matrix types can be:
c        0 = generic
c        1 = identity (Only its dimension is stored.)
c        2 = symmetric
c        3 = symmetric and block-diagonal (Only its first block, its
c            dimension, and number of blocks are stored.)
c        4 = sparse in the sense that there is at most one nonzero
c            element in any one row.  (Only its nonzero elements
c            and their column position are stored.)
c
c     This routine only applies to B symmetric and C block-diagonal.
c
c     Integer factor determines:
c        factor = 0     C = A*B*A'
c              else     C = C + factor*A*B*A'
c
c     If factor!=0, it is assumed that upon input, C is already
c     symmetric, so that upon output, C is still symmetric.
c
c     Note some rules for differentiation.   If C = A*B*A', then
c        dA = dC*A*B' + dC'*A*B = 2*dC*A*B if dC and B are symmetric.
c        dB = A'*dC*A
c
c     This routine is very obscure.  See m3mult (which computes the
c     product of three arbitrary matrices) for details.  The current
c     routine is a special case of m3mult, taking advantage of the
c     fact that C is symmetric.
c
c     Constantijn Panis, 6 August 2002
 
      implicit none
      integer task,nrowa,dimb,dimblockc,factor,i,j,k,l,
     *   ia,ib,ic,ja,jb,jc,sa,sb,sc,jsa,jsb,jsc,cid
      double precision a(nrowa*dimb),b(dimb*dimb),
     *   cblock(dimblockc*dimblockc),abij
 
      if(factor.eq.0)call zerodp(dimblockc*dimblockc,cblock)
 
      if(task.eq.0)then
         ja=nrowa
         jsa=1
         jc=1
         jsc=nrowa
      else
         ja=1
         jsa=dimb
         jc=dimb
         jsc=1
      endif
      jb=1
      jsb=dimb
 
      sa=1
      do 110 i=1,nrowa
         sb=1
         sc=1
         do 120 j=1,dimb
            abij = 0.d0
            ia=sa
            ib=sb
            ic=sc
            do 130 k=1,dimb
               abij = abij + a(ia)*b(ib)
               ia=ia+ja
               ib=ib+jb
  130       continue
            if(factor.ne.0.and.factor.ne.1)abij=dble(factor)*abij
c           Loop over l=1,i and multiply AB(i,j) with A(.,j)
c           to accumulate C(i,.).  No need to cover the entire
c           C(i,.) because C is block-diagonal.
            ic=sc+((i-1)/dimblockc)*dimblockc*jc
            cid=mod(i-1,dimblockc)+1
            do 140 l=1,mod(i-1,dimblockc)+1
               cblock(cid) = cblock(cid) + abij*a(ic)
               cid=cid+dimblockc
               ic=ic+jc
  140       continue
            sb=sb+jsb
            sc=sc+jsc
  120    continue
         sa=sa+jsa
  110 continue
      call reflect(cblock,dimblockc)
 
      return
      end
 
