      subroutine quadratic423(nrowa,numblockc,dimblockc,
     *   asparse,b,cblock,factor)
 
c     Subroutine to compute a quadratic form:
c
c     A'*B*A = C    A(nrowa,dimc)
c                   B(nrowa,nrowa)
c                   C(dimc,dimc)
c     where dimc=numblockc*dimblockc.
c
c     Matrix types can be:
c        0 = generic
c        1 = identity (Only its dimension is stored.)
c        2 = symmetric
c        3 = symmetric and block-diagonal (Only its first block, its
c            dimension, and number of blocks are stored.)
c        4 = sparse in the sense that there is at most one nonzero
c            element in any one row.  (Only its nonzero elements
c            and their column position are stored.)
c
c     This routine only applies to A sparse, B symmetric, and C
c     block-diagonal.
c
c     Integer factor determines:
c        factor = 0     C = A'*B*A
c              else     C = C + factor*A'*B*A
c
c     If factor!=0, it is assumed that upon input, C is already
c     symmetric, so that upon output, C is still symmetric.
c
c     This routine is very obscure.  See m3mult (which computes the
c     product of three arbitrary matrices) for details.  The current
c     routine is a special case of m3mult, taking advantage of the
c     fact that C is symmetric.
c
c     Constantijn Panis, 1 December 2001
 
      implicit none
      integer nrowa,numblockc,dimblockc,factor,i,j,k,l,ii,kk
      double precision asparse(nrowa,2),b(nrowa,nrowa),
     *   cblock(dimblockc,dimblockc),abij
 
      if(factor.eq.0)call zerodp(dimblockc*dimblockc,cblock)
 
c     Approach: first compute A'B(i,j), then use C(i,k)=+A'B(i,j)*A(j,k),
c     where only k=acol(j) is relevant.
      do 100 i=1,numblockc*dimblockc
         do 200 j=1,nrowa
c           A'B(i,j) will affect C(i,k):
            k=int(asparse(j,1))
            if(i.lt.k.or.k.le.0.or.(i-1)/dimblockc.ne.(k-1)/dimblockc)
     *         goto 200
            ii=mod(i-1,dimblockc)+1
            kk=mod(k-1,dimblockc)+1
            abij=0.d0
            do 300 l=1,nrowa
               if(int(asparse(l,1)).eq.i)abij=abij+
     *            asparse(l,2)*b(l,j)
  300       continue
            if(factor.ne.0.and.factor.ne.1)abij=dble(factor)*abij
            cblock(ii,kk)=cblock(ii,kk)+abij*asparse(j,2)
  200    continue
  100 continue
 
      call reflect(cblock,dimblockc)
 
      return
      end
 
