      subroutine read_dspl(levelmax,mdllvl,numspl,maxmeno,moel,meno,
     *   nxold,namepntr,nold,xnodepos,maxxnode,maxnumfreq,numfreq,
     *   freqloc,struc,xnode,dursplinedone,oldname,splname,line,il,
     *   ipt,jpt)
 
c     Subroutine to read a duration spline specification.  The syntax is:
c        DURATION SPLINE (ORIGIN = varname, REF = splinename)
c        DURATION SPLINE (ORIGIN = varname, REFVAR = varname)
c
c     DURATION SPLINE may be abbreviated to DURSPLINE.
c
c     (1) ORIGIN  for the time lag of the spline, i.e., the time between
c     the beginning of the episode and the moment the spline becomes active.
c     An expression may be specified in lieu of a variable name.
c
c     (2) REF/REFVAR  for the selection of the functional form of the
c     spline.  Can be specified directly by a spline name (REF=), or
c     indirectly by a variable name (REFVAR=).
c
c     Information is stored in moel(*):
c     moel(1) = 5  this is a duration spline
c     moel(2) = xnode position of expression
c     moel(3) = 1  spline definition directly specified
c               2  spline definition indirectly referenced
c     moel(4) = spline definition number if moel(3)=1
c               expression position in xnode if moel(3)=2
c
c     Integer dursplinedone stores information on the type of duration
c     spline specified:
c        -1  [in] no duration spline allowed
c         0   no duration spline specified yet
c         1   one or more duration splines specified
c        99  [in] duration splines OK; do not change
c             this value (saves variables)
c
c     Constantijn Panis, 20 March 1998
 
      implicit none
      integer levelmax,numspl,maxmeno,moel(maxmeno),meno,xnodepos,
     *   maxxnode,mdllvl,nold,nxold(levelmax),namepntr(levelmax),
     *   maxnumfreq,numfreq,freqloc(maxnumfreq),dursplinedone,struc,
     *   level,ref,i1,i2,il,ipt,jpt,status,trm
      real xnode(maxxnode)
      logical smtxt
      character line*128,oldname(nold)*8,splname(numspl)*12,
     *   string*60,idstring*60
 
      if(dursplinedone.eq.-1)goto 840
      if(dursplinedone.eq.0)dursplinedone=1
 
      if(meno+4.gt.maxmeno)call
     *   maxexceed('maxmeno',meno+4,maxmeno,ipt,jpt)
 
c     This is a duration spline:
      moel(meno+1)=5
 
c     ORIGIN (i.e., time lag)
      if(.not.smtxt(4,line,il,'('))goto 800
      if(.not.smtxt(4,line,il,'ORIGIN='))goto 800
      moel(meno+2)=xnodepos+1
      call readexpr(1,levelmax,mdllvl,level,1,nold,maxxnode,xnodepos,
     *   xnode,nxold,namepntr,numfreq,maxnumfreq,freqloc,oldname,line,
     *   il,ipt,jpt,struc,',')
 
c     REF=   or   REFVAR=
      if(smtxt(4,line,il,','))continue
      if(smtxt(4,line,il,'REFVAR='))then
c        Indirectly referenced:
         moel(meno+3)=2
         moel(meno+4)=xnodepos+1
         call readexpr(1,levelmax,mdllvl,level,1,nold,maxxnode,
     *      xnodepos,xnode,nxold,namepntr,numfreq,maxnumfreq,freqloc,
     *      oldname,line,il,ipt,jpt,struc,')')
      elseif(smtxt(4,line,il,'REFerence='))then
c        Directly referenced:
         call getword(4,line,il,i1,i2,status)
         if(status.gt.0)goto 800
         call char12match(line(i1:i2),splname,numspl,ref)
         if(ref.eq.0)goto 800
         moel(meno+3)=1
         moel(meno+4)=ref
         if(.not.smtxt(4,line,il,')'))goto 800
      else
         goto 800
      endif
      meno=meno+4
      goto 999
 
  800 string=idstring(struc,0,0,0,0)
      if(ipt.gt.0)then
         call caretline(8,line,il)
         call wrapstring(8,0,72,0,1,'ERROR in the syntax of a '//
     *      'duration spline reference in a model specification'//
     *      string(1:trm(string))//'  The syntax is:')
         write(8,801)
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         call wrapstring(6,0,72,0,1,'ERROR in the syntax of a '//
     *      'duration spline reference in a model specification'//
     *      string(1:trm(string))//'  The syntax is:')
         write(6,801)
      endif
  801 format(6x,'DURSPLINE (ORIGIN = varname, REF = splinename)',/,
     *   'or, in case the spline is indirectly referenced:',/,
     *   6x,'DURSPLINE (ORIGIN = varname, REFVAR = varname)',/,
     *   'where the varnames may also be expressions.')
      stop
  840 if(ipt.gt.0)then
         call caretline(8,line,il)
         write(8,841)
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         write(6,841)
      endif
  841 format(/,'ERROR:  no duration spline allowed here.')
      stop
  999 return
      end
