      subroutine read_model7(mtype,levelmax,nfull,maxmodel,datadim,
     *   numdist,numrset,numspl,numpar,nold,prof,maxmeno,meno,moel,
     *   moelpntr,distdim,strctre,datas,numfreq,maxnumfreq,freqloc,
     *   nfreq,mdllvl,nxold,namepntr,modeltype,minlnl,parpos,
     *   pardim,fulltype,xnodepos,maxxnode,
     *   nummodel,xnode,datastructure,rsetname,splname,
     *   parname,resname,oldname,line,il,ipt,jpt)
 
c     Subroutine to read a multinomial logit/probit specification.
c     If mtype=71, mn logit, mtype=72, mn probit.
c
c     The syntax is:
c
c     multinomial {logit|probit} model;  [DATA STRUCTURE = n;]
c        [({KEEP | DROP} IF ...)];
c        [NUMERATOR;]  [DENOMINATOR;]
c        outcome=expr;
c        model n1 = ...;
c        model n2 = ...;
c        model :  = ...;
c        model nk = ...;
c
c     where the outcome is an integer-valued expression.  The model distinguishe
c     n1, n2, ..., nk; the omitted category is anything else.
c
c     For multinomial logit, moel contains:
c        1 = keep/drop (pointer to xnode)
c        2 = number of equations (=number of choices-1)
c        3 = outcome (pointer to xnode)
c        4 = potential outcome 1
c        5 = potential outcome 2
c        ++  et cetera
c        4+neqn  model specification, first equation
c        ++  model specification, next equation,
c        ++  etc.
c
c     For multinomial probit, moel contains:
c        1 = keep/drop (pointer to xnode)
c        2 = number of equations (=number of choices-1)
c        3 = outcome (pointer to xnode)
c        4 = potential outcome 1
c        5 = potential outcome 2
c        ++  et cetera
c        4+neqn  pointer to model specification for first outcome (=4+2*neqn)
c        5+neqn  pointer to model specification for second outcome
c        ++  et cetera
c        4+2*neqn  model specification, first equation
c        ++  model specification, next equation,
c        ++  etc.
c
c     Constantijn Panis, 16 October 2002
c     Last change: logit and probit
 
      implicit none
      integer mtype,levelmax,nfull,maxmodel,datadim,numdist,numrset,
     *   numspl,numpar,nold,prof,maxmeno,meno,moel(maxmeno),
     *   moelpntr(maxmodel),distdim(numdist),strctre(maxmodel),
     *   datas(maxmodel),numfreq,maxnumfreq,freqloc(maxnumfreq),
     *   nfreq(maxmodel),mdllvl(maxmodel),
     *   nxold(levelmax,datadim),namepntr(levelmax,datadim),
     *   modeltype(maxmodel),minlnl(maxmodel),parpos(numpar),
     *   pardim(numpar),fulltype(nfull),xnodepos,maxxnode,
     *   nummodel,struc,ds,il,ipt,jpt,resdone,eqn,outcome,status,
     *   i,neqn,match
      real xnode(maxxnode),datastructure(datadim)
      logical smtxt
      character line*128,rsetname(numrset)*12,splname(numspl)*12,
     *   parname(numpar)*12,resname(*)*12,oldname(nold)*8
 
      nummodel=nummodel+1
      if(nummodel.gt.maxmodel)call bug(ipt,jpt)
      modeltype(nummodel)=mtype
      moelpntr(nummodel)=meno+1
 
c     Record beginning position of summary stats; number follows below.
      if(numfreq.lt.maxnumfreq)then
         numfreq=numfreq+1
         nfreq(nummodel)=numfreq
         freqloc(numfreq)=-1
      else
         numfreq=maxnumfreq
         nfreq(nummodel)=0
      endif
 
c     [DATA STRUCTURE = n;]
      call getstruc(datadim,struc,ds,il,ipt,jpt,datas(nummodel),
     *   strctre(nummodel),datastructure,line)
 
c     [{KEEP | DROP} IF condition;]
      call getkeepdrop(levelmax,xnodepos,moel(moelpntr(nummodel)),nold,
     *   maxxnode,nxold(1,ds),namepntr(1,ds),numfreq,maxnumfreq,freqloc,
     *   ipt,jpt,il,struc,xnode,oldname,line)
 
c     [NUMERATOR;]  [DENOMINATOR;]
      call getnumdenom(minlnl(nummodel),line,il)
 
      if(.not.smtxt(4,line,il,'OUTCOME='))goto 800
      meno=moelpntr(nummodel)+2
      moel(meno)=xnodepos+1
      call readexpr(1,levelmax,levelmax,mdllvl(nummodel),1,nold,
     *   maxxnode,xnodepos,xnode,nxold(1,ds),namepntr(1,ds),0,0,0,
     *   oldname,line,il,ipt,jpt,struc,';')
 
c     Outcomes may be given as, for example, OUTCOME=0; but only if
c     there is only one level in the data.  Otherwise, a variable
c     is required.
      if(mdllvl(nummodel).eq.0)then
         if(levelmax.gt.1)goto 810
         mdllvl(nummodel)=1
      endif
 
c     MODEL n1 =....;
c     MODEL n2 =....;
c     MODEL :  =....;
c     MODEL nk =....;
 
      if(.not.smtxt(4,line,il,'MODEL'))goto 800
      eqn=0
  100 eqn=eqn+1
      if(neqn.gt.3.and.mtype.eq.72)goto 830
      call getint(4,outcome,line,il,status)
      if(status.ne.0)goto 800
      meno=meno+1
      moel(meno)=outcome
      if(.not.smtxt(4,line,il,'='))goto 800
      if(mtype.eq.71)then
c        Non-integrated residuals not allowed in multinomial logit
         resdone=-1
         call read_model(levelmax,struc,mdllvl(nummodel),nfull,
     *      numrset,numspl,numdist,numpar,distdim,maxmeno,moel,meno,
     *      maxxnode,xnodepos,numfreq,maxnumfreq,freqloc,nold,prof,
     *      nxold(1,ds),namepntr(1,ds),parpos,pardim,
     *      modeltype(nummodel),resdone,99,-1,xnode,rsetname,splname,
     *      parname,oldname,resname,'MODEL n = ','multinomial logit,',
     *      ';',line,il,ipt,jpt)
      else
c        Non-integrated residuals OK but not required in multinomial probit
         resdone=0
         call read_model(levelmax,struc,mdllvl(nummodel),nfull,
     *      numrset,numspl,numdist,numpar,distdim,maxmeno,moel,meno,
     *      maxxnode,xnodepos,numfreq,maxnumfreq,freqloc,nold,prof,
     *      nxold(1,ds),namepntr(1,ds),parpos,pardim,
     *      modeltype(nummodel),resdone,99,-1,xnode,rsetname,splname,
     *      parname,oldname,resname,'MODEL n = ','multinomial probit,',
     *      ';',line,il,ipt,jpt)
         if(resdone.eq.0)goto 840
      endif
 
c     More model statements?
      if(smtxt(4,line,il,'MODEL'))goto 100
 
c     There were eqn model equations, i.e., eqn+1 choices.
      neqn=eqn
      moel(moelpntr(nummodel)+1)=neqn
 
c     Re-arrange moel such that all potential outcomes are contiguously listed
c     after the position in xnode with the outcome expression.
      do 200 i=meno-1,moelpntr(nummodel)+3,-1
         if(moel(i).eq.-99)then
            outcome=moel(i+1)
            call copyintr(moel(moelpntr(nummodel)+4),
     *         moel(moelpntr(nummodel)+5),i-moelpntr(nummodel)-3)
            moel(moelpntr(nummodel)+4)=outcome
         endif
  200 continue
c     Check that there are no duplicates among the alternative outcomes:
      do 210 eqn=1,neqn
         i=moelpntr(nummodel)+2+eqn
         call intmatch(moel(i),moel(i+1),neqn-eqn,match)
         if(match.ne.0)goto 820
  210 continue
 
c     For multinomial probit, re-arrange moel such that the neqn potential
c     outcomes are followed by neqn pointers to where in moel model
c     specifications start.
      if(mtype.eq.72)then
         call copyintr(moel(moelpntr(nummodel)+neqn+3),
     *      moel(moelpntr(nummodel)+2*neqn+3),
     *      meno-moelpntr(nummodel)-neqn-2)
         meno=meno+neqn
         eqn=1
         moel(moelpntr(nummodel)+neqn+2+eqn)=moelpntr(nummodel)+2*neqn+3
         do 220 i=moelpntr(nummodel)+2*neqn+3,meno-1
            if(moel(i).eq.-99)then
               eqn=eqn+1
               moel(moelpntr(nummodel)+neqn+2+eqn)=i+1
            endif
  220    continue
         if(eqn.ne.neqn)call bug(ipt,jpt)
      endif
 
c     Frequency tables will be computed for selected variables.  Array
c     nfreq stores at which position in freqloc the locations of data
c     structure m start and how many such variables there are.
      numfreq=min(numfreq,maxnumfreq)
      if(nfreq(nummodel).gt.0)nfreq(nummodel) = nfreq(nummodel) +
     *   (numfreq-nfreq(nummodel)+1)*(maxnumfreq+1)
 
      goto 999
 
  800 if(ipt.gt.0)then
         call caretline(8,line,il)
         if(mtype.eq.71)write(8,801)'logit'
         if(mtype.eq.72)write(8,801)'probit'
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         if(mtype.eq.71)write(6,801)'logit'
         if(mtype.eq.72)write(6,801)'probit'
      endif
  801 format('ERROR encountered while reading a multinomial ',a,
     *   'model specification.',/,'The syntax is:',//,
     *   'MULTINOMIAL LOGIT MODEL; [DATA STRUCTURE = n;]',/,
     *   3x,'[(KEEP|DROP IF ...);]',/,
     *   3x,'[NUMERATOR;]  [DENOMINATOR;]',/,
     *   3x,'OUTCOME = ...;',/,
     *   3x,'MODEL n1 = ...;',/,
     *   3x,'MODEL n2 = ...;',/,
     *   3x,'MODEL  : = ...;',/,
     *   3x,'MODEL nk = ...;',//,
     *   'where n1 through nk are integers corresponding to outcomes ',
     *   'of interest;',/,'the omitted category consists of any other ',
     *   'outcome value(s).')
      stop
  810 if(ipt.gt.0)then
         call caretline(8,line,il)
         if(mtype.eq.71)write(8,811)'logit'
         if(mtype.eq.72)write(8,811)'probit'
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         if(mtype.eq.71)write(6,811)'logit'
         if(mtype.eq.72)write(6,811)'probit'
      endif
  811 format(/,'ERROR:  the outcome in a multinomial ',a,' model ',
     *   'specification does not',/,'involve a data variable, so that ',
     *   'aML does not know at what level (how',/,'often) the ',
     *   'equation contributes to the likelihood.')
      stop
 
  820 i=moelpntr(nummodel)+3
      if(ipt.gt.0)then
         if(mtype.eq.71)call writeaia(8,72,0,'ERROR: the model '//
     *      'specifications in a multinomial logit model contain '//
     *      'duplicate alternatives:  ',moel(i),';')
         if(mtype.eq.72)call writeaia(8,72,0,'ERROR: the model '//
     *      'specifications in a multinomial probit model contain '//
     *      'duplicate alternatives:  ',moel(i),';')
      endif
      if(jpt.gt.0)then
         if(mtype.eq.71)call writeaia(6,72,0,'ERROR: the model '//
     *      'specifications in a multinomial logit model contain '//
     *      'duplicate alternatives:  ',moel(i),';')
         if(mtype.eq.72)call writeaia(6,72,0,'ERROR: the model '//
     *      'specifications in a multinomial probit model contain '//
     *      'duplicate alternatives:  ',moel(i),';')
      endif
      do 825 i=moelpntr(nummodel)+4,moelpntr(nummodel)+1+neqn
         if(ipt.gt.0)call writeaia(8,0,0,' ',moel(i),';')
         if(jpt.gt.0)call writeaia(6,0,0,' ',moel(i),';')
  825 continue
      i=moelpntr(nummodel)+2+neqn
      if(ipt.gt.0)call writeaia(8,0,1,' ',moel(i),'.')
      if(jpt.gt.0)call writeaia(6,0,1,' ',moel(i),'.')
      stop
 
  830 if(ipt.gt.0)then
         call caretline(8,line,il)
         write(8,831)
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         write(6,831)
      endif
  831 format(/,'ERROR:  Multinomial probit models may have at most ',
     *   'four alternative',/,'outcomes, i.e., at most three model ',
     *   'equations.')
      stop
 
  840 if(ipt.gt.0)then
         call caretline(8,line,il)
         write(8,841)
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         write(6,841)
      endif
  841 format('ERROR: no (non-integrated) residual was specified in ',
     *   'a multinomial',/,'probit model.  By default, aML assumes an ',
     *   'IID N(0,1) residual in simple',/,'and ordered probit ',
     *   'models, because the residual is not identified.  No',/,
     *   'such assumption is made for multinomial probits.  While the ',
     *   'residual',/,'variance is not identified, the multinomial ',
     *   'probit allows estimation of',/,'correlation across ',
     *   'equations.  Even if you are not interested in this',/,
     *   'correlation, you must specify a (non-integrated) residual.')
      stop
 
  999 return
      end
