      subroutine read_par(levelmax,nfull,struc,maxmeno,moel,meno,numpar,
     *   pardim,parpos,nold,nxold,namepntr,maxxnode,xnodepos,maxlevel,
     *   numfreq,maxnumfreq,freqloc,xnode,parname,oldname,
     *   line,il,ipt,jpt)
 
c     Subroutine to read in a parameter or vector reference as part of a
c     model specification.
c
c     There are alternative ways of referencing a parameter:
c     Direct referencing:
c        PARAMETER parname
c        PARAMETER (REFERENCE = parname)   /* obsolete */
c     Indirect referencing:
c        PARAMETER (REFVAR = varname)
c     where varname may also be an integer expression.
c
c     Parameter names may also be elements of an inverse matrix:
c        PARameter INVerse(matrixname(i,j))
c
c     moel(1) = 2 this is a parameter
c     moel(2) = 1 reference is direct;
c               2 reference is direct of (inverse) matrix element
c               3 reference is indirect
c     moel(3)  = parameter rank number if moel(2)=1
c                parameter location (may be >nfull) if moel(2)=2
c                expression position in xnode if moel(2)=3
c
c     Constantijn Panis, 11 January 1998
c     Last change: new REFVAR machinery
 
      implicit none
      integer levelmax,nfull,struc,maxmeno,moel(maxmeno),meno,numpar,
     *   pardim(numpar),parpos(numpar),nxold(levelmax),nold,maxxnode,
     *   xnodepos,namepntr(levelmax),maxlevel,numfreq,maxnumfreq,
     *   freqloc(maxnumfreq),il,level,i1,i2,loc,i,ref,ipt,jpt,status,trm
      real xnode(maxxnode)
      logical smtxt,paren
      character parname(numpar)*12,oldname(*)*8,line*128,
     *   string*60,idstring*60
 
      if(meno+3.gt.maxmeno)call
     *   maxexceed('maxmeno',meno+3,maxmeno,ipt,jpt)
 
c     This is a parameter:
      meno=meno+1
      moel(meno)=2
 
      if(smtxt(4,line,il,'(REFVAR='))then
c        Indirectly referenced
         moel(meno+1)=3
         moel(meno+2)=xnodepos+1
         meno=meno+2
         call readexpr(1,levelmax,maxlevel,level,1,nold,maxxnode,
     *      xnodepos,xnode,nxold,namepntr,numfreq,maxnumfreq,freqloc,
     *      oldname,line,il,ipt,jpt,struc,')')
      else
c        Directly referenced:
         paren=smtxt(4,line,il,'(REFerence=')
         if(smtxt(4,line,il,'INVerse'))then
c           Inverse matrix element
            if(.not.smtxt(4,line,il,'('))goto 810
            call getword(4,line,il,i1,i2,status)
            if(status.gt.0)goto 810
            call char12match(line(i1:i2),parname,numpar,ref)
            if(ref.eq.0)goto 810
c           Flag that an element of the inverse matrix is used:
            pardim(ref)=abs(pardim(ref))
            if(pardim(ref).le.nfull)goto 820
            if(pardim(ref)/nfull.ne.mod(pardim(ref),nfull))goto 820
            if(.not.smtxt(4,line,il,'('))goto 810
            call getint(4,i1,line,il,status)
            if(status.gt.0)goto 810
            call getint(4,i2,line,il,status)
            if(status.gt.0)goto 810
            if(.not.smtxt(4,line,il,'))'))goto 810
c           Inverse matrix element; point at location in fullbeta.
c           Recall from define_par that
c              dim1 = abs(pardim)/nfull
c              dim2 = mod(abs(pardim),nfull)
            loc=nfull
            do 110 i=1,ref-1
               if(abs(pardim(i)).le.nfull)goto 110
               loc=loc+(abs(pardim(i))/nfull)*mod(abs(pardim(i)),nfull)
  110       continue
c           Here loc>nfull
            loc=loc+i1+(i2-1)*(pardim(ref)/nfull)
            moel(meno+1)=2
            moel(meno+2)=loc
            meno=meno+2
         else
c           Regular parameter, vector, or matrix element.
            call getword(4,line,il,i1,i2,status)
            if(status.gt.0)goto 800
            call char12match(line(i1:i2),parname,numpar,ref)
            if(ref.eq.0)goto 830
c           If the parameter is a parameter or vector, just record which
c           one it is.  If a matrix is referenced, read in which element
c           and record the location in fullbeta.
            if(abs(pardim(ref)).le.nfull)then
               moel(meno+1)=1
               moel(meno+2)=ref
               meno=meno+2
            else
c              Unusual case: element of a matrix (not inverse matrix)
               if(.not.smtxt(4,line,il,'('))goto 840
               call getint(4,i1,line,il,status)
               if(status.gt.0)goto 840
               call getint(4,i2,line,il,status)
               if(status.gt.0)goto 840
               if(.not.smtxt(4,line,il,')'))goto 840
c              Here loc<=nfull
               loc=parpos(ref)-1+i1+(i2-1)*(abs(pardim(ref))/nfull)
               moel(meno+1)=2
               moel(meno+2)=loc
               meno=meno+2
            endif
         endif
         if(paren)then
            if(.not.smtxt(4,line,il,')'))goto 800
         endif
      endif
      goto 999
 
  800 string=idstring(struc,0,0,0,0)
      if(ipt.gt.0)then
         call caretline(8,line,il)
         call wrapstring(8,0,72,0,1,'ERROR in the syntax of a '//
     *      'parameter reference in a model specification'//
     *      string(1:trm(string))//'  The syntax is:')
         write(8,801)
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         call wrapstring(6,0,72,0,1,'ERROR in the syntax of a '//
     *      'parameter reference in a model specification'//
     *      string(1:trm(string))//'  The syntax is:')
         write(6,801)
      endif
  801 format(6x,'PARAMETER parname',/,
     *   'or, in case the parameter is indirectly referenced:',/,
     *   6x,'PARAMETER (REFVAR = varname)',/,'where varname may ',
     *   'also be an integer expression.')
      stop
  810 if(ipt.gt.0)then
         call caretline(8,line,il)
         write(8,811)
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         write(6,811)
      endif
  811 format('ERROR in the reference if an inverse matrix element.  ',
     *   'The syntax is:',/,3x,'PAR INV(matrixname(i,j))')
      stop
  820 if(ipt.gt.0)then
         call caretline(8,line,il)
         write(8,821)
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         write(6,821)
      endif
  821 format('ERROR:  inverse matrix elements must refer to square ',
     *   'matrices.')
      stop
  830 il=i1
      if(ipt.gt.0)then
         call caretline(8,line,il)
         write(8,831)
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         write(6,831)
      endif
  831 format('ERROR:  parameter not defined.')
      stop
  840 if(ipt.gt.0)then
         call caretline(8,line,il)
         write(8,841)
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         write(6,841)
      endif
  841 format('ERROR in the reference if a matrix element.  ',
     *   'The syntax is:',/,3x,'PAR matrixname(i,j)')
      stop
  999 return
      end
