      subroutine read_restrict(dim,corrrestrict,dimp1,ncorr,line,il,
     *   ipt,jpt)
 
c     subroutine to read restrictions on a covariance structure and
c     to store this information in array corrrestrict.  The syntax is:
c        SIGMA(n1) = SIGMA(n2) = sigma(n3)
c        RHO(n1,n2) = RHO(n3,n4)
c        ;
c
c     Matrix corrrestrict must be initialized (see subroutine init_corr)
c     before this subroutine is called.  Element (1,1) is set to zero,
c     indicating that there are no restrictions imposed.  This routine
c     will reset it to 1+1*(dim+1).
c     Standard deviations may not be restricted be equal to correlations.
c     Integer variable ncorr is set to the number of parameters that
c     the distribution takes.  Each restriction reduces this number by 1.
c     It must be initialized in the calling routine, probably to
c     dim*(dim+1)/2.  Logical variable restrict (which must also be
c     initialized by the calling routine) is set to .true. if any
c     restrictions are imposed.
c
c     Constantijn Panis, 22 June 1994
c     Last change: smtxt rather than sametext
 
      implicit none
      integer dim,corrrestrict(dim,dim),dimp1,ncorr,n1,n2,n3,n4,
     *   il,ipt,jpt,status
      logical smtxt
      character line*128
 
c     There may not be any restrictions:
      if(smtxt(4,line,il,';'))goto 999
 
c     Indicate that there are restrictions:
      corrrestrict(1,1)=1+dimp1
 
  100 if(smtxt(4,line,il,';'))goto 999
      call get_element(n1,n2,dim,line,il,status)
      if(status.gt.0)goto 800
      if(.not.smtxt(4,line,il,'='))goto 800
      call get_element(n3,n4,dim,line,il,status)
      if(status.gt.0)goto 800
      if(n1.eq.n3.and.n2.eq.n4)goto 800
c     Do not allow to restrict standard deviations to be equal to
c     correlation coefficients:
      if((n1.eq.n2.and.n3.ne.n4).or.(n1.ne.n2.and.n3.eq.n4))goto 800
c     Do not allow restrictions of parameters that are out of range:
      if(n1.gt.dim.or.n2.gt.dim.or.n3.gt.dim.or.n4.gt.dim)goto 800
      if(n1.eq.n2)then
c        SIGMA(n3) is equal to SIGMA(n1)
         if(corrrestrict(n3,n3).ne.n3+n3*dimp1)goto 800
         corrrestrict(n3,n3) = n1+n1*dimp1
      else
c        RHO(n3,n4) is equal to RHO(n1,n2); store in lower(!) triangle
         if(corrrestrict(n4,n3).ne.n4+n3*dimp1)goto 800
         corrrestrict(n4,n3) = n2+n1*dimp1
      endif
      ncorr=ncorr-1
  200 if(.not.smtxt(4,line,il,'='))goto 100
      call get_element(n3,n4,dim,line,il,status)
      if(status.gt.0)goto 800
      if(n1.eq.n3.and.n2.eq.n4)goto 800
      if((n1.eq.n2.and.n3.ne.n4).or.(n1.ne.n2.and.n3.eq.n4))goto 800
      if(n3.gt.dim.or.n4.gt.dim)goto 800
      if(n1.eq.n2)then
c        SIGMA(n3) is equal to SIGMA(n1)
         if(corrrestrict(n3,n3).ne.n3+n3*dimp1)goto 800
         corrrestrict(n3,n3) = n1+n1*dimp1
      else
c        RHO(n3,n4) is equal to RHO(n1,n2); store in lower(!) triangle
         if(corrrestrict(n4,n3).ne.n4+n3*dimp1)goto 800
         corrrestrict(n4,n3) = n2+n1*dimp1
      endif
      ncorr=ncorr-1
      goto 200
 
  800 if(ipt.gt.0)then
         call caretline(8,line,il)
         write(8,801)
      endif
      if(jpt.gt.0)then
         call caretline(8,line,il)
         write(6,801)
      endif
  801 format('ERROR in a RESTRICTION of a covariance structure.  The ',
     *   'syntax is:',/,6x,'RESTRICTIONS SIGMA(n1) = SIGMA(n2) = ',
     *   'SIGMA(n3) = ...',/,19x,'RHO(n1,n2) = RHO(n3,n4) = RHO(n5,n6)',
     *   ' = ...',/,19x,';',/,'Either you attempt to restrict a ',
     *   'standard deviation to be equal to a',/,'correlation ',
     *   'coefficient, or you attempt to restrict a standard deviation',
     *   /,'or correlation coefficient to be equal to itself, or you ',
     *   'attempt to',/,'restrict a nonexistent standard deviation or ',
     *   'correlation coefficient, or',/,'you attempt to restrict the ',
     *   'same standard deviation or correlation',/,
     *   'coefficient twice, or you made a syntax error.')
      stop
  999 return
      end
