      subroutine readexpr(task,levelmax,maxlevel,level,numexpr,nold,
     *   maxxnode,xnodepos,xnode,nxold,namepntr,numfreq,maxnumfreq,
     *   freqloc,oldname,line,il,ipt,jpt,struc,endmark)
 
c     Subroutine to read in one or more expressions, such as
c       x1 + (x2==98) - 2*x3 + (x4<100-x5) + (log(x6)>0);
c       x2  log(x3)  x3/5  spline(x4, 30 55);
c
c     The expression(s) must be delimited by endmark.
c
c     This routine stores variable names in one of two ways.  If
c     task=1  the data structure of the variable is known, and
c             the variable's level and position (relative to
c             namepntr(level) is recorded as level*(nold+1)+varpos.
c     task=2  the data structure is not yet known.  Only the variable
c             name is stored as the position in oldname (relative
c             to the first element of oldname) in which the variable
c             first appears.
c
c     levelmax [in] number of levels in the data
c     maxlevel [in] (only relevant if task=1) is the lowest permissible
c           level of variables entering in the expression; variables
c           below this level result in an error.  If maxlevel<0, the
c           variable(s) must (all) be at level abs(maxlevel)
c     level [out] (only relevant if task=1) is the lowest level of any
c           variable actually present.  If the expression contains no
c           variables, level=0.
c
c     numexpr [in/out]
c       0 = read next expression(s) and return the number read as
c           numexpr.  Note: a spline is counted as n+1 expressions,
c           where n is the number of nodes.
c      >0 = must read numexpr expressions, ending in endmark
c
c     Constantijn Panis, 23 March 1998
 
      implicit none
      integer task,levelmax,maxlevel,level,numexpr,nold,il,i1,i2,
     *   maxxnode,nexpr,varpos,lvl,nxold(levelmax),namepntr(levelmax),
     *   numfreq,maxnumfreq,freqloc(maxnumfreq),oper,prev,ipt,jpt,struc,
     *   xnodepos,numnodes,spline,splinenest,minmax,minmaxnest,nest,
     *   mmc,status,previl,prevnest
      real xnode(maxxnode),x,varnext,opernext,enditem,endexpr,endexprs
      character oldname(nold)*8,line*128,endmark*(*)
      logical smtxt
c     Maximum level of nesting at which min/max may appear:
      parameter(mmc=20)
 
      nexpr=0
      if(task.eq.1)level=0
      numnodes=0
      spline=0
      splinenest=0
      minmax=0
      minmaxnest=0
      nest=0
      prev=0
      oper=0
 
  100 prev=oper
      previl=il
      prevnest=nest
      if(smtxt(4,line,il,endmark))then
         if(endmark.eq.';'.or.endmark.eq.');')then
c           Semicolon: always end of expression
            goto 200
         elseif(endmark.eq.')')then
c           End of expression?
            if(nest.eq.0)goto 200
c           Just a closing parenthesis, part of expression
            oper=39
            nest=nest-1
         elseif(endmark.eq.',')then
c           Comma may mark the end or just be part of a min/max
c           expression.  The comma will never separate two expressions
c           (including splines), as in those cases the comma will not
c           be an endmark.
            if(numexpr.gt.1)call bug(ipt,jpt)
            if(minmax.gt.0)then
c              Currently reading min(...) or max(...) expression;
c              deal with it below through prev=70.
               if(prev.eq.70.or.prev.eq.71)goto 800
               oper=70
               goto 100
            else
c              End of expression
               goto 200
            endif
         elseif(endmark.eq.'(-INF='.or.endmark.eq.'(INF=')then
c           Always end of expression
            if(numexpr.ne.1)call bug(ipt,jpt)
            goto 200
         else
            call bug(ipt,jpt)
         endif
      else
         call getoper(4,line,il,i1,i2,oper,x,nest,ipt,jpt)
      endif
      if(xnodepos+2.gt.maxxnode)call
     *   maxexceed('maxxnode',xnodepos+2,maxxnode,ipt,jpt)
 
      if(oper.eq.0)then
         if(endmark.ne.';')goto 800
         call bug(ipt,jpt)
      elseif(oper.eq.26)then
c        Spline expression; may not be used where only one expression
c        is expected.
         if(numexpr.eq.1)goto 880
c        May not be nested in other splines:
         if(spline.ne.0)goto 830
c        May not be interacted with other splines.
         if(numnodes.gt.0.and.prev.ne.10.and.prev.ne.1.and.
     *      prev.ne.39.and.prev.ne.70)goto 860
      elseif(oper.eq.70)then
c        Comma may be used to separate expressions, to separate nodes
c        in a spline, or arguments in min(...) or max(...).
c        Deal with it below (through prev=70), recognizing that a new
c        (sub-)expression starts.
         if(prev.eq.70.or.prev.eq.71)goto 800
         goto 100
      elseif(oper.eq.71)then
c        nodes=, must be part of spline(var=..., nodes=...)
         if(spline.eq.0)goto 800
c        Must have been preceded by a comma
         if(prev.ne.70)goto 810
c        Ignore the nodes= statement.
         oper=70
         goto 100
      endif
 
c     Unbalanced parentheses?
      if(nest.lt.0)goto 820
 
c     New expression if {varname, real, (, or ???(} follow
c     {varname, real, or )} or if anything follows a comma.
      if((oper.eq.1.or.oper.eq.10.or.oper.eq.11.or.
     *   (20.le.oper.and.oper.lt.39)).and.
     *   (prev.eq.1.or.prev.eq.10.or.prev.eq.11.or.prev.eq.39).or.
     *   prev.eq.70)then
         if(spline.gt.0.and.prevnest.eq.splinenest+1)then
c           Currently reading spline definition; new expression
c           generates new node.  Insert end-of-item.
            numnodes=numnodes+1
            xnodepos=xnodepos+1
            xnode(xnodepos)=enditem()
         elseif(minmax.gt.0)then
c           Currently reading min(...) or max(...).
c           Insert end-of-item.
            xnodepos=xnodepos+1
            xnode(xnodepos)=enditem()
         else
            if(prevnest.ne.0)goto 820
            if(numexpr.eq.1)goto 880
            nexpr=nexpr+numnodes+1
            numnodes=0
c           Insert end-of-expression
            xnodepos=xnodepos+1
            xnode(xnodepos)=endexpr()
c           Record operator/variable/number and keep reading
         endif
      endif
 
c     Record the current operator/variable/number and
c     detect and check for error.
      if(oper.eq.1)then
c        Real number: simply place in xnode
         xnodepos=xnodepos+1
         xnode(xnodepos)=x
      elseif(oper.eq.10)then
c        Variable name; place varnext and position in xnode
         xnodepos=xnodepos+1
         xnode(xnodepos)=varnext()
c        Method of storage depends on whether the data structure is known.
         if(task.eq.1)then
            call findvar(levelmax,maxlevel,lvl,line(i1:i2),oldname,
     *         namepntr,nxold,varpos,line,il,struc,ipt,jpt,status)
            if(status.gt.0)stop
            varpos=lvl*(nold+1)+varpos
            xnodepos=xnodepos+1
            xnode(xnodepos)=varpos
            level=max(level,lvl)
c           Add variable to frequency tables, if desired.
            call addvarfreq(maxnumfreq,numfreq,freqloc,varpos)
        else
            call char8match(line(i1:i2),oldname,nold,varpos)
            if(varpos.eq.0)goto 850
            xnodepos=xnodepos+1
            xnode(xnodepos)=varpos
         endif
      elseif(oper.eq.11)then
c        uniform(); treat as variable name, varpos=-1
         xnodepos=xnodepos+1
         xnode(xnodepos)=varnext()
         xnodepos=xnodepos+1
         xnode(xnodepos)=-1
      else
         if((prev.ne.1.and.prev.ne.10.and.prev.ne.11.and.prev.ne.39)
     *      .and.(39.le.oper.and.oper.le.61.and.oper.ne.43.and.
     *      oper.ne.44))goto 830
         xnodepos=xnodepos+1
         xnode(xnodepos)=opernext()
         xnodepos=xnodepos+1
         xnode(xnodepos)=real(oper)
         if(oper.eq.26)then
c           Start of a spline
            spline=1
            splinenest=prevnest
         elseif(oper.eq.28.or.oper.eq.29)then
c           Start of a min(...) or max(...)
            if(nest.ge.mmc)goto 840
            minmax=minmax+1
            if(minmax.ge.6)goto 840
            minmaxnest=mmc*minmaxnest+prevnest
         elseif(oper.eq.39)then
            if(spline.eq.1.and.nest.eq.splinenest)then
c              End of a spline
               spline=0
            elseif(minmax.gt.0.and.nest.eq.mod(minmaxnest,mmc))then
c              End of a min or max
               minmax=minmax-1
               minmaxnest=minmaxnest/mmc
            endif
         endif
      endif
      goto 100
 
c     End of expression(s).
c     Check that the expression was not empty:
  200 if(prev.eq.0)goto 800
c     Check that parentheses match:
      if(nest.ne.0)goto 820
      if(spline.ne.0)goto 800
c     Check that the number of expressions read is as expected:
      nexpr=nexpr+numnodes+1
      if(numexpr.eq.0)then
         numexpr=nexpr
      elseif(numexpr.eq.1)then
         if(nexpr.ne.1)goto 830
      elseif(numexpr.gt.0)then
         if(nexpr.ne.numexpr)goto 870
      else
         call bug(ipt,jpt)
      endif
c     Mark end of expression(s)
      xnodepos=xnodepos+1
      xnode(xnodepos)=endexprs()
      goto 999
 
  800 il=min(previl,il-1)
      if(ipt.gt.0)then
         call caretline(8,line,il)
         write(8,801)
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         write(6,801)
      endif
  801 format('ERROR:  unexpected symbol or string.')
      stop
  810 il=min(previl,il-1)
      if(ipt.gt.0)then
         call caretline(8,line,il)
         write(8,811)
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         write(6,811)
      endif
  811 format('ERROR in spline syntax:  spline(var=varname, nodes=...);',
     *   /,'Note that a comma is required to separate the variable ',
     *   'from the nodes.')
      stop
c     Unbalanced parentheses
  820 il=il-1
      if(ipt.gt.0)then
         call caretline(8,line,il)
         write(8,821)
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         write(6,821)
      endif
  821 format('ERROR:  unbalanced parentheses.')
      stop
  830 il=min(previl,il-1)
      if(ipt.gt.0)then
         call caretline(8,line,il)
         write(8,831)
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         write(6,831)
      endif
  831 format('ERROR:  invalid expression.')
      stop
  840 il=min(previl,il-1)
      if(ipt.gt.0)then
         call caretline(8,line,il)
         write(8,841)
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         write(6,841)
      endif
  841 format(/,'ERROR: MIN(...) or MAX(...) is too deeply nested.')
      stop
  850 il=il-(i2-i1+1)
      if(ipt.gt.0)then
         call caretline(8,line,il)
         write(8,851)line(i1:i2)
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         write(6,851)line(i1:i2)
      endif
  851 format(/,'ERROR:  variable ''',a,''' not found in the data.')
      stop
  860 il=il-7
      if(ipt.gt.0)then
         call caretline(8,line,il)
         write(8,861)
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         write(6,861)
      endif
  861 format('ERROR:  splines may not be interacted with other splines')
      stop
  870 il=min(previl,il-1)
      if(ipt.gt.0)then
         call caretline(8,line,il)
         call writea2ia(8,72,1,'ERROR: ',numexpr,' variables or '//
     *      'expressions expected, ',nexpr,' read.')
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         call writea2ia(6,72,1,'ERROR: ',numexpr,' variables or '//
     *      'expressions expected, ',nexpr,' read.')
      endif
      stop
  880 il=min(previl+1,il-1)
      if(ipt.gt.0)then
         call caretline(8,line,il)
         write(8,881)endmark
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         write(6,881)endmark
      endif
  881 format('ERROR:  expected delimiter ''',a,'''.')
      stop
  999 return
      end
 
