      subroutine scanctl(convcrit,nx1,stprng,output,ipt,jpt,version,
     *   nobs,datanobs,maxiter,wghtpos,maxdraws,maxmeno,maxnmapmoel,
     *   numdist,numrset,numpar,numspl,nummodel,dimmax,sumdim,
     *   sumdim2,sumdim4,
     *   ngrid,nfull,nfullplus,maxds,vdimmax,maxfreqcat,
     *   maxnfreq,maxnode,maxref,maxxnode,maxmspl,useid,useobs,prof,
     *   sparsity,numsearch,detail,check99999,posdef,ieee,oldname,
     *   title,scrfile,gridfile,line)
 
c     Subroutine to scan the control file for some maximum dimensions
c     and all optional values.  If a syntax error is encountered, just
c     return; it will be caught and properly written out in readaml.
c
c     Constantijn Panis, 12 March 2003
c     Last change: remove maxpnts
 
      implicit none
      double precision convcrit(4),tmp
      integer nx1,stprng(3),output(10),ipt,jpt,version,nobs,datanobs,
     *   maxiter,wghtpos,maxdraws,maxmeno,maxnmapmoel,numdist,
     *   numrset,numpar,numspl,nummodel,dimmax,sumdim,sumdim2,
     *   sumdim4,ngrid,nfull,nfullplus,maxds,
     *   vdimmax,maxfreqcat,maxnfreq,maxnode,maxref,maxxnode,maxmspl,
     *   useid,useobs,prof,sparsity,numsearch,
     *   il,intpnts,dim,dim1,dim2,i1,i2,posdef,status
      logical detail,check99999,ieee,smtxt
      character oldname(nx1)*8,title*80,scrfile*128,gridfile*128,
     *   line*128
 
c     Initialize (guaranteed to work by getdatainfo)
      rewind 4
      read(4,'(a80)',end=800,err=800)line
      il=1
 
c     ngrid = number of parameters over which grid search is requested
c     nfull = number of parameters in the model
c     nfullplus = nfull plus space for inverse matrices
c     nummodel = number of model statements
c     numdist = number of definitions of distributions
c     numpar = number of definitions of parameters (vectors)
c     numrset = number of definitions of regressor sets
c     numspl = number of definitions of splines
c     dimmax = maximum dimension of any distribution
c     sumdim = sum over all distributions of dim
c     sumdim2 = sum over all distributions of dim*dim
c     sumdim4 = sum over all distributions of dim*dim*dim*dim
      ngrid=0
      nfull=0
      nfullplus=0
      nummodel=0
      numdist=0
      numpar=0
      numrset=0
      numspl=0
      dimmax=0
      sumdim=0
      sumdim2=0
      sumdim4=0
 
c     To the extent that it is of interest, the syntax is:
c
c     option ...
c     define spline [name]; ...
c     define regressor set [name]; ...
c     define vector [name]; ...
c     define matrix name; ...
c     define normal distribution; dim=..; ...
c     define ARMA(p,q) distribution; dim=..; ...
c     define CARMA(p,q) distribution; dim=..; ...
c     define finite mixture distribution; dim=..;
c     hazard model ...
c     logit model ...
c     ordered logit model ...
c     multinomial logit model ...
c     multinomial probit model ...
c     probit model ...
c     ordered probit model ...
c     tobit model ...
c     continuous model ...
c     binomial model ...
c     negative binomial model ...
c     Poisson model ...
c     starting values; ... name {T|F} grid(begin,end,number) ...
 
 
  100 if(smtxt(4,line,il,'OPTION'))then
c        All optional parameters are determined here:
         call readoption(convcrit,version,nx1,stprng,output,nobs,
     *      datanobs,maxiter,wghtpos,maxmeno,maxnmapmoel,maxds,vdimmax,
     *      maxfreqcat,maxdraws,maxnfreq,maxnode,maxref,maxxnode,
     *      maxmspl,useid,useobs,prof,sparsity,numsearch,detail,
     *      check99999,posdef,ieee,oldname,title,scrfile,gridfile,
     *      line,il,ipt,jpt)
         goto 100
      elseif(smtxt(4,line,il,'DEFINE'))then
         if(smtxt(4,line,il,'NORMal DISTribution;'))then
            numdist=numdist+1
c           Also get the dimension and number of integration points, if any:
            if(.not.smtxt(4,line,il,'DIM='))goto 810
            call getint(4,dim,line,il,status)
            if(status.gt.0)goto 810
  110       call gotochar(4,';',line,il,status)
            if(status.gt.0)goto 810
            if(smtxt(4,line,il,'NAME ='))goto 110
            if(smtxt(4,line,il,'REF ='))goto 110
            if(smtxt(4,line,il,'RESTRICTION'))goto 110
            if(smtxt(4,line,il,'SEARCH='))goto 110
            if(smtxt(4,line,il,'NUMber of INTegration POINTS ='))then
  120          if(.not.smtxt(4,line,il,';'))then
                  call getint(4,intpnts,line,il,status)
                  if(status.gt.0)goto 810
                  goto 120
               endif
            endif
            dimmax=max(dimmax,dim)
            sumdim=sumdim+dim
            sumdim2=sumdim2+dim*dim
            sumdim4=sumdim4+dim*dim*dim*dim
c           All possible specifications have been covered:
            goto 100
         elseif(smtxt(4,line,il,'FINite MIXture DISTribution;'))then
            numdist=numdist+1
c           Also get the dimension:
            if(.not.smtxt(4,line,il,'DIM='))goto 820
            call getint(4,dim,line,il,status)
            if(status.gt.0)goto 820
            dimmax=max(dimmax,dim)
            sumdim=sumdim+dim
            sumdim2=sumdim2+dim*dim
            sumdim4=sumdim4+dim*dim*dim*dim
         elseif(smtxt(4,line,il,'REGressor SET'))then
            numrset=numrset+1
         elseif(smtxt(4,line,il,'SPLINE'))then
            numspl=numspl+1
         elseif(smtxt(4,line,il,'PARameter'))then
            numpar=numpar+1
         elseif(smtxt(4,line,il,'VECTOR'))then
            numpar=numpar+1
         elseif(smtxt(4,line,il,'MATRIX'))then
            numpar=numpar+1
c           Must get dimension (n1,n2) and allocated space to store
c           the inverse matrix.
            call gotochar(4,';',line,il,status)
            if(status.gt.0)goto 830
            if(.not.smtxt(4,line,il,'DIMension='))goto 830
            if(.not.smtxt(4,line,il,'('))goto 830
            call getint(4,dim1,line,il,status)
            if(status.gt.0)goto 830
            call getint(4,dim2,line,il,status)
            if(status.gt.0)goto 830
            if(.not.smtxt(4,line,il,')'))goto 830
            nfullplus=nfullplus+dim1*dim2
         elseif(smtxt(4,line,il,'AR(').or.smtxt(4,line,il,'MA(').or.
     *      smtxt(4,line,il,'ARMA(').or.
     *      smtxt(4,line,il,'Cumulative AR(').or.
     *      smtxt(4,line,il,'Cumulative MA(').or.
     *      smtxt(4,line,il,'Cumulative ARMA('))then
            numdist=numdist+1
            dim=1
            dimmax=max(dimmax,dim)
            sumdim=sumdim+dim
            sumdim2=sumdim2+dim*dim
            sumdim4=sumdim4+dim*dim*dim*dim
         else
c           Syntax error; let readctl catch it
            continue
         endif
      elseif(smtxt(4,line,il,'HAZARD MODEL'))then
         nummodel=nummodel+1
      elseif(smtxt(4,line,il,'LOGIsTic MODEL'))then
         nummodel=nummodel+1
      elseif(smtxt(4,line,il,'ORDered LOGIsTic MODEL'))then
         nummodel=nummodel+1
      elseif(smtxt(4,line,il,'MultiNomial LOGIsTic MODEL'))then
         nummodel=nummodel+1
      elseif(smtxt(4,line,il,'MultiNomial PROBIT MODEL'))then
         nummodel=nummodel+1
      elseif(smtxt(4,line,il,'PROBIT MODEL'))then
         nummodel=nummodel+1
      elseif(smtxt(4,line,il,'ORDered PROBIT MODEL'))then
         nummodel=nummodel+1
      elseif(smtxt(4,line,il,'CONTINUOUS MODEL'))then
         nummodel=nummodel+1
      elseif(smtxt(4,line,il,'TOBIT MODEL'))then
         nummodel=nummodel+1
      elseif(smtxt(4,line,il,'BINomial MODEL'))then
         nummodel=nummodel+1
      elseif(smtxt(4,line,il,'NEGative BINomial MODEL'))then
         nummodel=nummodel+1
      elseif(smtxt(4,line,il,'POISSON MODEL'))then
         nummodel=nummodel+1
      elseif(smtxt(4,line,il,'STARTING VALUES;'))then
         nfull=0
  140    if(smtxt(4,line,il,';'))goto 999
         call getstring(4,line,il,i1,i2)
         if(i2.eq.-1)goto 999
         call getstring(4,line,il,i1,i2)
         if(i2.eq.-1)goto 999
         if(smtxt(4,line,il,'GRID('))then
c           Syntax:   parname {T|F}  grid(min,max,num)
            ngrid=ngrid+1
            call getdp(4,tmp,line,il,status)
            if(status.ne.0)goto 999
            call getdp(4,tmp,line,il,status)
            if(status.ne.0)goto 999
            call getint(4,i1,line,il,status)
            if(status.ne.0)goto 999
            if(.not.smtxt(4,line,il,')'))goto 999
         else
            call getstring(4,line,il,i1,i2)
            if(i2.eq.-1)goto 999
         endif
         nfull=nfull+1
         nfullplus=nfullplus+1
         goto 140
      endif
c     Either the first part of a definition or model statement has just
c     been read in, or parameters related to such statements are being
c     set, or there is a syntax error.  Read until end of statement.
      call gotochar(4,';',line,il,status)
      if(status.gt.0)then
c        Control file entirely read.
         if(nfull.eq.0)goto 840
         goto 999
      endif
      goto 100
 
  800 call bug(ipt,jpt)
  810 if(ipt.gt.0)then
         call caretline(8,line,il)
         write(8,811)
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         write(6,811)
      endif
  811 format('ERROR encountered in the definition of a normal ',
     *   'distribution.  The syntax is:',//,
     *   3x,'DEFINE NORMAL DISTRIBUTION;  DIM = n;',/,
     *   6x,'[NUMBER OF INTEGRATION POINTS = n [n...n];]',/,
     *   6x,'NAME = [name]; [REF = ...;]',/,9x,':',/,
     *   6x,'NAME = [name]; [REF = ...;]',/,
     *   6x,'[RESTRICTIONS [sigma(n)=sigma(n) rho(n,n)=rho(n,n)];]',/,
     *   6x,'[SEARCH=CHOLESKY;]')
      stop
  820 if(ipt.gt.0)then
         call caretline(8,line,il)
         write(8,821)
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         write(6,821)
      endif
  821 format(/,'ERROR encountered in the definition of a finite ',
     *   'mixture',/,'definition.  The syntax is:',//,
     *   'DEFINE FINITE MIXTURE DISTRIBUTION;  ',
     *   'DIM = 1;',/,3x,'FORM = ASYMMETRIC;',/,
     *   3x,'POINTS = vector;  WEIGHTS = vector;',/,
     *   3x,'NAME = [name]; [REF = ...;]')
      stop
  830 if(ipt.gt.0)then
         call caretline(8,line,il)
         write(8,833)
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         write(6,833)
      endif
  833 format('ERROR encountered in a MATRIX definition.  The ',
     *   'syntax is:',//,
     *   6x,'DEFINE MATRIX matrixname;  DIM = (n1,n2);')
      stop
  840 if(ipt.gt.0)write(8,841)
      if(jpt.gt.0)write(6,841)
  841 format(/,'ERROR:  aML did not read any starting values.  All ',
     *   'model parameters must',/,'be initialized with a "starting ',
     *   'values" statement.  If you specified',/,'such a statement, ',
     *   'did you end the preceding statement with a semicolon?')
      stop
  999 return
      end
