      subroutine searchmax(maxcc,ccmax,nfull,nfree,stprng,fulltype,
     *   iffull,ipt,jpt,fullbeta,newfullbeta,searchdir,nchol,chol,
     *   dfreebeta,fp,fp2,fpt,fpt2,oldlnl,lnl,stepsize,locdim,
     *   dd,rr,ii,ll,cc,status)
 
c     Subroutine to search for maximum (amlfunc) function value.
c
c     Upon input, a0 is the current parameter vector.
c     If function value failed to improve, the first alternate return
c     is used and fullbeta is not affected (so that it is the best set
c     of parameters, given failure of this routine.)  If an improved
c     value was found, the best function value will be in lnl (oldlnl
c     is not changed); the best parameters will be in fullbeta, the
c     final step size in stepsize.
c
c     Note:  status fulfills two roles.  During execution of this subroutine,
c     status = -1: there has been at least one attempt, never improved
c               0: this is the first attempt
c               1: there has been at least one attempt, always improved
c     Upon return, status=0 indicates that a next maximum was found;
c                  status=1 indicates that the function failed to improve.
c
c     Constantijn Panis, 8 July 1995
 
      implicit none
      integer maxcc,ccmax
      double precision dd(maxcc/8)
      real rr(maxcc/4)
      integer ii(maxcc/4),locdim(120)
      logical ll(maxcc/4)
      character cc(maxcc)*1
      integer nfull,nfree,stprng(3),fulltype(nfull),iffull(nfull),
     *   ipt,jpt,nchol,status,status1,step,skip,trm,intlen
      double precision fullbeta(nfull),newfullbeta(nfull),
     *   searchdir(nfree),chol(nchol),dfreebeta(nfree),fp(nfree),
     *   fp2(nfree),fpt(nfree),fpt2(nfree),
     *   oldlnl,lnl,lnltry,factor,stepsize,dpnan
      logical worse,isnan
      character fmt*24,i2c*12
 
      status = 0
      lnl = oldlnl
 
      if(oldlnl.gt.-1.d8)then
         if(ipt.ge.3)write(8,101)oldlnl
         if(jpt.ge.3)write(6,101)oldlnl
      else
         if(ipt.ge.3)write(8,102)oldlnl
         if(jpt.ge.3)write(6,102)oldlnl
      endif
c     if(ipt.ge.3)call flush(8)
      if(ipt.ge.3)call flush(8)
  101 format(/,'SEARCHING TO IMPROVE FUNCTION VALUE (OLD LN-L =',
     *   f16.6,'):')
  102 format(/,'SEARCHING TO IMPROVE FUNCTION VALUE (OLD LN-L =',
     *   1pe16.8,'):')
 
c     With option "save step size", the step size is kept at the best
c     value of the preceding iteration.
      if(stprng(3).eq.0)stepsize=1.d0
      factor = stepsize
 
  100 call fignewbeta(1,nfull,nfree,fulltype,iffull,fullbeta,searchdir,
     *   factor,newfullbeta,0.d0,nchol,chol,status1)
      if(status1.eq.0)then
         call amlfunc(100,maxcc,ccmax,nfull,nfree,newfullbeta,fulltype,
     *      iffull,lnltry,nchol,chol,dfreebeta,fp,fp2,fpt,fpt2,
     *      0.d0,0.d0,locdim,dd,rr,ii,ll,cc,ipt,jpt,1)
      else
         lnltry=dpnan()
      endif
      if(.not.isnan(lnltry))goto 120
 
c     Function evaluation error; parameters went into illegal region.
c     Try to save the run by halving the stepsize.
      rewind 13
 
c     Write out the result of this step:
  120 if(factor.eq.int(factor))then
         step=int(factor)
         fmt='(18x,a,i'//i2c(intlen(step)+2,0)
         skip=max(8-intlen(step),2)
         fmt=fmt(1:trm(fmt))//','//i2c(skip,0)
         if(isnan(lnltry))then
            fmt=fmt(1:trm(fmt))//'x,a)'
            if(ipt.ge.3)write(8,fmt)'STEPSIZE:',step,'NEW LN-L =   NaN'
            if(jpt.ge.3)write(6,fmt)'STEPSIZE:',step,'NEW LN-L =   NaN'
         elseif(lnltry.gt.-1.d8)then
            fmt=fmt(1:trm(fmt))//'x,a,f16.6)'
            if(ipt.ge.3)write(8,fmt)'STEPSIZE:',step,'NEW LN-L =',lnltry
            if(jpt.ge.3)write(6,fmt)'STEPSIZE:',step,'NEW LN-L =',lnltry
         else
            fmt=fmt(1:trm(fmt))//'x,a,1pe16.8)'
            if(ipt.ge.3)write(8,fmt)'STEPSIZE:',step,'NEW LN-L =',lnltry
            if(jpt.ge.3)write(6,fmt)'STEPSIZE:',step,'NEW LN-L =',lnltry
         endif
      else
         step=int(1.d0/factor)
         fmt='(18x,a,i'//i2c(intlen(step),0)
         skip=max(6-intlen(step),2)
         fmt=fmt(1:trm(fmt))//','//i2c(skip,0)
         if(isnan(lnltry))then
            fmt=fmt(1:trm(fmt))//'x,a)'
            if(ipt.ge.3)write(8,fmt)'STEPSIZE:  1/',step,
     *         'NEW LN-L =   NaN'
            if(jpt.ge.3)write(6,fmt)'STEPSIZE:  1/',step,
     *         'NEW LN-L =   NaN'
         elseif(lnltry.gt.-1.d8)then
            fmt=fmt(1:trm(fmt))//'x,a,f16.6)'
            if(ipt.ge.3)write(8,fmt)'STEPSIZE:  1/',step,'NEW LN-L =',
     *         lnltry
            if(jpt.ge.3)write(6,fmt)'STEPSIZE:  1/',step,'NEW LN-L =',
     *         lnltry
         else
            fmt=fmt(1:trm(fmt))//'x,a,1pe16.8)'
            if(ipt.ge.3)write(8,fmt)'STEPSIZE:  1/',step,'NEW LN-L =',
     *         lnltry
            if(jpt.ge.3)write(6,fmt)'STEPSIZE:  1/',step,'NEW LN-L =',
     *         lnltry
         endif
      endif
c     if(ipt.ge.3)call flush(8)
      if(ipt.ge.3)call flush(8)
 
c     If new function value is worse or NaN:
c        if previous attempt was better, take previous and leave
c        else half stepsize
c     If new function value is better:
c        if previous attempts were worse, take it and leave
c        else double stepsize
c     Alpha compiler does not allow comparisons of NaN and is not sufficiently
c     smart to bypass the second condition if the first is true:
c     if(isnan(lnltry).or.lnltry.le.lnl)then
      worse=isnan(lnltry)
      if(.not.worse)worse=(lnltry.le.lnl)
      if(worse)then
         if(status.eq.1)then
c           Place best parameter values (found in previous round) in fullbeta:
            call fignewbeta(1,nfull,nfree,fulltype,iffull,fullbeta,
     *         searchdir,stepsize,newfullbeta,0.d0,nchol,chol,status1)
            call copy8(newfullbeta,fullbeta,nfull)
            status=0
            goto 999
         else
c           Worse; half step size, if permitted
            if(factor.le.2.d0**stprng(1))then
c              Step size becomes too small: failure to improve.
               status=1
               goto 999
            endif
            factor = .5d0*factor
            status=-1
            goto 100
         endif
      else
c        Function is better:
         stepsize=factor
         lnl=lnltry
         if(status.ge.0)then
c           Improving; double step size, if permitted.
            if(factor.ge.2.d0**stprng(2))then
c              Function kept improving, but step size exceeded:
               call copy8(newfullbeta,fullbeta,nfull)
               status=0
               goto 999
            endif
            factor = factor*2.d0
            status=1
            goto 100
         else
c           First time function improves after multiple attempts; take it
            call copy8(newfullbeta,fullbeta,nfull)
            status=0
            goto 999
         endif
      endif
 
  999 return
      end
 
