      logical function smtxt(unit,line,begin,text)
 
c     Logical function to check that the next text in line is equal
c     to character string text.  This routine differs from sametext
c     in that the next line is automatically read if needed (i.e.,
c     wrap=.true.) and in the ability to distinguish optional characters
c     from required.  Sametext is case insensitive; smtxt is also,
c     but in providing the argument (text), lower case letters are
c     optional, uppercase required.  For example, if text='REFerence',
c     'REF' will be checked and determines whether smtxt is true or false;
c     then 'eference' is also checked and "begin" is positioned after
c     the entire string, if matched.  Non-alpha characters are always
c     required.
c     This routine treats a backslash just like every other nonwhite
c     character.
c
c     Upon input, line(begin:begin) is the first character that is being
c     considered.  Upon output, line(begin:begin) is either the first
c     nonwhite character (if smtxt=.false.) or the first character
c     following the matched text (if smtxt=.true.) or the first nonwhite
c     character following the matched text (if smtxt=.true., but the
c     last characters of text were optional and not in the parsed file.)
c
c     Note: this routine requires that the compiler preserves input
c     string case, which is not a standard Fortran 77 feature.
c
c     Warning: do not call this routine with a lowercase (optional)
c     character as first character or following a white character.
c     Ambiguities would arise.
c
c     Constantijn Panis, 5 June 1996
c     Last change: eliminate byte type.
 
      implicit none
      integer begin,il,unit,it,length,newlines,tab,i,matched,
     *   invascii
      parameter(tab=9)
      character line*128,text*(*),ct*1,cl*1,ascii*1
      logical skip1,skip2,option,cmpstri
 
      newlines=0
 
c     Perhaps perfect match; then save time:
      length=len(text)
      if(length.eq.0)then
         smtxt=.true.
         goto 999
      elseif(begin.ge.1.and.begin+length-1.le.80)then
         if(text.eq.line(begin:begin+length-1))then
            begin=begin+length
            smtxt=.true.
            goto 999
         endif
      endif
 
c     Position pointer in parsed file to first nonwhite character:
  100 if(begin.gt.80)then
         if(unit.le.0)goto 800
         read(unit,'(a80)',end=800,err=800)line
         begin=1
      endif
      cl=line(begin:begin)
      if(cl.eq.' '.or.cl.eq.ascii(tab))then
         begin=begin+1
         goto 100
      endif
 
c     line is now the first line on which a nonwhite character appears;
c     that character appears in position begin.  This is also the line
c     and position that need to be returned if the text turns out not
c     to match the following characters on the line.
      il=begin-1
      it=0
      matched=1
 
c     Find next character in text:
  300 skip1=.false.
  310 if(it.ge.length)goto 900
      it=it+1
      ct=text(it:it)
c     Lowercase characters in text are optional
      option = (invascii(ct).ge.97.and.invascii(ct).le.122)
c     Make text character uppercase:
      if(option)ct=ascii(invascii(ct)-32)
      if(ct.eq.' '.or.ct.eq.ascii(tab))then
         skip1=.true.
c        If there is a next word, optional characters may or may not
c        match, either way is fine.
         goto 310
      elseif(option.and.matched.eq.0)then
c        Optional character: if any previous optional character did
c        not match, don't check whether next optional characters do match.
c        For example, PARameter is allowed to match with PAR, PARA, PARAM,
c        et cetera, but not with PARTER and not with PAR AMETER.
         goto 310
      endif
 
c     ct is now the next character in text.  Find next character in
c     the control file, unless the previous one was not matched.
      if(matched.eq.0)goto 500
  400 skip2=.false.
  410 il=il+1
      if(il.gt.80)then
         if(unit.le.0)goto 800
         read(unit,'(a80)',end=800,err=800)line
         il=1
         newlines=newlines+1
         skip2=.true.
      endif
      cl=line(il:il)
      if(cl.eq.' '.or.cl.eq.ascii(tab))then
         skip2=.true.
         goto 410
      endif
c     If text had a white character but line didn't, that may be OK,
c     e.g., when "REGressor SPLINE" appears as "REGSPLINE"
 
c     Check whether the characters are equal, ignoring case.
  500 if(cmpstri(ct,cl))then
c        Exactly the same character, or only different in case.  May
c        well be a good match, but is risky in case the text character
c        is optional and the line character is the first character of
c        a new word.  E.g., text="REGressor SET" and line="regres set".
         if(skip1.and.skip2)then
            matched=1
         elseif(option.and.skip2)then
            matched=0
         else
            matched=1
         endif
         goto 300
      elseif(option)then
c        Optional alpha-character:
         if(skip2)then
c           line went to new word; don't attempt to match.
            matched=0
            goto 300
         else
c           line did not go to new word; do attempt to match.
            if(invascii(ct)-32.eq.invascii(cl))then
               matched=1
            else
c              still OK, e.g. when text="REGressor SET" and line="REGSET".
               matched=0
            endif
            goto 300
        endif
      endif
 
c     Not the same text:
  800 smtxt=.false.
c     Do not update begin; make sure same line is returned:
      if(newlines.gt.0)then
         if(unit.le.0)goto 800
         do 810 i=1,newlines+1
            backspace unit
  810    continue
         read(unit,'(a80)',end=999,err=999)line
      endif
      goto 999
  900 smtxt=.true.
      begin=il
      if(matched.eq.1)begin=begin+1
  999 return
      end
