      subroutine svd(a,n1,n2,w,v,work,status)
 
c     This subroutine performs a singular value decomposition of a
c     matrix:  A = U*W*V'.  Arguments:
c     a  [in/out]  input: arbitrary matrix of dimension (n1,n2), n1>=n2
c                  output: column orthogonal matrix U.
c     w  [out]  diagonal matrix with positive or zero elements, but only
c               diagonal elements are stored.  Dimension (n2).
c     v  [out]  orthogonal matrix of dimension (n2,n2)
c     work [scr] work space of dimension (n2)
c     status [out] 0 if OK; 1 otherwise
c
c     This routine is derived from subroutine svdcmp as published in
c     Numerical Recipes, written by Press, Flannery, Teukolsky and
c     Vetterling (Cambridge University Press 1986), section 2.9,
c     pp 52-63.
c
c     This routine is equivalent to Numerical Recipes routine svdcmp.
c     Main changes are different handling of errors and work space.
c
c     Constantijn Panis, 23 March 2001
c     Last change: status instead of alternate return
 
      implicit none
      integer n1,n2,status,maxiter,i,j,k,l,its,nm
      parameter(maxiter=30)
      double precision a(n1,n2),w(n2),v(n2,n2),work(n2),f,g,h,s,x,y,z,c,
     *   scale,anorm
 
      status=0
      if(n1.lt.n2)goto 800
 
      g=0.d0
      scale=0.d0
      anorm=0.d0
      do 25 i=1,n2
        l=i+1
        work(i)=scale*g
        g=0.d0
        s=0.d0
        scale=0.d0
        if(i.le.n1)then
          do 11 k=i,n1
            scale=scale+dabs(a(k,i))
   11     continue
          if(scale.ne.0.d0)then
            do 12 k=i,n1
              a(k,i)=a(k,i)/scale
              s=s+a(k,i)*a(k,i)
   12       continue
            f=a(i,i)
            g=-sign(dsqrt(s),f)
            h=f*g-s
            a(i,i)=f-g
            if(i.ne.n2)then
              do 15 j=l,n2
                s=0.d0
                do 13 k=i,n1
                  s=s+a(k,i)*a(k,j)
   13           continue
                f=s/h
                do 14 k=i,n1
                  a(k,j)=a(k,j)+f*a(k,i)
   14           continue
   15         continue
            endif
            do 16 k= i,n1
              a(k,i)=scale*a(k,i)
   16       continue
          endif
        endif
        w(i)=scale*g
        g=0.d0
        s=0.d0
        scale=0.d0
        if((i.le.n1).and.(i.ne.n2))then
          do 17 k=l,n2
            scale=scale+dabs(a(i,k))
   17     continue
          if(scale.ne.0.d0)then
            do 18 k=l,n2
              a(i,k)=a(i,k)/scale
              s=s+a(i,k)*a(i,k)
   18       continue
            f=a(i,l)
            g=-sign(dsqrt(s),f)
            h=f*g-s
            a(i,l)=f-g
            do 19 k=l,n2
              work(k)=a(i,k)/h
   19       continue
            if(i.ne.n1)then
              do 23 j=l,n1
                s=0.d0
                do 21 k=l,n2
                  s=s+a(j,k)*a(i,k)
   21           continue
                do 22 k=l,n2
                  a(j,k)=a(j,k)+s*work(k)
   22           continue
   23         continue
            endif
            do 24 k=l,n2
              a(i,k)=scale*a(i,k)
   24       continue
          endif
        endif
        anorm=max(anorm,(dabs(w(i))+dabs(work(i))))
   25 continue
      do 32 i=n2,1,-1
        if(i.lt.n2)then
          if(g.ne.0.d0)then
            do 26 j=l,n2
              v(j,i)=(a(i,j)/a(i,l))/g
   26       continue
            do 29 j=l,n2
              s=0.d0
              do 27 k=l,n2
                s=s+a(i,k)*v(k,j)
   27         continue
              do 28 k=l,n2
                v(k,j)=v(k,j)+s*v(k,i)
   28         continue
   29       continue
          endif
          do 31 j=l,n2
            v(i,j)=0.d0
            v(j,i)=0.d0
   31     continue
        endif
        v(i,i)=1.d0
        g=work(i)
        l=i
   32 continue
      do 39 i=n2,1,-1
        l=i+1
        g=w(i)
        if(i.lt.n2)then
          do 33 j=l,n2
            a(i,j)=0.d0
   33     continue
        endif
        if(g.ne.0.d0)then
          g=1.d0/g
          if(i.ne.n2)then
            do 36 j=l,n2
              s=0.d0
              do 34 k=l,n1
                s=s+a(k,i)*a(k,j)
   34         continue
              f=(s/a(i,i))*g
              do 35 k=i,n1
                a(k,j)=a(k,j)+f*a(k,i)
   35         continue
   36       continue
          endif
          do 37 j=i,n1
            a(j,i)=a(j,i)*g
   37     continue
        else
          do 38 j= i,n1
            a(j,i)=0.d0
   38     continue
        endif
        a(i,i)=a(i,i)+1.d0
   39 continue
      do 49 k=n2,1,-1
        do 48 its=1,maxiter
          do 41 l=k,1,-1
            nm=l-1
            if(dabs(work(l))+anorm.eq.anorm)goto 2
            if(dabs(w(nm))+anorm.eq.anorm)goto 1
   41     continue
   1      c=0.d0
          s=1.d0
          do 43 i=l,k
            f=s*work(i)
            if(dabs(f)+anorm.ne.anorm)then
              g=w(i)
              h=dsqrt(f*f+g*g)
              w(i)=h
              h=1.d0/h
              c=g*h
              s=-f*h
              do 42 j=1,n1
                y=a(j,nm)
                z=a(j,i)
                a(j,nm)=y*c+z*s
                a(j,i)=-y*s+z*c
   42         continue
            endif
   43     continue
   2      z=w(k)
          if(l.eq.k)then
            if(z.lt.0.d0)then
              w(k)=-z
              do 44 j=1,n2
                v(j,k)=-v(j,k)
   44         continue
            endif
            goto 3
          endif
          if(its.eq.maxiter)goto 800
          x=w(l)
          nm=k-1
          y=w(nm)
          g=work(nm)
          h=work(k)
          f=((y-z)*(y+z)+(g-h)*(g+h))/(2.0*h*y)
          g=dsqrt(f*f+1.d0)
          f=((x-z)*(x+z)+h*((y/(f+sign(g,f)))-h))/x
          c=1.d0
          s=1.d0
          do 47 j=l,nm
            i=j+1
            g=work(i)
            y=w(i)
            h=s*g
            g=c*g
            z=dsqrt(f*f+h*h)
            work(j)=z
            c=f/z
            s=h/z
            f= x*c+g*s
            g=-x*s+g*c
            h=y*s
            y=y*c
            do 45 nm=1,n2
              x=v(nm,j)
              z=v(nm,i)
              v(nm,j)= x*c+z*s
              v(nm,i)=-x*s+z*c
   45       continue
            z=dsqrt(f*f+h*h)
            w(j)=z
            if(z.ne.0.d0)then
              z=1.d0/z
              c=f*z
              s=h*z
            endif
            f= c*g+s*y
            x=-s*g+c*y
            do 46 nm=1,n1
              y=a(nm,j)
              z=a(nm,i)
              a(nm,j)= y*c+z*s
              a(nm,i)=-y*s+z*c
   46       continue
   47     continue
          work(l)=0.d0
          work(k)=f
          w(k)=x
   48   continue
   3    continue
   49 continue
      goto 999
 
  800 status=1
  999 return
      end
 
