      subroutine wrapdble(unit,col,width,lf,n,x)
 
c     Subroutine to neatly write out a double precision number, wrapping
c     whenever column width is exceeded.  Arguments:
c        unit [in] unit number to which to write
c        col  [in/out] Upon input, column where first character will
c                  end up (important for comparison to width);  upon
c                  output, column where any subsequent write statement
c                  will start writing.
c        width [in] number of columns to be used in writing; if <=0,
c                   no wrapping is done.
c        lf    [in] if =0, no linefeed is written out after integer;
c                   else, linefeed will be written out.
c        n    [in] related to number of spaces available to write out x:
c                  if n>0, use n spaces.
c                     Trailing zeroes are replaced by leading blanks.
c                  if n<0, use abs(n) spaces, unless fixed format is
c                     best, in which case leading blanks and trailing
c                     zeroes are removed.
c        x [in] double precision number to be written
c
c     Note:  this routine is mainly intended to be called by subroutines
c     writeada, writea2da, et cetera.
c
c     Constantijn Panis, 7 June 1996
 
      implicit none
      integer unit,col,width,lf,n,an,nsd,m,i,dlen
      double precision x,dn,tmp
      logical isnan,isinf,isna
 
c     Determine the number of spaces that will be used.  If n>0, this
c     is simply n.  If n<0, trailing zeroes will be stripped.  The
c     following is an attempt to determine the actual length.  It
c     is based on subroutine writedble, and may be buggy.
      if(n.gt.0)then
         an=n
      else
         an=abs(n)
         dn=dble(an)
         if(isnan(x).or.isinf(x).or.isinf(-x).or.isna(x))then
            if(isnan(x))an=min(an,3)
            if(isinf(x))an=min(an,8)
            if(isinf(-x))an=min(an,9)
            if(isna(x))an=min(an,3)
            goto 200
         endif
         if(x.eq.int(x).and.-10.d0**(dn-2).lt.x.and.x.lt.10.d0**(dn-1))
     *      then
            an=min(an,dlen(x)+2)
            goto 200
         endif
         if((x.lt.0.d0.and.an.le.6).or.(x.ge.0.d0.and.an.le.5))then
            continue
         elseif(x-.5d0.le.-10.d0**(dn-2))then
            nsd=0
         elseif(x.le.-1.d0)then
            nsd=an-2
         elseif(x.lt.0.d0)then
            nsd=an-3+int(dlog10(dint(-x)+.5d0))
         elseif(x.lt.1.d0)then
            nsd=an-2+int(dlog10(dint(x)+.5d0))
         elseif(x+.5d0.le.10.d0**(dn-1))then
            nsd=an-1
         else
            nsd=0
         endif
         if((x.lt.0.d0.and.(an.le.6.or.nsd.ge.an-6)).or.
     *      (x.ge.0.d0.and.(an.le.5.or.nsd.ge.an-5)))then
            if(x.le.-1.d0)then
               m=an-3-int(dlog10(dint(-x)+.5d0))
            elseif(x.lt.0.d0)then
               m=an-3
            elseif(x.lt.1.d0)then
               m=an-2
            else
               m=an-2-int(dlog10(dint(x)+.5d0))
            endif
            m=max(m,0)
            do 110 i=m,1,-1
               tmp=(abs(x)-dint(abs(x)))*10.d0**dble(i)
               if(abs(tmp-10.d0*dint(.1d0*tmp+.5d0)).gt..5d0)goto 120
  110       continue
  120       an=an-(m-i)
         endif
      endif
 
  200 if(width.le.0.or.col+an-1.lt.width)then
         call writedble(unit,x,0,n,0,lf)
         col=col+an
      elseif(col+an-1.eq.width)then
         call writedble(unit,x,0,n,0,1)
         col=1
      elseif(col+an-1.gt.width)then
         write(unit,'(1x)')
         call writedble(unit,x,0,n,0,lf)
         col=an+1
      endif
 
      if(lf.ne.0)col=1
 
      return
      end
