      subroutine wrapstring(unit,column,width,blanks,lf,string)
 
c     Subroutine to neatly write out a character string, wrapping
c     whenever column width is exceeded.  Arguments:
c        unit   [in] unit number to which to write
c        column [in/out] Upon input, column where first character will
c                    end up (important for comparison to width);  upon
c                    output, column where any subsequent write statement
c                    will start writing.  Note: column=0 is interpreted as
c                    column=1, but column will not be updated upon return.
c        width  [in] number of columns to be used in writing; if <=0,
c                    no wrapping is done.
c        blanks [in] if =0, leading blanks are never written out;
c                    else, leading blanks are written out, but only on
c                    the first line.
c        lf     [in] if =0, no linefeed is written out after string;
c                    else, linefeed will be written out.
c        string [in] character string.
c
c     Note: tab characters are counted as one character.
c
c     Note:  this routine is mainly intended to be called by writeaia,
c        writea2ia, writea3ia, writea4ia, writeada, and writea2da.
c
c     Constantijn Panis, 30 June 1996
 
      implicit none
      integer unit,column,width,blanks,lf,length,lineno,lastspace,
     *   col,pos,i
      character string*(*)
 
      length=len(string)
      col=max(1,column)
      lineno=0
 
c     First character to be written is string(1:1)
      lastspace=0
 
c     Write the next line.
c     First character to be written is string(lastspace+1:lastspace+1):
  100 lineno=lineno+1
      pos=lastspace+1
      if(pos.gt.length)goto 200
      lastspace=0
c     Don't write out leading blanks, except possibly on the first line
c     if blanks>0.
  110 if(col.eq.1.and.string(pos:pos).eq.' '.and.
     *   (lineno.gt.1.or.blanks.eq.0))then
         pos=pos+1
         if(pos.gt.length)goto 200
         goto 110
      endif
      do 120 i=pos,length
         if(width.le.0.or.col.le.width)then
            if(i.lt.length)then
c              Line is not full yet; string contains more
               if(string(i:i).eq.' ')lastspace=i
            elseif(i.eq.length)then
c              Line is not full yet; remaining string may be written
c              entirely (including trailing spaces, if any; just
c              don't let trailing spaces become leading spaces):
               if((width.le.0.or.col.lt.width).and.lf.eq.0)then
c                 write(unit,'(a,$)')string(pos:length)
                  write(unit,'(a,$)')string(pos:length)
cD                 write(unit,'(a)',advance='no')string(pos:length)
                  col=col+1
               else
                  write(unit,'(a)')string(pos:length)
                  col=1
               endif
               goto 200
            endif
         elseif(col.gt.width)then
c           Line is full; not everything may be written
            if(lastspace.eq.0)then
c              Word is wider than available width; just cut off
               write(unit,'(a)')string(pos:i-1)
c              Make sure the next record starts at string(i:i)
               lastspace=i-1
            else
               if(string(i:i).eq.' ')lastspace=i
c              Write through previous space:
               write(unit,'(a)')string(pos:lastspace-1)
            endif
c           Write next line:
            col=1
            goto 100
         endif
c        Next character will end up in line column col:
         col=col+1
  120 continue
 
c     Update column, if desired
  200 if(column.gt.0)column=col
 
c     The following fixes an apparent bug in MS Fortran Powerstation 4.0
cD     if(unit.ne.6.and.lf.ne.0)call flush(unit)
 
  999 return
      end
 
