      subroutine write_freq(varname,values,freqs,maxcat,unit,header,
     *   size)
 
c     Subroutine to write frequency table.
c
c     If freqs(1)<0, values(*) contain summary statistics
c     rather than frequencies.
c
c     header = 0: do not write out header of summary statistics
c              1: do write summary statistics header
c              2: table is for Tobit outcome types; write value labels
c
c     Values are ordered before being written.  The original values
c     order is not touched.
c
c     Constantijn Panis, 9 September 2002
 
      implicit none
      integer maxcat,ncat,freqs(maxcat),i,imin,cat,unit,header,size,
     *   intvecsum
      real values(maxcat),sum,mean,var,maximum,minimum,
     *   value,percent,previous,realmax
      character varname*(*),name8*8,name10*10,tlabel(5)*21
      data tlabel/'outcome < lower limit','outcome = lower limit',
     *   'uncensored outcome','outcome = upper limit',
     *   'outcome > upper limit'/
 
      if(maxcat.lt.2)goto 999
 
      if(freqs(1).lt.0)then
c        Write summary statistics rather than a frequency table.
         if(maxcat.lt.5)goto 999
         sum=values(1)
         if(sum.eq.-99999.)sum=0.
         if(sum.ge.1)then
            minimum=values(4)
            maximum=values(5)
            if(minimum.eq.maximum)then
               mean=minimum
               var=0.
            else
               mean=values(2)/sum
               var=values(3)
               if(sum.gt.1.)var=(var/sum-mean*mean)*sum/(sum-1.)
               var=sqrt(max(var,0.))
            endif
         endif
 
         if(size.le.8)then
            if(header.eq.1)write(unit,101)
  101       format(/,12x,'|',6x,'#',8x,'Mean',5x,'Std Dev',9x,'Min',9x,
     *         'Max',/,3x,9('-'),'+',55('-'))
            name8=varname
c           write(unit,'(3x,a8,a2,$)')name8,'|'
            write(unit,'(3x,a8,a2,$)')name8,'|'
cD           write(unit,'(3x,a8,a2)',advance='no')name8,'|'
         elseif(size.le.10)then
            if(header.eq.1)write(unit,102)
  102       format(/,14x,'|',6x,'#',8x,'Mean',5x,'Std Dev',9x,'Min',9x,
     *         'Max',/,3x,11('-'),'+',55('-'))
            name10=varname
c           write(unit,'(3x,a10,a2,$)')name10,'|'
            write(unit,'(3x,a10,a2,$)')name10,'|'
cD           write(unit,'(3x,a10,a2)',advance='no')name10,'|'
         else
            call bug(1,1)
         endif
         if(sum.eq.0.)then
            write(unit,'(i7)')0
         else
c           if(sum.eq.int(sum))write(unit,'(i7,$)')int(sum)
            if(sum.eq.int(sum))write(unit,'(i7,$)')int(sum)
cD          if(sum.eq.int(sum))write(unit,'(i7)',advance='no')int(sum)
            if(sum.ne.int(sum))call writedble(unit,dble(sum),1,8,0,0)
            call writedble(unit,dble(mean),4,8,0,0)
c           if(sum.le.1.)write(unit,'(a12,$)')'.'
            if(sum.le.1.)write(unit,'(a12,$)')'.'
cD          if(sum.le.1.)write(unit,'(a12)',advance='no')'.'
            if(sum.gt.1.)call writedble(unit,dble(var),4,8,0,0)
c           if(minimum.eq.int(minimum))write(unit,'(i12,$)')int(minimum)
            if(minimum.eq.int(minimum))write(unit,'(i12,$)')int(minimum)
cD          if(minimum.eq.int(minimum))write(unit,'(i12)',advance='no')int(minim
            if(minimum.ne.int(minimum))call
     *         writedble(unit,dble(minimum),4,8,0,0)
            if(maximum.eq.int(maximum))write(unit,'(i12)')int(maximum)
            if(maximum.ne.int(maximum))call
     *         writedble(unit,dble(maximum),4,8,0,1)
         endif
         goto 999
      endif
 
c     Write frequency table.  First figure out how many categories there are.
      call realmatch(-99999.,values,maxcat,ncat)
      if(ncat.eq.0)ncat=maxcat+1
      ncat=ncat-1
      sum = intvecsum(freqs,ncat)
 
c     Write out table in increasing value order; the ranking procedure
c     is far from efficient, but the original order is preserved.
      if(header.eq.2)then
c        Use value labels for tobit outcome types:
         write(unit,201)
      else
c        Use values:
         write(unit,202)varname
      endif
      previous = -realmax()
      do 200 cat=1,ncat
         minimum = realmax()
         do 210 i=1,ncat
            if(values(i).lt.minimum.and.values(i).gt.previous)then
               minimum = values(i)
               imin = i
            endif
  210    continue
         previous = minimum
         percent = 100.*freqs(imin)/sum
         value=values(imin)
         if(header.eq.2)then
            write(unit,'(3x,a,$)')tlabel(int(value))
         else
c           if(value.eq.int(value))write(unit,'(i14,$)')int(value)
            if(value.eq.int(value))write(unit,'(i14,$)')int(value)
cD          if(value.eq.int(value))write(unit,'(i14)',advance='no')int(value)
            if(value.ne.int(value))call writedble(unit,dble(value),
     *         6,8,0,0)
         endif
         value=freqs(imin)
c        if(value.eq.int(value))write(unit,'(a,i11,$)')' |',int(value)
         if(value.eq.int(value))write(unit,'(a,i11,$)')' |',int(value)
cD       if(value.eq.int(value))write(unit,'(a,i11)',advance='no')' |',int(value
c        if(value.ne.int(value))write(unit,'(a),$)')' |'
         if(value.ne.int(value))write(unit,'(a),$)')' |'
cD       if(value.ne.int(value))write(unit,'(a)',advance='no')' |'
         if(value.ne.int(value))call writedble(unit,dble(value),3,8,0,0)
         write(unit,'(f12.2)')percent
  200 continue
 
      if(header.eq.2)then
         write(unit,203)
         if(sum.eq.int(sum))write(unit,204)int(sum),100.
c        if(sum.ne.int(sum))write(unit,'(19x,a,$)')'Total |'
         if(sum.ne.int(sum))write(unit,'(19x,a,$)')'Total |'
cD       if(sum.ne.int(sum))write(unit,'(19x,a)',advance='no')'Total |'
      else
         write(unit,205)
         if(sum.eq.int(sum))write(unit,206)int(sum),100.
c        if(sum.ne.int(sum))write(unit,'(9x,a,$)')'Total |'
         if(sum.ne.int(sum))write(unit,'(9x,a,$)')'Total |'
cD       if(sum.ne.int(sum))write(unit,'(9x,a)',advance='no')'Total |'
      endif
      if(sum.ne.int(sum))call writedble(unit,dble(sum),3,8,0,0)
      if(sum.ne.int(sum))write(unit,'(f12.2)')100.
 
  201 format(/,3x,'Tobit outcome type',4x,'|       Freq.    Percent',/,
     *   3x,22('-'),'+',24('-'))
  202 format(/,2x,a12,' |       Freq.    Percent',/,
     *   3x,12('-'),'+',24('-'))
  203 format(3x,22('-'),'+',24('-'))
  204 format(19x,'Total |',i11,f12.2)
  205 format(3x,12('-'),'+',24('-'))
  206 format(9x,'Total |',i11,f12.2)
 
  999 return
      end
