      subroutine write_m_freq(levelmax,modeltype,nfreq,maxnumfreq,
     *   maxfreqcat,freqloc,namepntr,nold,freqcat,freqs,unit,oldname)
 
c     Subroutine to write out frequency tables of variables that have
c     been used to indirectly reference any model element of a
c     particular model statement.
c
c     Constantijn Panis,  9 september 2002
c     Last change: Tobit models
 
      implicit none
      integer levelmax,modeltype,nfreq,maxnumfreq,maxfreqcat,
     *   freqloc(maxnumfreq),namepntr(levelmax),nold,
     *   freqs(maxfreqcat,maxnumfreq),unit,pos,num,i,n,header,match
      real freqcat(maxfreqcat,maxnumfreq)
      character oldname(nold)*8
 
      num=nfreq/(maxnumfreq+1)
      pos=mod(nfreq,maxnumfreq+1)
      if(num.eq.0)goto 999
 
      write(unit,101)
  101 format(/,3x,'Summary statistics of the outcome and selected ',
     *   'variables:')
 
c     Outcome tabulations or summary statistics, if any:
      if(modeltype.eq.1)then
c        Hazard models:
         write(unit,102)
  102    format(/,3x,'Hazard spell durations (for noncensored spells, ',
     *      'lower and upper',/,3x,'duration variables and their ',
     *      'difference; for censored spells,',/,3x,'spell duration):',
     *      //,7x,'censor |',6x,'#',8x,'Mean',5x,'Std Dev',9x,'Min',9x,
     *      'Max',/,3x,11('-'),'+',55('-'))
         call write_freq('  / lower',freqcat(1,pos),freqs(1,pos),
     *      maxfreqcat,unit,0,10)
         pos=pos+1
         call write_freq('0 - upper',freqcat(1,pos),freqs(1,pos),
     *      maxfreqcat,unit,0,10)
         pos=pos+1
c        call write_freq('  \\ window',freqcat(1,pos),freqs(1,pos),
c    *      maxfreqcat,unit,0,10)
         call write_freq('  \ window',freqcat(1,pos),freqs(1,pos),
     *      maxfreqcat,unit,0,10)
         pos=pos+1
         call write_freq('1 - spell',freqcat(1,pos),freqs(1,pos),
     *      maxfreqcat,unit,0,10)
         num=num-3
      elseif(modeltype.eq.41.or.modeltype.eq.43.or.modeltype/10.eq.5.or.
     *   modeltype.eq.6.or.modeltype/10.eq.7)then
c        Simple probits/logits, (negative) binomial, continuous, or
c        multinomial logit:
         call write_freq('outcome',freqcat(1,pos),freqs(1,pos),
     *      maxfreqcat,unit,1,8)
      elseif(modeltype.eq.42.or.modeltype.eq.44)then
c        Ordered logit/probit; two outcomes
         header=1
         call write_freq('outcome1',freqcat(1,pos),freqs(1,pos),
     *      maxfreqcat,unit,header,8)
         if(freqs(1,pos).lt.0)header=0
         pos=pos+1
         num=num-1
         call write_freq('outcome2',freqcat(1,pos),freqs(1,pos),
     *      maxfreqcat,unit,header,8)
      elseif(modeltype.eq.46)then
c        Tobit: freqcat(.,pos) contains outcome types for tabulation;
c        freqcat(.,pos+1) contains summary statistics of noncensored
c        outcomes.  The outcome types do not yet count the number
c        of noncensored outcomes.  Copy those over:
         if(freqs(1,pos+1).ge.0)call bug(1,1)
         call realmatch(3.,freqcat(1,pos),maxfreqcat,match)
         if(match.eq.0)call bug(1,1)
         freqs(match,pos)=int(freqcat(1,pos+1))
         call write_freq('12345678',freqcat(1,pos),freqs(1,pos),
     *      maxfreqcat,unit,2,8)
         write(unit,202)
  202    format(/,3x,'Uncensored outcomes:')
         pos=pos+1
         num=num-1
         call write_freq('outcome',freqcat(1,pos),freqs(1,pos),
     *      maxfreqcat,unit,1,8)
      else
         call bug(1,1)
      endif
      num=num-1
      pos=pos+1
 
      if(num.le.0)goto 999
c     Write out summary statistics and/or frequency tables:
c     First frequency tables
      do 200 i=pos,pos+num-1
         if(freqloc(i).le.0)call bug(1,1)
         n=namepntr(freqloc(i)/(nold+1))+mod(freqloc(i),nold+1)-1
         if(freqs(1,i).lt.0)goto 200
         write(unit,'(1x)')
         call write_freq(oldname(n),freqcat(1,i),
     *      freqs(1,i),maxfreqcat,unit,header,8)
  200 continue
c     Then mean/stddev/min/max
      header=1
      do 210 i=pos,pos+num-1
         n=namepntr(freqloc(i)/(nold+1))+mod(freqloc(i),nold+1)-1
         if(freqs(1,i).ge.0)goto 210
         call write_freq(oldname(n),freqcat(1,i),
     *      freqs(1,i),maxfreqcat,unit,header,8)
         header=0
  210 continue
 
  999 return
      end
