      subroutine writeresults(nfull,nfree,oldnfree,output,numsearch,
     *   round,nrounds,iffull,ipt,jpt,convcrit,lnl,oldlnl,fullbeta,
     *   bhhhsp,numsp,sp2,sp3,sp4,fullname,freename,converged)
 
c     Subroutine to write out the results of ML optimization.  Final
c     estimates, their standard errors and t-statistics are always
c     written out.  Integer array output() controls what other information
c     is written out:
c        4 = Hessian matrix
c        5 = variance-covariance matrix
c        6 = correlation matrix
c        7 = table format
c        8 = starting value format
c        9 = Huber-corrected covariances
c
c     Transformed parameters only serve a purpose in likelihood optimization.
c     In this routine, only untransformed parameters matter.
 
c     Arguments:
c        bhhhsp [in]  second derivatives (wrt untransformed parameters).
c                     Computed as BHHH.  (When numsearch=1, BHHH is computed
c                     and available only if also output(10)=1, for Huber
c                     correction.)  This matrix remains untouched.
c        numsp  [in]  second derivatives (wrt untransformed parameters).
c                     Computed as numerical derivatives of analytical Jacobian.
c                     This matrix remains untouched.
c        sp2   [out]  covariance matrix based on BHHH (inverse of -bhhhsp)
c        sp3   [out]  covariance matrix based on numerical Hessian (inverse of -
c        sp4   [out]  Huber-corrected covariance matrix
c
c     This routine is called by subroutine optim.  For bhhhsp-sp4, call using
c     sp,numsp,spt,sp2,sp3.
c
c     Constantijn Panis,  15 May 2004
c     Last change: Catch negative variances, prevent floating point exception
c     when computing the standard deviation.
 
      implicit none
      integer nfull,nfree,oldnfree,output(10),numsearch,round,nrounds,
     *   iffull(nfull),ipt,jpt,status,i,j,n1,n2,df,dlen
      double precision convcrit(4,2),lnl,oldlnl,fullbeta(nfull),
     *   bhhhsp(nfree,nfree),numsp(nfree,nfree),
     *   sp2(nfree,nfree),sp3(nfree,nfree),sp4(nfree,nfree),
     *   diff,p,std,tratio,pchi2,dprowmin,dpabsvecmin
      logical converged,huber,numerical
      character star*3,fullname(nfull)*8,freename(nfree)*8,filler*16
      data filler/'                '/
      status=0
 
      if(converged)then
         if(nrounds.eq.1)then
            if(ipt.gt.0)write(8,101)
            if(jpt.gt.0)write(6,101)
  101        format(//,70('='),/,'=',17x,'ESTIMATION CONVERGED ',
     *          'SUCCESSFULLY',18x,'=',/,'=',23x,
     *          'RESULTS OF ESTIMATION',24x,'=',/,70('='))
         else
            if(ipt.gt.0)write(8,102)round
            if(jpt.gt.0)write(6,102)round
  102       format(//,70('='),/,'=',19x,'ROUND',i2,
     *         ' CONVERGED SUCCESSFULLY',19x,'=',/,70('='))
            if(round.ge.2)then
c              Report improvement of likelihood:
               df=nfree-oldnfree
               if(df.eq.0)goto 110
               diff=lnl-oldlnl
               n1=dlen(diff)+5
               p=pchi2(2.d0*diff,df)
               if(ipt.gt.3)then
                  write(8,'(/,a,$)')'The log-likelihood improved by'
                  call writedble(8,diff,1,-8,4,0)
                  write(8,'(a)')' relative to the previous round,'
                  call writeaia(8,0,0,'with ',df,
     *               ' extra degrees of freedom.  P(chi2 > 2*')
                  call writedble(8,diff,0,-8,4,0)
                  if(p.lt.1.d-6)then
                     write(8,'(a)')') < 0.000001'
                  else
                     write(8,'(a,$)')') ='
                     call writedble(8,p,1,dlen(1.d0/p)+5,0,1)
                  endif
               endif
               if(jpt.gt.3)then
                  write(6,'(/,a,$)')'The log-likelihood improved by'
                  call writedble(6,diff,1,-8,4,0)
                  write(6,'(a)')' relative to the previous round,'
                  call writeaia(6,0,0,'with ',df,
     *               ' extra degrees of freedom.  P(chi2 > 2*')
                  call writedble(6,diff,0,-8,4,0)
                  if(p.lt.1.d-6)then
                     write(6,'(a)')') < 0.000001'
                  else
                     write(6,'(a,$)')') ='
                     call writedble(6,p,1,dlen(1.d0/p)+5,0,1)
                  endif
               endif
            endif
  110       oldlnl=lnl
            oldnfree=nfree
c           If this is not the final round, don't write out any results.
            if(round.lt.nrounds)then
               if(ipt.ge.3)write(8,106)
               if(jpt.ge.3)write(6,106)
  106          format(/,'Continuing with the next round of ',
     *            'estimation...')
               goto 999
            else
               if(ipt.gt.0)write(8,101)
               if(jpt.gt.0)write(6,101)
            endif
         endif
      else
         if(ipt.gt.0)write(8,107)
         if(jpt.gt.0)write(6,107)
  107    format(//,70('='),/,'=',19x,'ESTIMATION FAILED TO CONVERGE',
     *      20x,'=',/,'=',7x,'THE FOLLOWING RESULTS OF ESTIMATION ',
     *      'MAY BE INCORRECT!!',7x,'=',/,70('='))
      endif
 
      call c8full2free(fullname,freename,iffull,nfull)
 
c     Convergence criteria:
      if(converged)then
         if(ipt.gt.0)write(8,111)
         if(jpt.gt.0)write(6,111)
  111    format(/,'Convergence based on:')
         call wrresult1(ipt,jpt,'Weighted gradient norm:',
     *      convcrit(1,2),convcrit(1,1))
         call wrresult1(ipt,jpt,'Relative function improvement:',
     *      convcrit(2,2),convcrit(2,1))
         call wrresult1(ipt,jpt,'Gradient norm:',
     *      convcrit(3,2),convcrit(3,1))
         call wrresult1(ipt,jpt,'Relative parameter change:',
     *      convcrit(4,2),convcrit(4,1))
      endif
 
c     Huber correction or numerical standard errors required??
      huber=(output(9).eq.1)
      numerical=(output(10).eq.1.or.numsearch.eq.1)
 
c     bhhhsp is BHHH Hessian (not available if numsearch=1)
c     numsp  is numerical Hessian (available only if huber or numerical)
c
c     These matrices remain untouched.  Compute the following:
c
c     sp2 = covariance matrix based on BHHH (inverse of -bhhhsp)
c     sp3 = covariance matrix based on numerical Hessian (inverse of -numsp)
c     sp4 = Huber-corrected covariance matrix
 
      if(numsearch.ne.1)then
c        Compute the default (BHHH) covariance matrix of parameter estimates.
         call opposite(bhhhsp,sp2,nfree*nfree)
         call definv(1,sp2,nfree,status)
         if(status.ne.0)call bug(ipt,jpt)
c        By construction, the BHHH Hessian is negative definite and the
c        resulting variances (diagonal elements of sp2) are positive.
      endif
 
      if(huber.or.numerical)then
c        Compute inverse of minus the numerical Hessian.
c        First compute the largest eigenvalue of the (positive) Hessian and
c        warn if it looks like a problem.  All eigenvalues are supposed to
c        be negative; the largest eigenvalue should be a smallish negative numbe
         call copy8(numsp,sp4,nfree*nfree)
         call eigen(1,sp4,0.d0,nfree)
         if(sp4(1,1).ge.1.d-3)then
c           The numerically computed Hessian is not definite.
            if(ipt.gt.0)write(8,211)
            if(converged.and.ipt.gt.0)write(8,212)
            if(numsearch.eq.1.and.ipt.gt.0)write(8,213)
            if(ipt.gt.0)write(8,214)
            if(jpt.gt.0)write(6,211)
            if(converged.and.jpt.gt.0)write(6,212)
            if(numsearch.eq.1.and.jpt.gt.0)write(6,213)
            if(jpt.gt.0)write(6,214)
            numerical=.false.
         endif
         if(dpabsvecmin(sp4,nfree).lt.1.d-3)then
            if(ipt.gt.0)write(8,215)dpabsvecmin(sp4,nfree)
            if(jpt.gt.0)write(6,215)dpabsvecmin(sp4,nfree)
         endif
  211    format(/,31('*'),' WARNING ',31('*'),/,'*  The likelihood ',
     *      'surface is not concave.  It is unlikely that the    *',
     *      /,'*  results reported below reflect the global ',
     *      'maximum.  Standard       *',/,'*  errors of parameter ',
     *      'estimates could not be calculated on the       *',/,
     *      '*  basis of numerical second derivatives.',t71,'*')
  212    format('*  Consider tightening the convergence criterion to ',
     *      'ensure concavity  *',/,'*  of the likelihood surface ',
     *      'upon convergence.',t71,'*')
  213    format('*  Consider searching using the default BHHH ',
     *      'approximation to second  *',/,'*  derivatives (i.e., ',
     *      'without "option numerical search") and switch   *',/,
     *      '*  back to the numerical search when the parameters ',
     *      'are closer to     *',/,'*  the maximum.',t71,'*')
  214    format(71('*'))
  215    format(/,31('*'),' WARNING ',31('*'),/,'*  The smallest ',
     *      'eigenvalue of the (numerically computed) matrix of    *',/,
     *      '*  second derivatives is worrisomely small:',1pe13.4,t71,
     *      '*',/,'*  This indicates that your model is not ',
     *      'identified, and may result   *',/,'*  in inaccurately ',
     *      'reported standard errors.',t71,'*',/,71('*'))
      endif
      if(huber.or.numerical)then
c        Compute inverse of minus the numerical Hessian.
         call opposite(numsp,sp3,nfree*nfree)
         call nrminv(1,sp3,sp4,nfree,0.d0,0.d0,status)
c        Potential issue if matrix is singular; addressed through warning
c        above and refusal to compute standard errors if also non-definite.
         call copy8(sp4,sp3,nfree*nfree)
 
c        Compute the Huber-corrected covariance matrix, sp4 = Hinv*BHHH*Hinv
         if(huber)then
            call quadratic(0,nfree,nfree,sp3,bhhhsp,sp4,0)
            call opposite(sp4,sp4,nfree*nfree)
         endif
 
c        Verify that to-be-reported variances are positive.  May not be if the
c        likelihood surface is not concave (numerical Hessian not definite).
c        This should have been caught above, but just in case:
         if(numerical.and.dprowmin(sp3,nfree+1,nfree).lt.0.d0)then
            if(ipt.gt.0)write(8,211)
            if(jpt.gt.0)write(6,211)
            numerical=.false.
         endif
c        I believe the Huber correction is guaranteed to generate a valid
c        covariance matrix, but just in case:
         if(huber.and.dprowmin(sp4,nfree+1,nfree).lt.0.d0)then
            if(ipt.gt.0)write(8,216)
            if(jpt.gt.0)write(6,216)
  216       format(/,31('*'),' WARNING ',31('*'),/,'*  The likelihood ',
     *         'surface is not concave.  It is unlikely that the    *',
     *         /,'*  results reported below reflect the global ',
     *         'maximum.  It is not      *',/,'*  possible to compute ',
     *         'Huber-corrected (robust) estimates of          *',/,
     *         '*  standard errors of parameter estimates.',t71,'*',/,
     *         71('*'))
            huber=.false.
         endif
      endif
 
c-------------------------------------------------------------------------
 
c     The basics: log-likelihood, estimates, standard errors, t-statistics
      if(ipt.gt.0)write(8,202)
      if(jpt.gt.0)write(6,202)
      if(huber)then
c        Huber-corrected results:
         if(ipt.gt.0)call wrresult2(8,nfull,nfree,lnl,fullbeta,sp4,
     *      fullname,iffull,'==== Huber-corrected ====')
         if(jpt.gt.0)call wrresult2(6,nfull,nfree,lnl,fullbeta,sp4,
     *      fullname,iffull,'==== Huber-corrected ====')
         if(ipt.gt.0)write(8,201)
         if(jpt.gt.0)write(6,201)
      endif
      if(numerical)then
c        Results based on numerical Hessian:
         if(ipt.gt.0)call wrresult2(8,nfull,nfree,lnl,fullbeta,sp3,
     *      fullname,iffull,'Based on numerical Hessian')
         if(jpt.gt.0)call wrresult2(6,nfull,nfree,lnl,fullbeta,sp3,
     *      fullname,iffull,'Based on numerical Hessian')
         if(numsearch.eq.0.and.ipt.gt.0)write(8,201)
         if(numsearch.eq.0.and.jpt.gt.0)write(6,201)
      endif
c     Write out non-corrected results if available
      if(numsearch.eq.0)then
         if(ipt.gt.0)call wrresult2(8,nfull,nfree,lnl,fullbeta,sp2,
     *      fullname,iffull,'BHHH-based, non-corrected')
         if(jpt.gt.0)call wrresult2(6,nfull,nfree,lnl,fullbeta,sp2,
     *      fullname,iffull,'BHHH-based, non-corrected')
      endif
c     Account for possibility that numerical standard errors were asked but
c     not computable.  Always write out something, if only parameter values.
      if(.not.huber.and..not.numerical.and.numsearch.ne.0)then
         if(ipt.gt.0)call wrresult3(8,nfull,nfree,lnl,fullbeta,
     *      fullname,iffull)
         if(jpt.gt.0)call wrresult3(6,nfull,nfree,lnl,fullbeta,
     *      fullname,iffull)
      endif
 
c     if output(4)=1: Hessian matrix
      if(output(4).eq.1)then
         if(ipt.gt.0)write(8,201)
         if(jpt.gt.0)write(6,201)
         if(huber.or.numerical)then
            if(ipt.gt.0)call writelt('Second derivatives:',nfree,
     *         freename,numsp,5,1,8)
            if(jpt.gt.0)call writelt('Second derivatives:',nfree,
     *         freename,numsp,5,1,6)
            if(ipt.gt.0)write(8,301)
            if(jpt.gt.0)write(6,301)
  301       format(/,'NOTE:  Second derivatives computed as numerical ',
     *         'derivatives of',/,7x,'analytical first derivatives.')
         else
            if(ipt.gt.0)call writelt('Second derivatives:',nfree,
     *         freename,bhhhsp,5,1,8)
            if(jpt.gt.0)call writelt('Second derivatives:',nfree,
     *         freename,bhhhsp,5,1,6)
         endif
      endif
 
c     if output(5)=1: variance-covariance matrix
      if(output(5).eq.1)then
         if(ipt.gt.0)write(8,201)
         if(jpt.gt.0)write(6,201)
         if(huber)then
            if(ipt.gt.0)call writelt('Huber-corrected variance-'//
     *         'covariance matrix:',nfree,freename,sp4,5,1,8)
            if(jpt.gt.0)call writelt('Huber-corrected variance-'//
     *         'covariance matrix:',nfree,freename,sp4,5,1,6)
            if(ipt.gt.0)write(8,201)
            if(jpt.gt.0)write(6,201)
         endif
         if(numerical)then
            if(ipt.gt.0)call writelt('Variance-covariance matrix '//
     *         'based on numerical Hessian:',nfree,freename,sp3,5,1,8)
            if(jpt.gt.0)call writelt('Variance-covariance matrix '//
     *         'based on numerical Hessian:',nfree,freename,sp3,5,1,6)
            if(ipt.gt.0)write(8,201)
            if(jpt.gt.0)write(6,201)
         endif
         if(numsearch.eq.0)then
            if(ipt.gt.0)call writelt('BHHH-based '//
     *         'variance-covariance matrix:',nfree,freename,sp2,5,1,8)
            if(jpt.gt.0)call writelt('BHHH-based '//
     *         'variance-covariance matrix:',nfree,freename,sp2,5,1,6)
         endif
      endif
 
c     if output(6)=1: correlation matrix
c     Use bhhhsp as scratch matrix.
      if(output(6).eq.1)then
         if(huber)then
            call cov2corr(1,nfree,sp4,bhhhsp,0.d0,0.d0,status)
            if(status.ne.0)goto 510
            call putrow(1.d0,bhhhsp,nfree,nfree+1)
            if(ipt.gt.0)write(8,201)
            if(jpt.gt.0)write(6,201)
            if(ipt.gt.0)call writelt('Huber-corrected '//
     *         'correlation matrix:',nfree,freename,bhhhsp,5,1,8)
            if(jpt.gt.0)call writelt('Huber-corrected '//
     *         'correlation matrix:',nfree,freename,bhhhsp,5,1,6)
         endif
         if(numerical)then
            call cov2corr(1,nfree,sp3,bhhhsp,0.d0,0.d0,status)
            if(status.ne.0)goto 510
            call putrow(1.d0,bhhhsp,nfree,nfree+1)
            if(ipt.gt.0)write(8,201)
            if(jpt.gt.0)write(6,201)
            if(ipt.gt.0)call writelt('Correlation matrix based on '//
     *         'numerical Hessian:',nfree,freename,bhhhsp,5,1,8)
            if(jpt.gt.0)call writelt('Correlation matrix based on '//
     *         'numerical Hessian:',nfree,freename,bhhhsp,5,1,6)
         endif
         if(numsearch.eq.0)then
            call cov2corr(1,nfree,sp2,bhhhsp,0.d0,0.d0,status)
            if(status.ne.0)goto 510
            call putrow(1.d0,bhhhsp,nfree,nfree+1)
            if(ipt.gt.0)write(8,201)
            if(jpt.gt.0)write(6,201)
            if(ipt.gt.0)call writelt('BHHH-based correlation matrix:',
     *         nfree,freename,bhhhsp,5,1,8)
            if(jpt.gt.0)call writelt('BHHH-based correlation matrix:',
     *         nfree,freename,bhhhsp,5,1,6)
         endif
         goto 610
  510    if(ipt.gt.0)write(8,511)
         if(jpt.gt.0)write(6,511)
  511    format(/,'ERROR:  illegitimate correlation matrix.')
      endif
 
c     if output(7)=1: table format
  610 if(output(7).eq.1)then
         if(ipt.gt.0)write(8,201)
         if(jpt.gt.0)write(6,201)
         if(ipt.gt.0)write(8,601)
         if(jpt.gt.0)write(6,601)
  601    format(/,'Results in table-format:')
         j=0
         do 600 i=1,nfull
            if(iffull(i).eq.1.and.
     *         (huber.or.numerical.or.numsearch.eq.0))then
               j=j+1
               if(huber)then
                  if(sp4(j,j).lt.0.d0)call bug(ipt,jpt)
                  std=dsqrt(sp4(j,j))
               elseif(numerical)then
                  if(sp3(j,j).lt.0.d0)call bug(ipt,jpt)
                  std=dsqrt(sp3(j,j))
               elseif(numsearch.eq.0)then
                  if(sp2(j,j).lt.0.d0)call bug(ipt,jpt)
                  std=dsqrt(sp2(j,j))
               endif
               star = '   '
               if(std.ne.0.d0)then
                  tratio = dabs(fullbeta(i)/std)
                  if(tratio.gt.1.645)star = '*  '
                  if(tratio.gt.1.960)star = '** '
                  if(tratio.gt.2.575)star = '***'
               endif
               if(ipt.gt.0)write(8,602)fullname(i),fullbeta(i),star
               if(jpt.gt.0)write(6,602)fullname(i),fullbeta(i),star
  602          format(/,a8,5x,f10.4,1x,a3)
c              "Best" std are in (...):
               n2=dlen(std)+5
               n1=22-n2
               if(ipt.gt.0)call writeada(8,0,1,filler(1:n1)//'(',
     *            n2,std,')')
               if(jpt.gt.0)call writeada(6,0,1,filler(1:n1)//'(',
     *            n2,std,')')
               if(huber.and.numerical)then
c                 Numeric std are in [...]:
                  if(sp3(j,j).lt.0.d0)call bug(ipt,jpt)
                  std=dsqrt(sp3(j,j))
                  n2=dlen(std)+5
                  n1=22-n2
                  if(ipt.gt.0)call writeada(8,0,1,filler(1:n1)//'[',
     *               n2,std,']')
                  if(jpt.gt.0)call writeada(6,0,1,filler(1:n1)//'[',
     *               n2,std,']')
               endif
               if((huber.or.numerical).and.numsearch.ne.1)then
c                 BHHH-based std are in {...}:
                  if(sp2(j,j).lt.0.d0)call bug(ipt,jpt)
                  std=dsqrt(sp2(j,j))
                  n2=dlen(std)+5
                  n1=22-n2
                  if(ipt.gt.0)call writeada(8,0,1,filler(1:n1)//'{',
     *               n2,std,'}')
                  if(jpt.gt.0)call writeada(6,0,1,filler(1:n1)//'{',
     *               n2,std,'}')
               endif
            else
               if(ipt.gt.0)write(8,604)fullname(i),fullbeta(i)
               if(jpt.gt.0)write(6,604)fullname(i),fullbeta(i)
  604          format(/,a8,5x,f10.4)
               if(huber.and.numerical)then
                  if(ipt.gt.0)write(8,'(1x)')
                  if(jpt.gt.0)write(6,'(1x)')
               endif
               if((huber.or.numerical).and.numsearch.ne.1)then
                  if(ipt.gt.0)write(8,'(1x)')
                  if(jpt.gt.0)write(6,'(1x)')
               endif
            endif
  600    continue
         if(ipt.gt.0)write(8,605)lnl
         if(jpt.gt.0)write(6,605)lnl
  605    format(/,'Log Likelihood:',f16.8)
         if(ipt.gt.0)write(8,611)
         if(jpt.gt.0)write(6,611)
  611    format(/,'NOTE: Asymptotic standard errors in parentheses;',/,
     *      6x,'Significance: ''*''=10%;  ''**''=5%;  ''***''=1%.')
         if(huber)then
            if(ipt.gt.0)write(8,612)
            if(jpt.gt.0)write(6,612)
  612       format(6x,'Standard errors in (...) are Huber-corrected;')
            if(numerical)then
               if(ipt.gt.0)write(8,613)
               if(jpt.gt.0)write(6,613)
  613          format(6x,'Standard errors in [...] are based on ',
     *            'numerical Hessian;')
            endif
         elseif(numerical)then
            if(ipt.gt.0)write(8,614)
            if(jpt.gt.0)write(6,614)
  614       format(6x,'Standard errors in (...) are based on ',
     *         'numerical Hessian;')
         endif
         if((huber.or.numerical).and.numsearch.ne.1)then
            if(ipt.gt.0)write(8,615)
            if(jpt.gt.0)write(6,615)
  615       format(6x,'Standard errors in {...} are BHHH-based;')
         endif
         if(huber)then
            if(ipt.gt.0)write(8,616)
            if(jpt.gt.0)write(6,616)
  616       format(6x,'Significance asterisks are Huber-corrected.')
         elseif(numerical)then
            if(ipt.gt.0)write(8,617)
            if(jpt.gt.0)write(6,617)
  617       format(6x,'Significance asterisks are based on ',
     *         'numerical Hessian.')
         endif
      endif
 
c     if output(8)=1: starting value format
      if(output(8).eq.1)then
         if(ipt.gt.0)write(8,701)
         if(jpt.gt.0)write(6,701)
  701    format(/,70('-'),//,'Results for new control file:',/)
         do 700 i=1,nfull
            if(ipt.gt.0)then
c              write(8,'(a8,l5,$)')fullname(i),(iffull(i).eq.1)
               write(8,'(a8,l5,$)')fullname(i),(iffull(i).eq.1)
cD              write(8,'(a8,l5)',advance='no')fullname(i),(iffull(i).eq.1)
               call writedble(8,fullbeta(i),-3,13,0,1)
            endif
            if(jpt.gt.0)then
c              write(6,'(a8,l5,$)')fullname(i),(iffull(i).eq.1)
               write(6,'(a8,l5,$)')fullname(i),(iffull(i).eq.1)
cD              write(6,'(a8,l5)',advance='no')fullname(i),(iffull(i).eq.1)
               call writedble(6,fullbeta(i),-3,13,0,1)
            endif
  700    continue
         if(ipt.gt.0)write(8,'(a)')';'
         if(jpt.gt.0)write(6,'(a)')';'
      endif
 
      if(ipt.gt.0)write(8,202)
      if(jpt.gt.0)write(6,202)
  201 format(/,70('-'))
  202 format(/,70('='))
      goto 999
 
  999 return
      end
c=====================================================================
      subroutine wrresult1(ipt,jpt,name,actual,limit)
 
c     Subroutine to write out the value of a convergence criterion
c     upon convergence and compares it with the set value.
c
c     Examples:
c        Relative function improvement:  0.000692 < 0.001
c        Gradient norm:                  597.3081
c
c     Constantijn Panis, 23 November 1995
 
      implicit none
      integer ipt,jpt
      double precision actual,limit
      character name*(*)
      logical isinf
 
      if(ipt.gt.0)then
         write(8,'(3x,a,t35,a,$)')name,' '
         if(actual.ge.limit.or.isinf(actual))then
            call writedble(8,actual,0,-8,0,1)
         else
            call writedble(8,actual,0,-8,0,0)
            write(8,'(a,$)')' <'
            call writedble(8,limit,1,-8,0,1)
         endif
      endif
      if(jpt.gt.0)then
         write(6,'(3x,a,t35,a,$)')name,' '
         if(actual.ge.limit.or.isinf(actual))then
            call writedble(6,actual,0,-8,0,1)
         else
            call writedble(6,actual,0,-8,0,0)
            write(6,'(a,$)')' <'
            call writedble(6,limit,1,-8,0,1)
         endif
      endif
 
      return
      end
c=====================================================================
      subroutine wrresult2(unit,nfull,nfree,lnl,fullbeta,covar,
     *   fullname,iffull,string)
      implicit none
      integer unit,nfull,nfree,iffull(nfull),i,j,dlen,trm
      double precision lnl,fullbeta(nfull),covar(nfree,nfree),std
      character fullname(nfull)*8,string*(*),fmt*12,i2c*12
 
      fmt='(/,a,f'//i2c(dlen(lnl)+7,0)
      fmt=fmt(1:trm(fmt))//'.4)'
      write(unit,fmt)'Log Likelihood:',lnl
      write(unit,101)string
  101 format(42x,a,/,'  Parameter',4x,'Free?',5x,'Estimate',9x,
     *   'Std Err',8x,'T-statistic',/)
      j=0
      do 100 i=1,nfull
         write(unit,'(i3,a10,l5,$)')i,fullname(i),(iffull(i).eq.1)
         call writedble(unit,fullbeta(i),-5,13,0,0)
         if(iffull(i).eq.1)then
            j=j+1
            if(covar(j,j).lt.0.d0)call bug(1,1)
            std=dsqrt(covar(j,j))
            call writedble(unit,std,-5,13,0,0)
            if(std.ne.0.d0)write(unit,'(f14.4,$)')fullbeta(i)/std
            if(std.eq.0.d0)write(unit,'(8x,a,$)')'------'
         endif
         write(unit,'(1x)')
  100 continue
 
      return
      end
c=====================================================================
      subroutine wrresult3(unit,nfull,nfree,lnl,fullbeta,
     *   fullname,iffull)
      implicit none
      integer unit,nfull,nfree,iffull(nfull),i,j,dlen,trm
      double precision lnl,fullbeta(nfull)
      character fullname(nfull)*8,fmt*12,i2c*12
 
      fmt='(/,a,f'//i2c(dlen(lnl)+7,0)
      fmt=fmt(1:trm(fmt))//'.4)'
      write(unit,fmt)'Log Likelihood:',lnl
      write(unit,101)
  101 format(/,'  Parameter',4x,'Free?',5x,'Estimate',9x,
     *   'Std Err',8x,'T-statistic',/)
      j=0
      do 100 i=1,nfull
         write(unit,'(i3,a10,l5,$)')i,fullname(i),(iffull(i).eq.1)
         if(iffull(i).eq.0)then
            call writedble(unit,fullbeta(i),-5,13,0,1)
         else
            call writedble(unit,fullbeta(i),-5,13,0,0)
            j=j+1
            write(unit,102)
  102       format(11x,'?',13x,'?')
         endif
  100 continue
 
      return
      end
 
