      subroutine wrmodel5(levelmax,unit,modeltype,nfull,numdist,
     *   sumdim,numpar,meno,moel,maxxnode,distdim,pardim,parpos,
     *   namepntr,nold,xnode,oldname,parname,rsetname,resname,splname,
     *   version)
 
c     Subroutine to write a binomial, negative binomial, or Poisson
c     model specification.  The model consists of EXPOSURE or SCALE=
c     and PROBABILITY= or INCIDENCE=, where the probability/incidence
c     involves (a linear combination of) regressor sets (etc) and
c     (a linear combination of) integrated residuals.
c
c     Constantijn Panis, 3 December 2002
c     Last change: version
 
      implicit none
      integer levelmax,unit,modeltype,nfull,numdist,sumdim,
     *   numpar,meno,moel(*),maxxnode,distdim(numdist),pardim(numpar),
     *   parpos(numpar),namepntr(levelmax),nold,version,exponent
      real xnode(maxxnode)
      character oldname(nold)*8,parname(*)*12,rsetname(*)*12,
     *   resname(sumdim)*8,splname(*)*12
 
c     outcome = varname;
c     write(unit,'(3x,a,$)')'outcome = '
      write(unit,'(3x,a,$)')'outcome = '
cD     write(unit,'(3x,a)',advance='no')'outcome = '
      call writeexpr(100,unit,maxxnode,0,levelmax,1,-moel(meno),
     *   0,0,0,0,nold,0,namepntr,-1,0.,xnode,0.,oldname,0)
      write(unit,'(a)')';'
      meno=meno+1
 
      if(modeltype.ge.51.and.modeltype.le.53)then
c        Binomial:
c        write(unit,'(3x,a,$)')'exposure = '
         write(unit,'(3x,a,$)')'exposure = '
cD        write(unit,'(3x,a)',advance='no')'exposure = '
         call writeexpr(100,unit,maxxnode,0,levelmax,1,-moel(meno),
     *      0,0,0,0,nold,0,namepntr,-1,0.,xnode,0.,oldname,0)
         write(unit,'(a)')';'
         meno=meno+1
      elseif(modeltype.ge.54.and.modeltype.le.56)then
c        Old-style negative binomial:
         meno=meno+1
         exponent=moel(meno)
         if(exponent.ne.3)call bug(1,1)
         if(version.ne.1)call bug(1,1)
c        scale = exp(linear combination of regressor sets (etc))
         call wrapstring(unit,0,0,1,0,'   scale = exp(')
         meno=meno+1
         call wrmodel(levelmax,unit,nfull,numdist,sumdim,numpar,meno,
     *      moel,maxxnode,distdim,pardim,parpos,namepntr,nold,xnode,
     *      oldname,parname,rsetname,resname,splname)
         write(unit,'(a)')');'
      elseif(modeltype.eq.57)then
c        Negative binomial:
         call wrapstring(unit,0,0,1,0,'   exposure = ')
         if(moel(meno).ne.-99)call writeexpr(100,unit,maxxnode,0,
     *      levelmax,1,-moel(meno),0,0,0,0,nold,0,namepntr,-1,0.,
     *      xnode,0.,oldname,0)
         write(unit,'(a)')';'
         meno=meno+1
         exponent=moel(meno)
         if(exponent.eq.1)then
c           dispersion = linear combination of regressor sets (etc)
            call wrapstring(unit,0,0,1,0,'   dispersion = ')
         elseif(exponent.eq.2)then
c           dispersion = exp(linear combination of regressor sets (etc))
            call wrapstring(unit,0,0,1,0,'   dispersion = exp(')
         else
            call bug(1,1)
         endif
         meno=meno+1
         call wrmodel(levelmax,unit,nfull,numdist,sumdim,numpar,meno,
     *      moel,maxxnode,distdim,pardim,parpos,namepntr,nold,xnode,
     *      oldname,parname,rsetname,resname,splname)
         if(exponent.eq.1)then
            write(unit,'(a)')';'
         else
            write(unit,'(a)')');'
         endif
      elseif(modeltype.eq.58)then
c        Poisson:
         call wrapstring(unit,0,0,1,0,'   exposure = ')
         if(moel(meno).ne.-99)call writeexpr(100,unit,maxxnode,0,
     *      levelmax,1,-moel(meno),0,0,0,0,nold,0,namepntr,-1,0.,
     *      xnode,0.,oldname,0)
         write(unit,'(a)')';'
         meno=meno+1
      else
         call bug(1,1)
      endif
 
c     incidence = exp(...);
c     probability = {linear(...) | logistic(...) | normprob(...)};
      if(modeltype.eq.57.or.modeltype.eq.58)then
         call wrapstring(unit,0,0,1,0,'   incidence = exp(')
      elseif(modeltype.eq.51.or.modeltype.eq.54)then
         call wrapstring(unit,0,0,1,0,'   prob = logistic(')
      elseif(modeltype.eq.52.or.modeltype.eq.55)then
         call wrapstring(unit,0,0,1,0,'   prob = normprob(')
      else
         call wrapstring(unit,0,0,1,0,'   prob = linear(')
      endif
      call wrmodel(levelmax,unit,nfull,numdist,sumdim,numpar,meno,
     *   moel,maxxnode,distdim,pardim,parpos,namepntr,nold,xnode,
     *   oldname,parname,rsetname,resname,splname)
      write(unit,'(a)')');'
 
      if(modeltype.ge.54.and.modeltype.le.56.and.version.eq.1)
     *   write(unit,104)
  104 format(/,'NOTE: syntax and algorithms of aML Version 1 apply, ',
     *   'so that larger',/,'probabilities increase the likelihood of ',
     *   'lower outcomes, and vice',/,'versa.  See the manual for ',
     *   'details.')
 
  999 return
      end
