      program amltest
 
c     This program reads two aML output files and tests whether the
c     parameter estimates are significantly different.  It assumes
c     that the models are nested (!).  Usage:
c        amltest name name
c     If name does not exist, name.out is assumed.  The output is
c     sent to standard output.
c
c     Constantijn Panis, 29 April 1999
c     Last change: Digital Visual Fortran 6.0 compatible
 
      implicit none
      integer iargc
      double precision lnl1,lnl2
      integer npar1,npar2,trm
      integer(2) arg
c     integer arg
      character filename*256
 
      if(iargc().ne.2)goto 800
      arg=1
      call getarg(arg,filename)
      call openfile(filename)
      call readfile(filename(1:trm(filename)),npar1,lnl1)
      arg=2
      call getarg(arg,filename)
      call openfile(filename)
      call readfile(filename(1:trm(filename)),npar2,lnl2)
      call test(npar1,npar2,lnl1,lnl2)
      goto 999
 
  800 write(6,801)
  801 format('Usage: amltest name name')
      goto 999
  999 continue
      end
c=========================================================================
      subroutine openfile(filename)
      implicit none
      integer nt,trm
      character filename*256
      logical exist
 
      nt=trm(filename)
      if(exist(filename(1:nt)//'.out'))then
         open(4,file=filename(1:nt)//'.out')
         filename=filename(1:nt)//'.out'
      elseif(exist(filename(1:nt)))then
         open(4,file=filename)
      else
         goto 800
      endif
      goto 999
 
  800 if(filename(nt-2:nt).eq.'out'.or.
     *   filename(nt-2:nt).eq.'OUT')then
         call not_found(6,filename(1:nt))
      else
         call not_found(6,filename(1:nt)//'.out')
      endif
      stop
  999 return
      end
c=========================================================================
      subroutine readfile(filename,npar,lnl)
      implicit none
      integer maxround
      parameter(maxround=20)
      integer npar,round,nround,nfree(maxround),il,status
      double precision lnl
      character filename*(*),line*128
      logical converged,smtxt
      round=1
 
c     First read until  "Number of parameters estimated:"
  100 read(4,'(a80)',err=800,end=800)line
      if(line(1:31).ne.'Number of parameters estimated:')goto 100
      il=32
      round=0
c     There may be multiple rounds, which looks like:
c     Number of parameters estimated:       9;  30;  34
  110 round=round+1
      if(round.gt.maxround)call bug(0,1)
      call getint(4,nfree(round),line,il,status)
      if(status.gt.0)goto 800
      if(smtxt(4,line,il,';'))goto 110
      nround=round
 
c     There may be multiple rounds of optimization; skip intermediate ones.
c     Ultimately, scan for "Log Likelihood", and check for convergence.
      round=0
  200 read(4,'(a80)',err=800,end=800)line
 
c...+....1....+....2....+....3....+....4....+....5....+....6....+....7....+....8
c                   ROUND 1 CONVERGED SUCCESSFULLY                   =
c                 ESTIMATION CONVERGED SUCCESSFULLY                  =
c                   ESTIMATION FAILED TO CONVERGE                    =
 
      if(line(21:25).eq.'ROUND'.and.
     *   line(29:50).eq.'CONVERGED SUCCESSFULLY')then
         round=round+1
         converged=.true.
      elseif(line(21:49).eq.'ESTIMATION FAILED TO CONVERGE')then
         round=round+1
         converged=.false.
      elseif(line(19:51).eq.'ESTIMATION CONVERGED SUCCESSFULLY')then
         if(round.eq.0)round=1
         converged=.true.
      endif
      if(line(1:15).ne.'Log Likelihood:')goto 200
      backspace 4
      read(4,*,end=800,err=800)line,line,lnl
 
      if(.not.converged)write(6,101)filename
  101 format('WARNING: ''',a,''' did not converge.')
      if(round.gt.nround)call bug(0,1)
      npar=nfree(max(1,round))
 
      goto 999
 
  800 write(6,801)filename
  801 format('ERROR or END-OF-FILE encountered while reading ''',
     *   a,'''.')
      goto 899
  899 stop
  999 return
      end
c=========================================================================
      subroutine test(npar1,npar2,lnl1,lnl2)
      implicit none
      double precision lnl1,lnl2,diff,p,pchi2
      integer npar1,npar2,n1,n2,df,intlen,dlen,trm
      character fmt*24,i2c*12
 
      n1=max(intlen(npar1),intlen(npar2))
      n2=dlen(min(lnl1,lnl2))+3
      fmt='(a,i'//i2c(n1,0)
      fmt=fmt(1:trm(fmt))//',a,f'//i2c(n2,0)
      fmt=fmt(1:trm(fmt))//'.2)'
      write(6,fmt)'Number of parameters ',npar1,'; log-likelihood ',lnl1
      write(6,fmt)'Number of parameters ',npar2,'; log-likelihood ',lnl2
 
      if(npar1.eq.npar2)goto 800
      if(npar1.gt.npar2.and.lnl1.lt.lnl2)goto 810
      if(npar1.lt.npar2.and.lnl1.gt.lnl2)goto 810
      df=abs(npar1-npar2)
      diff=abs(lnl1-lnl2)
      p=pchi2(2.d0*diff,df)
      call writeaia(6,0,0,'With ',df,' degrees of freedom')
      call writeada(6,0,0,', P(chi2 > 2*',dlen(diff)+3,diff,')')
      if(p.lt.1.d-6)then
         write(6,'(a)')' < 0.000001'
      else
         call writeada(6,0,1,' = ',dlen(1.d0/p)+4,p,' ')
      endif
      goto 999
 
  800 write(6,801)
  801 format(/,'ERROR: number of parameters is equal; no test ',
     *   'possible.')
      goto 899
  810 write(6,811)
  811 format(/,'ERROR: the model with fewer parameters has a better ',
     *   'likelihood;',/,'the specifications cannot have been nested.')
      goto 899
  899 return
  999 return
      end
