      subroutine countvars(numvar,line,il)
 
c     Subroutine count the number of variabels in a raw2aml control
c     file variable list.  Also generate syntax error messages.  Variable
c     names must:
c        start with a letter (a-z, A-Z);
c        consist of letters, numbers, or underscores;
c        be eight or fewer characters in length;
c        not be among: EXP LOG LN ABS MIN MAX IN SQRT DUMMY UNIFORM
c
c     Note: numvar accumulates the number of variables; it is NOT
c     initialized in this routine.
c
c     Subroutine varlist actually reads in the names.
c
c     Names may be followed by something in parentheses:
c        name(a)  variable must remain in absolute value, even if
c                 option deviations=yes;  is turned on.
c        name(d)  variable must be converted into deviations from its mean.
c        name(d=x) variable must be converted in deviations from x, where
c                 x is a real number.
c
c     Constantijn Panis, 20 July 1999
 
      implicit none
      integer numvar,il,i1,i2,status,trm
      real x
      logical smtxt
      character line*128,tmpname*8
 
  100 if(smtxt(4,line,il,';'))goto 999
      call getname(4,line,il,i1,i2)
      if(i2.lt.i1)goto 800
      if(i2-i1.gt.7)goto 810
      if(line(i1:i1).eq.'_')goto 810
      call uppercase(line(i1:i2),tmpname)
      if(tmpname.eq.'EXP'.or.tmpname.eq.'LOG'.or.tmpname.eq.'LN'.or.
     *   tmpname.eq.'ABS'.or.tmpname.eq.'MIN'.or.tmpname.eq.'MAX'.or.
     *   tmpname.eq.'IN'.or.tmpname.eq.'SQRT'.or.tmpname.eq.'DUMMY'.or.
     *   tmpname.eq.'UNIFORM')goto 840
      numvar=numvar+1
      if(numvar.eq.3.and.tmpname.eq.'ID')then
c        Warn against inclusion of ID among the variable list.
         write(8,101)line(i1:i2)
         write(6,101)line(i1:i2)
  101    format(/,'WARNING: your very first variable is ''',a,
     *      '''.  Note that the observation',/,9x,'ID is assumed to ',
     *      'be on every record in the first position;',/,9x,
     *      'it should not be listed among the data variables, ',
     *      'unless it',/,9x,'really appears twice in the data.',/)
      endif
      if(.not.smtxt(4,line,il,'('))goto 100
      if(smtxt(4,line,il,'A)'))then
c        This variable in absolute value
         continue
      elseif(smtxt(4,line,il,'D)'))then
c        This variable in deviations from its mean
         continue
      elseif(smtxt(4,line,il,'D='))then
c        This variable in deviations from user-specified constant
         call getreal(4,x,line,il,status)
         if(status.gt.0)goto 830
         if(.not.smtxt(4,line,il,')'))goto 830
      else
         goto 830
      endif
      goto 100
 
  800 call caretline(8,line,il)
      write(8,801)
      call caretline(6,line,il)
      write(6,801)
  801 format('ERROR encountered while reading a variable list.')
      stop
  810 call caretline(8,line,il)
      call wrapstring(8,0,72,1,1,'ERROR:  illegal variable name, '''//
     *   line(i1:i2)//'''.  Variable names must start with a letter '//
     *   '(a-z, A-Z), may contain only letters, numbers, and '//
     *   'underscores, and may not be more than eight characters in '//
     *   'length.')
      call caretline(6,line,il)
      call wrapstring(6,0,72,1,1,'ERROR:  illegal variable name, '''//
     *   line(i1:i2)//'''.  Variable names must start with a letter '//
     *   '(a-z, A-Z), may contain only letters, numbers, and '//
     *   'underscores, and may not be more than eight characters in '//
     *   'length.')
      stop
  830 call caretline(8,line,il)
      write(8,831)
      call caretline(6,line,il)
      write(6,831)
  831 format(/,'ERROR encountered while reading a variable list.  ',
     *   'The syntax for',/,'transforming variables into deviations ',
     *   'is:',//,3x,'varname(a)   variable remains in absolute ',
     *   'value, even if',/,16x,'option "deviations=yes;" is ',
     *   'turned on;',/,3x,'varname(d)   variable is converted into ',
     *   'deviations from its mean;',/,3x,'varname(d=x) variable is ',
     *   'converted into deviations from x, where',/,
     *   16x,'x is a real number.')
      stop
  840 call caretline(8,line,il)
      write(8,841)tmpname(1:trm(tmpname))
      call caretline(6,line,il)
      write(6,841)tmpname(1:trm(tmpname))
  841 format(/,'ERROR: ',a,' is a reserved name (upper- and ',
     *   'lowercase).')
      stop
  999 return
      end
