      subroutine readcomp(unit,id,struc,level,maxnbdim,nb,nx,
     *   maxxdim,xdim,x,status,line,iobs)
 
c     Subroutine to read in ID, [structure,] numbers of branches, and
c     variables X, in compressed format.
c     Upon return, input array nb is built up as follows:
c        nb(1) = number of level 3 branches
c        nb(2) = number of level 4 branches from the first level 3 branch
c        nb(3) = number of level 4 branches from the second level 3 branch
c        nb(nb(1)+1) = number of level 4 branches from the last level 3 branch
c        nb(nb(1)+2) = number of level 5 branches from the first level 4 branch
c                            from the first level 3 branch
c        etc.
c
c     status [out]  0 = all is OK
c                   1 = ID or structure not what was expected
c                   2 = insufficient space available
c                   3 = read error or end-of-file (should not happen!)
c                   4 = invalid numbers of branches, nb() ==> stop
c
c     Constantijn Panis, 17 November 2000
c     Last change: X1 now part of X.
 
      implicit none
      integer unit,id,struc,level,maxnbdim,nb(maxnbdim),nx(level),
     *   maxxdim,xdim,status,i,ll,iobs,n,
     *   dimnb,numbranch,numcompx,intvecmin,intvecmax,intlen,trm
      double precision thisid,thisstruc
      real x(maxxdim)
      character line*128,fmt*16,i2c*12
 
      status=0
      if(level.eq.1)then
         xdim=nx(1)
      elseif(level.eq.2)then
         xdim=nx(1)+nx(2)
      endif
 
      if(struc.eq.0)then
         if(level.le.2)then
            read(unit,*,end=800,err=800)thisid,(x(i),i=1,xdim)
         elseif(level.eq.3)then
            read(unit,*,end=800,err=800)thisid,nb(1),
     *         (x(i),i=1,min(maxxdim,nx(1)+nx(2)+nb(1)*nx(3)))
         else
c           Hellishly complicated......
            read(unit,*,end=800,err=800)thisid,nb(1),
     *         ((nb(i),i=dimnb(ll,nb)+1,dimnb(ll,nb)+numbranch(ll,nb)),
     *         ll=3,level-1),
     *         (x(i),i=1,min(maxxdim,numcompx(level,nx,nb)))
         endif
         if(thisid.ne.id)status=1
      else
         if(level.le.2)then
            read(unit,*,end=800,err=800)thisid,thisstruc,(x(i),i=1,xdim)
         elseif(level.eq.3)then
            read(unit,*,end=800,err=800)thisid,thisstruc,nb(1),
     *         (x(i),i=1,min(maxxdim,nx(1)+nx(2)+nb(1)*nx(3)))
         else
c           Hellishly complicated......
            read(unit,*,end=800,err=800)thisid,thisstruc,nb(1),
     *         ((nb(i),i=dimnb(ll,nb)+1,dimnb(ll,nb)+numbranch(ll,nb)),
     *         ll=3,level-1),
     *         (x(i),i=1,min(maxxdim,numcompx(level,nx,nb)))
         endif
         if(thisid.ne.id.or.thisstruc.ne.struc)status=1
      endif
 
c     Check the numbers of branches:
      if(level.ge.3)then
         if(nb(1).lt.0)goto 810
         do 100 ll=3,level-1
            if(intvecmin(nb(dimnb(ll,nb)+1),numbranch(ll,nb)).lt.0)
     *         goto 810
  100    continue
      endif
 
c     Actual number of variables read in:
      xdim=numcompx(level,nx,nb)
      if(maxxdim.lt.xdim)status=2
      if(dimnb(level,nb).gt.maxnbdim)status=2
      goto 999
 
  800 status=3
      goto 999
c     Number of branches supposedly negative
  810 inquire(unit=unit,name=line)
      call writea2ia(8,72,1,'ERROR encountered while reading '''//
     *   line(1:trm(line))//''' (observation ',iobs,', ID=',id,
     *   ').  One or more indicated number of subbranches in '//
     *   'your data are negative:')
      call writea2ia(6,72,1,'ERROR encountered while reading '''//
     *   line(1:trm(line))//''' (observation ',iobs,', ID=',id,
     *   ').  One or more indicated number of subbranches in '//
     *   'your data are negative:')
      n=max(4,intlen(intvecmin(nb,maxnbdim)),
     *   intlen(intvecmax(nb,maxnbdim)))
      fmt='(a,i2,a,'//i2c(maxnbdim,0)
      fmt=fmt(1:trm(fmt))//'i'//i2c(n,0)
      fmt=fmt(1:trm(fmt))//')'
      do 805 ll=2,level-1
         write(8,fmt)'   Number of level',ll+1,' subbranches:',
     *      (nb(dimnb(ll,nb)+i),i=1,numbranch(ll,nb))
         write(6,fmt)'   Number of level',ll+1,' subbranches:',
     *      (nb(dimnb(ll,nb)+i),i=1,numbranch(ll,nb))
  805 continue
      write(8,811)
      write(6,811)
  811 format(/,'A potential cause of this error is that your raw data ',
     *   'are rectangular',/,'while you did not specify that.  The ',
     *   'raw2aml control file syntax for',/,'rectangular data is:',//,
     *   3x,'LEVEL 1 VAR = varlist;',/,
     *   3x,'LEVEL 2 VAR = varlist;  /* always one level 2 branch per ',
     *   'record */',/,3x,'LEVEL 3 VAR (NB=n3) = varlist;  /* n3 ',
     *   'level 3 branches */',/,3x,'LEVEL 4 VAR (NB=n4) = varlist;  ',
     *   '/* n4 level 4 branches */',/,3x,'et cetera...',/)
      stop
  999 return
      end
