      subroutine figure_model(levelmax,nb2,nemax,nehmax,nelmax,nepmax,
     *   necmax,nebmax,nenmax,nepomax,nemlmax,nnemlmax,
     *   ne,neh,nel,nep,nec,neb,nen,nepo,neml,nneml,nemp,nnemp,nihmax,
     *   nold,nbdim,nb,struc,nummodel,datadim,maxmeno,datas,modeltype,
     *   moelpntr,moel,datastructure,xpntr,nxold,namepntr,branch,
     *   strctre,mdllvl,strucfreqs,maxxnode,minlnl,plusmin,plusobs,
     *   minobs,dropobs,xdim,x,xnode,obsweight,sumweight,
     *   oldname,ipt,jpt,iobs,id,status)
 
c     Subroutine to keep track of data structure frequencies and to
c     determine to which model equation each episode belongs.
c     Not every episode needs to be used; the model specification
c     may specify fewer dimensions than feasible with the data.
c     Furthermore, episodes within data structures that are modeled
c     may be dropped by the KEEP or DROP statement.
c     Finally, some episodes may be modeled, but only appear in the
c     numerator or denominator of the likelihood function.
c
c     Also count the number of various types of episodes and intervals,
c     and check them against maximum allowable dimensions.
c
c     If (a subset of) the episodes enters into the denominator of the
c     likelihood (as specified through a DENOMINATOR; statement in the
c     model statement), then two rounds are made through this routine,
c     per observation.  Integer plusne counts the number of episodes
c     in the numerator; minne idem in the denominator.  Integer plusmin
c     (which is written out in the scratch data) summarizes whether
c     there are any numerator/denominator episodes, and which part
c     is currently done:
c        plusne  minne  plusmin  plusmin
c          0       0       na      na
c          0       +       na      -2
c          +       0        0      na
c          +       +        1      -1
c
c     Constantijn Panis, 10 November 2002
c     Last change:  Poisson models
 
      implicit none
      integer levelmax,nb2,nemax,nehmax,nelmax,nepmax,necmax,nebmax,
     *   nenmax,nepomax,nemlmax,nnemlmax,ne,neh,nel,nep,nec,
     *   neb,nen,nepo,neml,nneml,nemp,nnemp,
     *   nihmax,nold,nbdim,nb(nbdim,2),struc(nb2),nummodel,datadim,
     *   maxmeno,moel(maxmeno),meno,datas(nummodel),modeltype(nummodel),
     *   moelpntr(nummodel),xpntr(2:levelmax,nb2),
     *   nxold(levelmax,datadim),namepntr(levelmax,datadim),
     *   branch(levelmax),branchnum,strctre(nummodel),mdllvl(nummodel),
     *   strucfreqs(datadim,3),maxxnode,
     *   minlnl(nummodel),plusne,minne,plusmin,plusobs,minobs,dropobs,
     *   select,xdim,status,ib2,ds,m,n,nc,np,nsb,timevar,ipt,jpt,
     *   iobs,id,nmbrnch,trm
      real x(xdim),xnode(*),datastructure(datadim),
     *   outcome,lowerlimit,upperlimit,obsweight,sumweight
      character oldname(nold)*8,string*60,idstring*60
 
      status=0
 
c     Keep track of frequency distribution of data structures.
c     Note that the table will only be written out if detail=.true.,
c     but strucfreqs also serves to count the number of level 2
c     branches, which is always written out.
      if(plusmin.eq.0)then
         do 100 ib2=1,nb2
            call figure_freq(real(struc(ib2)),datastructure,strucfreqs,
     *         datadim)
  100    continue
      endif
 
  200 plusne=0
      minne=0
      neh=0
      nel=0
      nep=0
      nec=0
      neb=0
      nen=0
      nepo=0
      neml=0
      nneml=0
      nemp=0
      nnemp=0
c     Loop over all model specifications:
      do 210 m=1,nummodel
 
c        What rank number data structure is that model?
         ds=strctre(m)
 
         select=moel(moelpntr(m))
 
c        Whether we are currently evaluating the numerator
c        (plusmin=0) or the denominator (plusmin!=0), both
c        modules that enter the numerator (plusne) and the
c        denominator (minne) need to be counted, because subroutine
c        accumfunc needs to be told whether both types appear.
c        (That information is stored in plusmin, which will be
c        written out in the scratch data.)
 
c        Loop over all level mdllvl subbranches in the data figure
c        out how many outcomes they generate
         n=0
         np=0
         nc=0
         ib2=1
         branchnum=0
  220    call nextbranch(1,mdllvl(m),branch,nbdim,nb,branchnum)
         if(branchnum.eq.0)goto 240
c        Is this data structure modeled at all?
         if(mdllvl(m).gt.1)then
            ib2=branch(2)
            if(struc(ib2).ne.datas(m))goto 220
         endif
 
c        Check for keep/drop statement:
         if(select.ne.0)then
            call evalexpr(1,maxxnode,xdim,abs(select),nold,levelmax,
     *         mdllvl(m),nbdim,nb,branch,xpntr(2,ib2),nxold(1,ds),
     *         namepntr(1,ds),xnode,x,outcome,oldname,
     *         struc(ib2),iobs,id,ipt,jpt,status)
            if(status.gt.0)stop
            if(outcome.ne.0..and.outcome.ne.1.)goto 800
            if(select.gt.0.and.outcome.eq.0.)goto 220
            if(select.lt.0.and.outcome.eq.1.)goto 220
         endif
 
         if(modeltype(m).eq.1)then
c           Time-varying variables?
            timevar=moel(moelpntr(m)+3)
            if(timevar.eq.0)then
               nihmax=max(nihmax,2)
            else
               nsb=nmbrnch(mdllvl(m),branch,mdllvl(m)+1,nbdim,nb)
c              Zero subbranches are not allowed; caught in figdata1.
c              Maximum number of intervals _per spell_. (Allow for
c              one extra for an inserted lower bound of event window.)
               nihmax=max(nihmax,nsb+1)
            endif
         elseif(modeltype(m).eq.46)then
c           Tobit model: distinguish continuous and probit outcomes
            meno=moelpntr(m)+1
            call filtertobit(levelmax,mdllvl(m),nbdim,nb,branch,
     *         moel,meno,maxmeno,
     *         namepntr(1,ds),nxold(1,ds),nold,xpntr(2,ib2),xdim,
     *         maxxnode,x,xnode,outcome,lowerlimit,upperlimit,oldname,
     *         struc(ib2),iobs,id,ipt,jpt)
            if(lowerlimit.lt.outcome.and.outcome.lt.upperlimit)then
               nc=nc+1
            else
               np=np+1
            endif
         endif
c        Count number of outcomes:
         n=n+1
c        For multinomial logits/probits, also count number of equations:
         if(modeltype(m).eq.71)nneml=nneml+moel(moelpntr(m)+1)
         if(modeltype(m).eq.72)nnemp=nnemp+moel(moelpntr(m)+1)
 
         if(plusmin.eq.0)then
            if(minlnl(m).ne.1)call figure_freq(real(struc(ib2)),
     *         datastructure,strucfreqs(1,2),datadim)
            if(minlnl(m).ge.1)call figure_freq(real(struc(ib2)),
     *         datastructure,strucfreqs(1,3),datadim)
         endif
         goto 220
 
c        minlnl = 0   data structure enters in numerator only
c        minlnl = 1   data structure enters in denominator only
c        minlnl = 2   data structure enters in both
  240    if(minlnl(m).eq.0)then
            plusne=plusne+n
            if(plusmin.ne.0)goto 210
         elseif(minlnl(m).eq.1)then
            minne=minne+n
            if(plusmin.eq.0)goto 210
         else
            plusne=plusne+n
            minne=minne+n
         endif
c        Number of outcomes generated:
         if(modeltype(m).eq.1)then
            neh=neh+n
         elseif(modeltype(m).eq.41.or.modeltype(m).eq.42)then
            nel=nel+n
         elseif(modeltype(m).eq.43.or.modeltype(m).eq.44)then
            nep=nep+n
         elseif(modeltype(m).eq.6)then
            nec=nec+n
         elseif(modeltype(m).ge.51.and.modeltype(m).le.53)then
            neb=neb+n
         elseif(modeltype(m).ge.54.and.modeltype(m).le.57)then
            nen=nen+n
         elseif(modeltype(m).eq.58)then
            nepo=nepo+n
         elseif(modeltype(m).eq.46)then
            nep=nep+np
            nec=nec+nc
         elseif(modeltype(m).eq.71)then
            neml=neml+n
c           Number of equations already accumulated in nneml.
         elseif(modeltype(m).eq.72)then
            nemp=nemp+n
c           Number of equations already accumulated in nnemp.
         endif
  210 continue
 
c     Technical number of outcomes:
c     (Conceptual number is
c        neh+nel+nep+ nemp+nec+neb+nen+nepo+neml)
      ne=neh+nel+nep+nnemp+nec+neb+nen+nepo+neml
c     Maximum number of outcomes per observation:
      nemax=max(nemax,ne)
c     There needs to be a check for space in draw(numdist,nemax);
c     this is done immediately after return of this routine.
      nehmax=max(nehmax,neh)
      nelmax=max(nelmax,nel)
      nepmax=max(nepmax,nep+nnemp)
      necmax=max(necmax,nec)
      nebmax=max(nebmax,neb)
      nenmax=max(nenmax,nen)
      nepomax=max(nepomax,nepo)
      nemlmax=max(nemlmax,neml)
      nnemlmax=max(nnemlmax,nneml)
 
c     Accumulate weights only in the numerator loop, even if there
c     only are denominator episodes; if neither numerator nor
c     denominator modules exist, increment dropobs and exit.
c     By far the most common case is plusmin=0 (currently evaluating
c     numerator), plusne>0 and minne=0.
      if(plusmin.eq.0)then
c        Currently evaluating numerator
         if(plusne.eq.0)then
            if(minne.eq.0)then
c              None of the episodes is modeled:
               dropobs=dropobs+1
               status=-1
               goto 999
            endif
c           Unusual: only denominator episodes exist.
            sumweight=sumweight+obsweight
            plusmin=-2
            goto 200
         else
            sumweight=sumweight+obsweight
            plusobs=plusobs+1
c           If denominator loop is required, flag so to prepscratch_,
c           and flag to accumfunc that a denominator likelihood will
c           follow.
            if(minne.gt.0)plusmin=1
         endif
      else
c        Currently evaluating denominator
         if(minne.eq.0)call bug(ipt,jpt)
         minobs=minobs+1
         if(plusne.eq.0)then
            plusmin=-2
         else
            plusmin=-1
         endif
      endif
      goto 999
 
  800 string=idstring(struc(ib2),id,iobs,mdllvl(m),branch)
      if(ipt.gt.0)then
         call wrapstring(8,0,72,0,1,'ERROR:  keep/drop expressions '//
     *      'in model statements must evaluate to zero or one ('//
     *      string(1:trm(string))//'):')
         call writeexpr(111,8,maxxnode,xdim,levelmax,nbdim,
     *      -abs(select),nb,branch,xpntr(2,ib2),1,nold,nxold(1,ds),
     *      namepntr(1,ds),-1,outcome,xnode,x,oldname,struc(ib2))
      endif
      if(jpt.gt.0)then
         call wrapstring(6,0,72,0,1,'ERROR:  keep/drop expressions '//
     *      'in model statements must evaluate to zero or one ('//
     *      string(1:trm(string))//'):')
         call writeexpr(111,6,maxxnode,xdim,levelmax,nbdim,
     *      -abs(select),nb,branch,xpntr(2,ib2),1,nold,nxold(1,ds),
     *      namepntr(1,ds),-1,outcome,xnode,x,oldname,struc(ib2))
      endif
      stop
  810 string=idstring(struc(ib2),id,iobs,mdllvl(m),branch)
      meno=meno-4
      if(ipt.gt.0)then
         write(8,811)string(1:trm(string))
         call wrapstring(8,0,0,0,0,'lower limit = ')
         call writeexpr(111,8,maxxnode,xdim,levelmax,nbdim,
     *      -moel(meno+2),nb,branch,xpntr(2,ib2),1,nold,nxold(1,ds),
     *      namepntr(1,ds),-1,lowerlimit,xnode,x,oldname,struc(ib2))
         call wrapstring(8,0,0,0,0,'upper limit = ')
         call writeexpr(111,8,maxxnode,xdim,levelmax,nbdim,
     *      -moel(meno+4),nb,branch,xpntr(2,ib2),1,nold,nxold(1,ds),
     *      namepntr(1,ds),-1,upperlimit,xnode,x,oldname,struc(ib2))
      endif
      if(jpt.gt.0)then
         write(6,811)string(1:trm(string))
         call wrapstring(6,0,0,0,0,'lower limit = ')
         call writeexpr(111,6,maxxnode,xdim,levelmax,nbdim,
     *      -moel(meno+2),nb,branch,xpntr(2,ib2),1,nold,nxold(1,ds),
     *      namepntr(1,ds),-1,lowerlimit,xnode,x,oldname,struc(ib2))
         call wrapstring(6,0,0,0,0,'upper limit = ')
         call writeexpr(111,6,maxxnode,xdim,levelmax,nbdim,
     *      -moel(meno+4),nb,branch,xpntr(2,ib2),1,nold,nxold(1,ds),
     *      namepntr(1,ds),-1,upperlimit,xnode,x,oldname,struc(ib2))
      endif
  811 format(/,'ERROR: the lower limit of a double-censored tobit ',
     *   'model equals or',/,'exceeds the upper limit (',a,'):')
      stop
  999 return
      end
