      subroutine integrate(nfullplus,ne,neh,nel,nep,nec,neb,nen,nepo,
     *   neml,nneml,numnest,numdist,sumintdim,sumdim2,sumdim4,
     *   sumbigintdim,dimii,dimlam,maxpnts,vpntr1,vpntr6,vpntr5,vpntr5n,
     *   vpntr5po,vpntr7,dimv,ipt,jpt,nested,distdim,distyp,distpos,
     *   lamtyp,lampos,draw,cdraw,thisdraw,ndraw,bdc,intdist,allint6,
     *   indx,dimused,intpnts,intpntr,like,lnl,lnlobs,fullbeta,
     *   dfullbeta,ddfullbeta,lam,dlam,ddlam,z,wt,weight,ui,ddui,epsh,
     *   ddepsh,epsl,ddepsl,epsp,ddepsp,epsc,ddepsc,epsb,ddepsb,epsn,
     *   ddepsn,epspo,ddepspo,epsml,ddepsml,cdm,ddcdm,dcdmdsigii,a6,
     *   siga6a6i,siga6a6ia6,dsiga6a6,ddsiga6a6,cmua4,cmui,dcmui,ddcmui,
     *   sigl4l6,dsigl4l6,ddsigl4l6,sigl4l6a6a6i,sigii,dsigii,ddsigii,
     *   dxbh,ddxbh,xbc,dxbc,ddxbc,dxblp,ddxblp,lnla,lnlb,dlnla,dlnlb,
     *   tau,dtau,ddtau,siga4a4,dsiga4a4,ddsiga4a4,deta6a6,a5,da5,dda5,
     *   p5,dp5,ddp5,xbml,dxbml,ddxbml,v,status,ifdrv,iflam,iftau,iid4,
     *   ibnnid4)
 
c     Subroutine to integrate heterogeneity out of all modules.
c     The procedure is as follows.
c     (1) For each heterogeneity (approximation) point, the
c           log-likelihood is computed, joint for all modules.
c           The log-likelihood is lnl: derivatives are stored in
c           d... arrays.
c     (2) These log-likelihoods are converted into likelihoods,
c           appropriately weighted, and accumulated.  The likelihood
c           is like: derivatives are stored in dd... arrays.
c     (3) The integrated likelihood is converted into a log-likelihood
c           and accumulated with the log-likelihood value upon calling
c           of this routine.  The log-likelihood is lnlobs; derivatives
c           are stored in d... arrays.
c
c     Constantijn Panis, 3 December 2002
 
      implicit none
      integer nfullplus,ne,neh,nel,nep,nec,neb,nen,nepo,neml,nneml,
     *   numnest,numdist,sumintdim,sumdim2,sumdim4,sumbigintdim,dimii,
     *   dimlam,maxpnts,vpntr1,vpntr6,vpntr5,vpntr5n,vpntr5po,vpntr7,
     *   dimv,ipt,jpt,nested(numdist,2),
     *   distdim(numdist),distyp(numdist),distpos(numdist),
     *   lamtyp(numdist,8),lampos(numdist,8),draw(numdist,ne),
     *   cdraw(ne),ccdraw,thisdraw(numdist),bdc(nep),
     *   ndraw(numdist),intdist(numnest),allint6,indx(sumintdim),
     *   dimused(sumintdim),intpnts(sumintdim),intpntr(sumintdim),posi1,
     *   posi2,posi4,bigposi1,bigposi2,intdim,intdimc,
     *   thisnest,nest,status,task
      double precision lnlobs,like(numnest),lnl(0:numnest),
     *   weight(numnest),fullbeta(nfullplus),
     *   dfullbeta(nfullplus,0:numnest),ddfullbeta(nfullplus,numnest),
     *   lam(dimlam),dlam(dimlam,0:numnest-1),ddlam(dimlam,numnest),
     *   cmui(sumbigintdim),dcmui(sumbigintdim,0:numnest-1),
     *   ddcmui(sumbigintdim,numnest),z(maxpnts),wt(maxpnts),
     *   ui(sumintdim),ddui(sumintdim),epsh(neh),ddepsh(neh),epsl(nel),
     *   ddepsl(nel),epsp(nep),ddepsp(nep),epsc(nec),ddepsc(nec),
     *   epsb(neb),ddepsb(neb),epsn(nen),ddepsn(nen),epspo(nepo),
     *   ddepspo(nepo),epsml(nneml),ddepsml(nneml),cdm(sumdim2),
     *   ddcdm(sumdim2),dcdmdsigii(sumdim4),a6(nec),siga6a6i(nec,nec),
     *   siga6a6ia6(nec),dsiga6a6(nec*nec,0:numnest),
     *   ddsiga6a6(nec*nec,numnest),cmua4(nep),sigl4l6(nep*nec*numdist),
     *   dsigl4l6(nep*nec*numdist,0:numnest-1),
     *   ddsigl4l6(nep*nec*numdist,numnest),sigl4l6a6a6i(*)
      double precision sigii(dimii),dsigii(dimii,0:numnest-1),
     *   ddsigii(dimii,numnest-1),
     *   dxbh(neh,0:numnest),ddxbh(neh,numnest),
     *   xbc(nec),dxbc(nec,0:numnest),ddxbc(nec,numnest),
     *   dxblp(nel+nep,0:numnest),ddxblp(nel+nep,numnest),
     *   lnla(neh),lnlb(neh),dlnla(nfullplus,neh),dlnlb(nfullplus,neh),
     *   tau(2,nel+nep),dtau(2*(nel+nep),0:numnest),
     *   ddtau(2*(nel+nep),numnest),siga4a4(nep,nep),
     *   dsiga4a4(nep*nep,0:numnest),ddsiga4a4(nep*nep,numnest),
     *   a5(nen),da5(nen,0:numnest),dda5(nen,numnest),
     *   p5(neb+nen+nepo),dp5(neb+nen+nepo,0:numnest),
     *   ddp5(neb+nen+nepo,numnest),xbml(nneml),dxbml(nneml,0:numnest),
     *   ddxbml(nneml,numnest),deta6a6,dpnan,minexp
      real v(dimv)
      logical ifdrv,iflam(numdist),iftau,iid4,ibnnid4
 
      status=0
c     Unravel cdraw(ne) into draw(numdist,ne)
      call nocombi(numdist,ne,cdraw,draw,ndraw,nested)
 
c     This is the start of the loop over nested distributions and
c     draws of those distributions and heterogeneity points within
c     each draw.
      task=1
c     What point of what draw of what distribution is evaluated next?
c     How deep is the integration (thisnest)?
  100 call nextpoint(task,numnest,thisnest,nest,intdist,intdim,
     *   intdimc,numdist,sumintdim,sumdim2,sumdim4,sumbigintdim,dimii,
     *   nfullplus,maxpnts,nec,allint6,posi1,posi2,posi4,bigposi1,
     *   bigposi2,nested,ne,thisdraw,draw,cdraw,ccdraw,ndraw,distdim,
     *   distpos,distyp,dimused,indx,intpnts,intpntr,sigii,cdm,
     *   dcdmdsigii,fullbeta,z,wt,weight,ui,cmui,ifdrv,ipt,jpt,status)
      if(status.gt.0)goto 800
c     Task indicates what needs to be done:
  200 if(task.eq.1)then
c        Next point has been assigned.  Compute conditional log-
c        likelihood (onepoint), weight and accumulate likelihood
c        (sumlnl2l) (only for nest=thisnest)
         call onepoint(numdist,sumintdim,ne,neh,nel,nep,nec,neb,nen,
     *      nepo,neml,nneml,nfullplus,dimlam,dimv,vpntr1,vpntr6,vpntr5,
     *      vpntr5n,vpntr5po,vpntr7,distdim,nested,allint6,lampos,
     *      lamtyp,cdraw,thisdraw,ccdraw,bdc,ipt,jpt,lnl(nest),ui,
     *      dfullbeta(1,nest),lam,epsh,epsl,epsp,epsc,epsb,epsn,epspo,
     *      epsml,dxbh(1,nest),xbc,dxbc(1,nest),dxblp(1,nest),lnla,lnlb,
     *      dlnla,dlnlb,a6,siga4a4,dsiga4a4(1,nest),siga6a6i,
     *      dsiga6a6(1,nest),siga6a6ia6,cmua4,sigl4l6,tau,dtau(1,nest),
     *      deta6a6,a5,da5(1,nest),p5,dp5(1,nest),xbml,dxbml(1,nest),v,
     *      ifdrv,iid4,ibnnid4,status)
         if(status.gt.0)goto 800
         call sumlnl2l(nest,thisnest,numdist,sumintdim,sumdim2,
     *      sumbigintdim,neh,nel,nep,nec,neb,nen,nepo,neml,nneml,
     *      nfullplus,dimlam,dimii,allint6,maxpnts,intpnts,intpntr,
     *      nested,lampos,lamtyp,indx,dimused,thisdraw,distyp,distpos,
     *      posi1,posi2,bigposi1,intdist,intdim,intdimc,like,lnl,z,wt,
     *      weight,fullbeta,dfullbeta,ddfullbeta,ui,ddui,cmui,dcmui,
     *      ddcmui,lam,dlam,ddlam,ddepsh,ddepsl,ddepsp,ddepsc,ddepsb,
     *      ddepsn,ddepspo,ddepsml,dxbh,ddxbh,dxbc,ddxbc,cdm,ddcdm,
     *      siga6a6i,dsiga6a6,ddsiga6a6,siga6a6ia6,sigl4l6,dsigl4l6,
     *      ddsigl4l6,sigl4l6a6a6i,dtau,ddtau,dxblp,ddxblp,dsiga4a4,
     *      ddsiga4a4,da5,dda5,dp5,ddp5,dxbml,ddxbml,dsigii,ddsigii,
     *      ifdrv,iflam,iftau,iid4)
         task=2
         goto 100
      elseif(task.eq.2)then
c        All points have been evaluated.  Accumulate like(nest) into
c        lnl(nest-1), then draw next draw (nextdraw).
         call suml2lnl(nest,thisnest,neh,nel,nep,nec,neb,nen,nepo,neml,
     *      nneml,nfullplus,allint6,numdist,sumintdim,sumdim2,sumdim4,
     *      sumbigintdim,dimii,dimlam,intdist,intdim,intdimc,posi1,
     *      posi2,posi4,bigposi2,ipt,jpt,distdim,distyp,ndraw,thisdraw,
     *      dimused,like,lnl,ddfullbeta,dfullbeta,ddxbh,dxbh,ddxbc,
     *      dxbc,ddsiga6a6,dsiga6a6,ddsigl4l6,dsigl4l6,ddtau,dtau,
     *      ddxblp,dxblp,ddsiga4a4,dsiga4a4,dda5,da5,ddp5,dp5,ddxbml,
     *      dxbml,ddlam,dlam,ddcmui,dcmui,cdm,ddcdm,dcdmdsigii,sigii,
     *      ddsigii,dsigii,ifdrv,iflam,iftau,iid4)
         call nextdraw(task,numnest,thisnest,nest,intdist,intdim,
     *      intdimc,numdist,sumintdim,sumdim2,sumdim4,sumbigintdim,
     *      dimii,nfullplus,maxpnts,nec,allint6,posi1,posi2,posi4,
     *      bigposi1,bigposi2,nested,ne,thisdraw,draw,cdraw,ccdraw,
     *      ndraw,distdim,distpos,distyp,dimused,indx,intpnts,intpntr,
     *      sigii,cdm,dcdmdsigii,fullbeta,z,wt,weight,ui,cmui,lnl,
     *      ifdrv,ipt,jpt,status)
         goto 200
      elseif(task.eq.3)then
c        Assignment of next point required initialization of more
c        inner distributions, resulting in a draw that never occurs.
c        Draw the next draw.
         call nextdraw(task,numnest,thisnest,nest,intdist,intdim,
     *      intdimc,numdist,sumintdim,sumdim2,sumdim4,sumbigintdim,
     *      dimii,nfullplus,maxpnts,nec,allint6,posi1,posi2,posi4,
     *      bigposi1,bigposi2,nested,ne,thisdraw,draw,cdraw,ccdraw,
     *      ndraw,distdim,distpos,distyp,dimused,indx,intpnts,intpntr,
     *      sigii,cdm,dcdmdsigii,fullbeta,z,wt,weight,ui,cmui,lnl,
     *      ifdrv,ipt,jpt,status)
         goto 200
      elseif(task.eq.4)then
c        Log-likelihood at level nest<thisnest has been computed;
c        now weight and accumulate likelihood at level nest.
         call sumlnl2l(nest,thisnest,numdist,sumintdim,sumdim2,
     *      sumbigintdim,neh,nel,nep,nec,neb,nen,nepo,neml,nneml,
     *      nfullplus,dimlam,dimii,allint6,maxpnts,intpnts,intpntr,
     *      nested,lampos,lamtyp,indx,dimused,thisdraw,distyp,distpos,
     *      posi1,posi2,bigposi1,intdist,intdim,intdimc,like,lnl,z,wt,
     *      weight,fullbeta,dfullbeta,ddfullbeta,ui,ddui,cmui,dcmui,
     *      ddcmui,lam,dlam,ddlam,ddepsh,ddepsl,ddepsp,ddepsc,ddepsb,
     *      ddepsn,ddepspo,ddepsml,dxbh,ddxbh,dxbc,ddxbc,cdm,ddcdm,
     *      siga6a6i,dsiga6a6,ddsiga6a6,siga6a6ia6,sigl4l6,dsigl4l6,
     *      ddsigl4l6,sigl4l6a6a6i,dtau,ddtau,dxblp,ddxblp,dsiga4a4,
     *      ddsiga4a4,da5,dda5,dp5,ddp5,dxbml,ddxbml,dsigii,ddsigii,
     *      ifdrv,iflam,iftau,iid4)
         task=2
         goto 100
      elseif(task.eq.5)then
         goto 300
      else
         call bug(ipt,jpt)
      endif
 
  300 lnlobs=lnlobs+lnl(0)
      status=0
      goto 999
 
c     Probit residual's standard deviation was zero or probit thresholds
c     were not ordered or covariance matrix of an integrated distribution
c     was not strictly positive definite.
  800 lnlobs=dpnan()
      goto 999
c     Likelihood underflows to zero; derivatives thus also:
  900 lnlobs=minexp()
      if(ifdrv)call zerodp(nfullplus,dfullbeta)
      status=-1
      goto 999
  999 return
      end
 
