      subroutine corr2cov(task,k,corr,covar,dcovar,dcorr)
 
c     Subroutine to convert correlation matrix with standard
c     deviations on the diagonal into a covariance matrix (task=1)
c     or to convert derivatives wrt the covariance matrix into
c     derivatives wrt standard deviations and correlations (task=2).
c     There is no check that input matrix corr is a legitimate
c     correlation matrix.
c     Derivatives with respect to a covariance matrix are typically
c     symmetric, but this need not be the case.  Symmetry is therefore
c     not exploited.
c     Input and output matrices may share the same storage space.
c     (This is the reason why unraveling is done using covar, rather
c     than the more convenient corr; the latter may actually be a
c     covariance matrix.)
c
c     Constantijn Panis, 1 December 1990
 
      implicit none
      integer task,k,i,j
      double precision corr(k,k),covar(k,k),dcovar(k,k),dcorr(k,k),temp
 
      if(task.eq.2)goto 200
 
      do 110 i=1,k
         do 120 j=i+1,k
            covar(i,j) = corr(i,j)*corr(i,i)*corr(j,j)
            covar(j,i) = covar(i,j)
  120    continue
         covar(i,i) = corr(i,i)*corr(i,i)
  110 continue
      goto 999
 
  200 do 210 i=1,k
         if(covar(i,i).eq.0.d0)then
c           Limit case.  Note that corr(i,j) is not meaningful, and
c           always zero if corr and covar share the same space.  Treat
c           as zero, though this may not entirely be correct.
            dcorr(i,i) = 0.d0
         else
            temp = 0.d0
            do 220 j=1,k
               temp = temp + covar(j,i)*(dcovar(j,i)+dcovar(i,j))
  220       continue
            dcorr(i,i) = temp/dsqrt(covar(i,i))
         endif
  210 continue
      do 230 i=1,k
         do 240 j=1,k
            if(i.ne.j)dcorr(i,j) =
     *         dcovar(i,j)*dsqrt(covar(i,i)*covar(j,j))
  240    continue
  230 continue
 
  999 return
      end
