      subroutine m2mult030(task,nrowa,numblockb,dimblockb,a,bblock,c,
     *   factor)
 
c     Subroutine to multiply matrices A*B = C.  This routine
c     is similar to m2mult, but exploits special matrix properties.
c     Matrix types can be:
c        0 = generic
c        1 = identity (Only its dimension is stored.)
c        2 = symmetric
c        3 = symmetric and block-diagonal (Only its first block, its
c            dimension, and number of blocks are stored.)
c        4 = sparse in the sense that there is at most one nonzero
c            element in any one row.  (Only its nonzero elements
c            and their column position are stored.)
c
c     In this routine, A is generic, B is block-diagonal, D generic.
c     B consists of numblockc blocks of dimension (dimblockb,dimblockb).
c     Only the first block is stored, as bblock(dimblockb,dimblockb).
c
c     Conceptual dimensions:
c            A(nrowa,dimb)   B(dimb,dimb)   C(nrowa,dimb)
c     where dimb=numblockb*dimblockb
c
c     The four matrices may be in transposed format.  Integer task
c     may be 100,010,001, et cetera, depending on which matrices
c     are transposed.  The first digit refers to A; the second to B
c     (irrelevant since B is symmetric); the third to C.  For example,
c        task = 110 means that A and B are transposed: A'*B' = C
c        task =   1 means that C is transposed: A*B = C'
c
c     Dimensions as given above assume no transpositions; if a matrix
c     is transposed, its dimensions really are reversed.
c
c     Integer factor determines:
c        factor = 0     C = A*B
c              else     C = C + factor*A*B
c
c     Constantijn Panis, 3 December 2001
 
      implicit none
      integer task,nrowa,numblockb,dimblockb,factor,dimb,numzero,
     *   i,j,k,ia,ic,ja,jc,sa,sc,jsa,jsc
      double precision a(nrowa*numblockb*dimblockb),
     *   bblock(dimblockb,dimblockb),c(nrowa*numblockb*dimblockb),abij
 
      dimb=numblockb*dimblockb
      if(factor.eq.0)call zerodp(nrowa*dimb,c)
 
      if(mod(task/100,10).eq.0)then
         ja=nrowa
         jsa=1
      else
         ja=1
         jsa=dimb
      endif
c     B is symmetric so second portion of task is irrelevant.
      if(mod(task,10).eq.0)then
         jc=nrowa
         jsc=1
      else
         jc=1
         jsc=dimb
      endif
 
      sa=1
      sc=1
      do 210 i=1,nrowa
         ic=sc
         do 220 j=1,dimb
c           AB(i,j) is the innerproduct of A(i,.) and B(.,j).  First numzero
c           elements of B(.,j) are zero, followed by dimblockb nonzero elements.
            numzero=((j-1)/dimblockb)*dimblockb
            ia=sa+numzero*ja
            abij=0.d0
            do 230 k=1,dimblockb
               abij = abij + a(ia)*bblock(k,j-numzero)
               ia=ia+ja
  230       continue
            if(factor.ne.0.and.factor.ne.1)abij=dble(factor)*abij
            c(ic) = c(ic) + abij
            ic=ic+jc
  220    continue
         sa=sa+jsa
         sc=sc+jsc
  210 continue
 
      return
      end
 
