      subroutine m2mult403(task,numrowa,numrowb,numblockc,dimblockc,
     *   asparse,b,cblock,factor)
 
c     Subroutine to multiply matrices A*B = C.  This routine
c     is similar to m2mult, but exploits special matrix properties.
c     Matrix types can be:
c        0 = generic
c        1 = identity (Only its dimension is stored.)
c        2 = symmetric
c        3 = symmetric and block-diagonal (Only its first block, its
c            dimension, and number of blocks are stored.)
c        4 = sparse in the sense that there is at most one nonzero
c            element in any one row.  (Only its nonzero elements
c            and their column position are stored.)
c
c     In this routine, A is sparse, B generic, C block-diagonal.
c     A is stored as two column vectors.  The first contains the column
c     number, the second the actual element.  In other words, A(i,.)=0
c     except A(i,as(i,1)) = as(i,2).  If as(i,1)<=0, all A(i,.)=0.
c     C consists of numblockc blocks of dimension (dimblockc,dimblockc).
c     Only the first block is stored, as bblock(dimblockc,dimblockc).
c
c     Conceptual dimensions:
c        A(numrowa,numrowb) -- sparse
c        B(numrowb,dimc)    -- generic
c        C(dimc,dimc)       -- symmetric and block-diagonal
c
c     where dimc = numblockc*dimblockc.  It must be the case that
c     numrowa=dimc, otherwise things would not be consistent.
c
c     Note: this routine multiplies A and B and collapses the results into
c     the top-left diagonal block of C.  It does not ensure that C is symmetric.
c     In real-life applications, C will often come out symmetric, but this
c     is not the case with generic A and B.  If symmetry is required,
c     follow this routine with "call makesym(dimblockc,cblock)".
c
c     Integer factor determines:
c        factor = 0     C = A*B
c              else     C = C + factor*A*B
c
c     Constantijn Panis, 4 March 2002
 
      implicit none
      integer task,numrowa,numrowb,numblockc,dimblockc,factor,
     *   i,j,k,ii,jj,jb,jsb
c     A is stored as (numrowa,2)=(dimc,2) if not transposed, (numrowb,2) if tran
      double precision asparse(*),b(numrowb*numrowa),
     *   cblock(dimblockc,dimblockc),abij
      logical atranspose
 
c     Check that numrowa=dimc:
      if(numrowa.ne.numblockc*dimblockc)call bug(1,1)
 
      if(factor.eq.0)call zerodp(dimblockc*dimblockc,cblock)
 
      atranspose=(mod(task/100,10).ne.0)
      if(mod(task/10,10).eq.0)then
         jb=1
         jsb=numrowb
      else
         jb=numrowa
         jsb=1
      endif
c     C is symmetric
 
      do 100 i=1,numrowa
         ii=mod(i-1,dimblockc)+1
         do 200 j=1,numrowa
            if((i-1)/dimblockc.ne.(j-1)/dimblockc)goto 200
            jj=mod(j-1,dimblockc)+1
            abij=0.d0
c           AB(i,j) = A(i,.)*B(.,j)
            if(atranspose)then
c              Sparse A is transposed.  Search for nonzero A(.,i).
c              A(numrowb,numrowa) is stored as (numrowb,2).
               do 300 k=1,numrowb
                  if(int(asparse(k)).eq.i)abij = abij +
     *               asparse(numrowb+k)*b(1+(k-1)*jb+(j-1)*jsb)
  300          continue
            else
c              Sparse A is not transposed.  There is only one nonzero A(i,.).
c              A(numrowa,numrowb) is stored as (numrowa,2).
               k=int(asparse(i))
               if(k.le.0)goto 200
               abij=asparse(numrowa+i)*b(1+(k-1)*jb+(j-1)*jsb)
            endif
            if(factor.ne.0.and.factor.ne.1)abij=dble(factor)*abij
            cblock(ii,jj) = cblock(ii,jj) + abij
  200    continue
  100 continue
 
      return
      end
 
