      subroutine m3mult0340(nrowa,numblockb,dimblockb,nrowc,
     *   a,bblock,csparse,d,factor)
 
c     Subroutine to multiply matrices A*B*C' = D.  This routine
c     is similar to m3mult, but exploits special matrix properties.
c     Matrix types can be:
c        0 = generic
c        1 = identity (Only its dimension is stored.)
c        2 = symmetric
c        3 = symmetric and block-diagonal (Only its first block, its
c            dimension, and number of blocks are stored.)
c        4 = sparse in the sense that there is at most one nonzero
c            element in any one row.  (Only its nonzero elements
c            and their column position are stored.)
c
c     In this routine, A is generic, B block-diagonal, C sparse, and D generic.
c     B consists of numblockc blocks of dimension (dimblockb,dimblockb).
c     Only the first block is stored, as bblock(dimblockb,dimblockb).
c     C is stored as two column vectors.  The first contains the column
c     number, the second the actual element.  In other words, C(i,.)=0
c     except C(i,cs(i,1)) = cs(i,2).  If cs(i,1)<=0, all C(i,.)=0.
c
c     Conceptual dimensions:
c            A(nrowa,dimb)   B(dimb,dimb)   C(nrowc,dimb)   D(nrowa,nrowc)
c     where dimb=numblockb*dimblockb
c
c     The four matrices may be in transposed format.  Integer task
c     may be 1000,0100,0010, et cetera, depending on which matrices
c     are transposed.  The first digit refers to A; the second to B
c     (irrelevant because B is symmetric); the third to C; the fourth to D.
c     For example,
c        task = 1100 means that A and B are transposed: A'*B'*C = D
c        task =   10 means that C is transposed: A*B*C' = D
c
c     Dimensions as given above assume no transpositions; if a matrix
c     is transposed, its dimensions really are reversed.
c
c     Integer factor determines:
c        factor = 0     D = A*B*C
c              else     D = D + factor*A*B*C
c
c     Constantijn Panis, 3 December 2001
 
      implicit none
      integer nrowa,numblockb,dimblockb,nrowc,factor,dimb,numzero,
     *   i,j,k,ia,id
      double precision a(nrowa*numblockb*dimblockb),
     *   bblock(dimblockb,dimblockb),csparse(nrowc,2),d(nrowa*nrowc),
     *   abij
 
      dimb=numblockb*dimblockb
      if(factor.eq.0)call zerodp(nrowa*nrowc,d)
 
      do 110 i=1,nrowa
         do 120 j=1,dimb
            abij = 0.d0
            id=i
c           AB(i,j) is the innerproduct of A(i,.) and B(.,j).  First numzero
c           elements of B(.,j) are zero, followed by dimblockb nonzero elements.
            numzero=((j-1)/dimblockb)*dimblockb
            ia=i+numzero*nrowa
            abij=0.d0
            do 130 k=1,dimblockb
               abij = abij + a(ia)*bblock(k,j-numzero)
               ia=ia+nrowa
  130       continue
            if(factor.ne.0.and.factor.ne.1)abij=dble(factor)*abij
c           AB(i,j) is multiplied by C'(j,k)=C(k,j) to add to D(i,k).
            do 140 k=1,nrowc
               if(int(csparse(k,1)).eq.j)d(id)=d(id) + abij*csparse(k,2)
               id=id+nrowa
  140       continue
  120    continue
  110 continue
 
      return
      end
