      subroutine m3mult0404(task,numrowa,numrowb,numrowc,numcolc,
     *   a,bsparse,c,dsparse,factor)
 
c     Subroutine to multiply three matrices A*B*C = D.  This routine
c     is similar to m3mult, but exploits special matrix properties.
c     Matrix types can be:
c        0 = generic
c        1 = identity (Only its dimension is stored.)
c        2 = symmetric
c        3 = symmetric and block-diagonal (Only its first block, its
c            dimension, and number of blocks are stored.)
c        4 = sparse in the sense that there is at most one nonzero
c            element in any one row.  (Only its nonzero elements
c            and their column position are stored.)
c
c     In this routine, A is generic, B is sparse, C is generic,
c     and D is sparse.
c     B is stored as two column vectors.  The first contains the column
c     number, the second the actual element.  In other words, B(i,.)=0
c     except B(i,bs(i,1)) = bs(i,2).  If bs(i,1)<=0, all B(i,.)=0.
c     D is stored analogously.
c
c     D's sparsity structure must be input to this routine, i.e.,
c     dsparse(.,1) must contain column numbers.
c
c     Conceptual dimensions:
c        A(numrowa,numrowb) -- generic
c        B(numrowb,numrowc) -- sparse
c        C(numrowc,numcolc) -- generic
c        D(numrowa,numcolc) -- sparse
c
c     Only matrix A and C may be in transposed format.
c     Integer task indicates transpositions:
c        task =    0 means that none is transposed: A*B*C = D
c        task =   10 means that C is transposed: A*B*C' = D
c        task = 1000 means that A is transposed: A'*B*C = D
c        task = 1010 means that A and C are transposed: A'*B*C' = D
c
c     Integer factor determines:
c        factor = 0     D = A*B*C'
c              else     D = D + factor*A*B*C'
c
c     Constantijn Panis, 7 March 2002
 
      implicit none
      integer task,numrowa,numrowb,numrowc,numcolc,factor,
     *   rowd,cold,cola,colb,jra,jca,jrc,jcc,posa,posc
      double precision a(numrowa*numrowb),bsparse(numrowb,2),
     *   c(numrowc*numcolc),dsparse(numrowa,2)
 
      if(factor.eq.0)call zerodp(numrowa,dsparse(1,2))
 
      if(mod(task/1000,10).eq.0)then
         jra=1
         jca=numrowa
      else
         jra=numrowb
         jca=1
      endif
c     B is sparse and must not be transposed.
      if(mod(task/100,10).ne.0)call bug(1,1)
      if(mod(task/10,10).eq.0)then
         jrc=1
         jcc=numrowc
      else
         jrc=numcolc
         jcc=1
      endif
c     D is sparse and must not be transposed.
      if(mod(task,10).eq.1)call bug(1,1)
 
      do 100 rowd=1,numrowa
         cold=int(dsparse(rowd,1))
         if(cold.eq.0)goto 100
c        D(rowd,cold) is the only relevant element in D(rowd,.).
c        It is the innerproduct of A(rowd,.) and BC(.,cold).
c        BC(i,cold) is the innerproduct of B(i,.) and C(.,cold).
c        Only one element B(i,.) is nonzero.
         do 200 cola=1,numrowb
            colb = int(bsparse(cola,1))
            if(colb.eq.0)goto 200
            posa=1+(rowd-1)*jra+(cola-1)*jca
            posc=1+(colb-1)*jrc+(cold-1)*jcc
            if(factor.eq.0.or.factor.eq.1)then
               dsparse(rowd,2)=dsparse(rowd,2) +
     *            a(posa)*bsparse(cola,2)*c(posc)
            else
               dsparse(rowd,2)=dsparse(rowd,2) +
     *            dble(factor)*a(posa)*bsparse(cola,2)*c(posc)
            endif
  200    continue
  100 continue
 
      return
      end
 
